@@ -3,7 +3,7 @@
 ;; Author: Shun-ichi TAHARA <jado@flowernet.gr.jp>
 ;;	   Hideyuki SHIRAI <shirai@mew.org>
 ;;
-;; Time-stamp: <09/05/2001 11:35 shirai>
+;; Time-stamp: <03/13/2002 20:45 shirai>
 
 ;;; Commentary:
 ;;
@@ -33,7 +33,8 @@
 ;;    ;;; lazy-lock-mode (for All Emacsen)
 ;;    (require 'font-lock)
 ;;    (setq font-lock-support-mode 'lazy-lock-mode)
-;;    (when (featurep 'xemacs) (setq lazy-lock-minimum-size 0))
+;;    (when (featurep 'xemacs)
+;;      (setq lazy-lock-minimum-size 0))
 ;;
 ;; 4. If you use 'jit-lock-mode' and feel 'scan is too slow'
 ;;    , please put bellow code to your init file.
@@ -181,6 +182,7 @@ return t, otherwise nil."
 ;;; Internal variables
 
 (defvar mew-fancy-summary-special-list nil)
+(defvar mew-fancy-summary-special-alist nil)
 
 (defvar mew-fancy-summary-scan-form nil)
 (make-variable-buffer-local 'mew-fancy-summary-scan-form)
@@ -200,6 +202,7 @@ return t, otherwise nil."
 
 (defalias 'mew-summary-cook-region 'mew-fancy-summary-make-invisible-region)
 (defalias 'mew-highlight-mark-line 'mew-fancy-summary-mark-line)
+(defalias 'mew-highlight-unmark-line 'mew-fancy-summary-mark-line)
 
 ;;; Setup
 
@@ -209,15 +212,15 @@ return t, otherwise nil."
 
 (defun mew-fancy-summary-special-setup ()
   (setq mew-fancy-summary-special-list mew-fancy-summary-special-persons)
-  (if mew-fancy-summary-special-addrbook
-      (let ((addrbook mew-addrbook-alist) nickname)
-	(while addrbook
-	  (setq nickname (nth 2 (car addrbook)))
-	  (if (and (stringp nickname)
+  (when mew-fancy-summary-special-addrbook
+    (let ((addrbook mew-addrbook-alist) nickname)
+      (while addrbook
+	(setq nickname (nth 2 (car addrbook)))
+	(when (and (stringp nickname)
 		   (not (member nickname mew-fancy-summary-special-list)))
-	      (setq mew-fancy-summary-special-list
-		    (cons nickname mew-fancy-summary-special-list)))
-	  (setq addrbook (cdr addrbook))))))
+	  (setq mew-fancy-summary-special-list
+		(cons nickname mew-fancy-summary-special-list)))
+	(setq addrbook (cdr addrbook))))))
 
 ;;; Workarounds about highlighting in Mew
 
@@ -230,33 +233,29 @@ return t, otherwise nil."
 
 (defun mew-fancy-summary-make-invisible-region (beg end &optional interrupt)
   "Faster invisible function for summary with many text properties."
-  (if (and (memq major-mode
-		 '(mew-summary-mode mew-virtual-mode mew-refile-view-mode))
-	   mew-summary-buffer-raw)
-      (let ((inhibit-point-motion-hooks t) ret start)
-	(catch 'loop
-	  (mew-elet
-	   (save-excursion
-	     (goto-char beg)
-	     (while (and (< (point) end)
-			 (search-forward "\r" end t))
-	       (setq start (match-beginning 0))
-	       (unless (get-text-property start 'invisible)
-		 (put-text-property start (progn (end-of-line) (point))
-				    'invisible t))
-	       (when (and interrupt (input-pending-p))
-		 (throw 'loop (setq ret t)))))))
-	(set-buffer-modified-p nil)
-	ret)))
+  (when (and (memq major-mode
+		   '(mew-summary-mode mew-virtual-mode mew-refile-view-mode))
+	     mew-summary-buffer-raw)
+    (let ((inhibit-point-motion-hooks t) ret start)
+      (catch 'loop
+	(mew-elet
+	 (save-excursion
+	   (goto-char beg)
+	   (while (and (< (point) end)
+		       (search-forward "\r" end t))
+	     (setq start (match-beginning 0))
+	     (unless (get-text-property start 'invisible)
+	       (put-text-property start (line-end-position) 'invisible t))
+	     (forward-line)
+	     (when (and interrupt (input-pending-p))
+	       (throw 'loop (setq ret t)))))))
+      (set-buffer-modified-p nil)
+      ret)))
 
-(defun mew-fancy-summary-mark-line (mark)
-  "Remove property so that font-lock works properly, because highlighting
-mark line is done by mew-fancy-summary."
+(defun mew-fancy-summary-mark-line (&rest args)
+  "Highlighting mark and unmark line is done by mew-fancy-summary."
   (save-excursion
-    (remove-text-properties
-     (progn (beginning-of-line) (point))
-     (progn (end-of-line) (point))
-     '(lazy-lock nil lazy-lock-fontified nil fontified nil))))
+    (mew-fancy-summary-region (point) (point) nil 'strip)))
 
 ;;; Activating fancy summary
 
@@ -266,6 +265,9 @@ mark line is done by mew-fancy-summary."
   (unless (mew-thread-p)
     (set (make-local-variable 'font-lock-fontify-buffer-function)
 	 'mew-fancy-summary-buffer)
+    (when (boundp 'font-lock-function)
+      (set (make-local-variable 'font-lock-function)
+	   'mew-fancy-summary-font-lock-function))
     (set (make-local-variable 'font-lock-fontify-region-function)
 	 'mew-fancy-summary-region)
     (setq mew-fancy-summary-scan-form
@@ -273,6 +275,19 @@ mark line is done by mew-fancy-summary."
     (setq mew-fancy-summary-thread-column -1)
     (font-lock-mode 1)))
 
+(defun mew-fancy-summary-font-lock-function (font-lock-mode)
+  (when font-lock-mode
+    (add-hook 'after-change-functions 'font-lock-after-change-function t t)
+    (font-lock-turn-on-thing-lock))
+  (unless font-lock-mode
+    (remove-hook 'after-change-functions 'font-lock-after-change-function t)
+    (font-lock-unfontify-buffer)
+    (font-lock-turn-off-thing-lock)))
+
+(when mew-xemacs-p
+  (defadvice mew-summary-folder-cache-load (after fancy-summary activate)
+    (font-lock-mode 1)))
+
 (defun mew-fancy-summary-thread-enable ()
   "Activate mew-fancy-summary in thread buffer."
   (when (mew-thread-p)
@@ -293,6 +308,9 @@ mark line is done by mew-fancy-summary."
 	   (progn (goto-char end)
 		  (forward-line (/ (frame-height) 2))
 		  (point))))))
+    (when (boundp 'font-lock-function)
+      (set (make-local-variable 'font-lock-function)
+	    'mew-fancy-summary-font-lock-function))
     (set (make-local-variable 'font-lock-fontify-buffer-function)
 	 'mew-fancy-summary-buffer)
     (set (make-local-variable 'font-lock-fontify-region-function)
@@ -310,16 +328,20 @@ mark line is done by mew-fancy-summary."
 	     'mew-face-header-marginal)
        face)))
 
-(defsubst mew-fancy-summary-special-p (key)
+(defsubst mew-fancy-summary-special-p (key len)
   (when (and mew-fancy-summary-special-list (> (length key) 0))
-    (let ((special mew-fancy-summary-special-list) entry)
-      (setq key (regexp-quote key))
-      (catch 'loop
-	(while special
-	  (setq entry (car special))
-	  (and (string-match key entry)
-	       (throw 'loop t))
-	  (setq special (cdr special)))))))
+    (member key (or (cdr (assq len mew-fancy-summary-special-alist))
+		    (let ((special mew-fancy-summary-special-list)
+			  lst str)
+		      (while special
+			(setq str (truncate-string-to-width (car special) len))
+			(when (string-match "^.+\\( +\\)$" str)
+			  (setq str (substring str 0 (match-beginning 1))))
+			(setq lst (cons str lst))
+			(setq special (cdr special)))
+		      (setq mew-fancy-summary-special-alist
+			    (cons (cons len lst) mew-fancy-summary-special-alist))
+		      lst)))))
 
 ;;; Main
 
@@ -331,7 +353,7 @@ mark line is done by mew-fancy-summary."
   (interactive)
   (mew-fancy-summary-region (point-min) (point-max)))
 
-(defun mew-fancy-summary-region (beg end &optional loudly)
+(defun mew-fancy-summary-region (beg end &optional loudly strip)
   "Highlight the region of summary buffer with font-lock-mode."
   (interactive "r")
   (let ((pos (point)) linebeg lineend col numend)
@@ -344,6 +366,8 @@ mark line is done by mew-fancy-summary."
        (setq beg (progn (goto-char beg)
 			(beginning-of-line)
 			(point)))
+       (when strip
+	 (put-text-property beg end 'face nil))
        (mew-fancy-summary-make-invisible-region beg end)
        ;; Highlight each line
        (while (< (point) end)
@@ -358,9 +382,9 @@ mark line is done by mew-fancy-summary."
 	   (let ((form mew-fancy-summary-scan-form) entry)
 	     (while form
 	       (setq entry (car form))
-	       (if (= col mew-fancy-summary-thread-column)
-		   (setq col (+ col
-				(mew-fancy-summary-do-highlight 'thread t))))
+	       (when (= col mew-fancy-summary-thread-column)
+		 (setq col (+ col
+			      (mew-fancy-summary-do-highlight 'thread t))))
 	       (setq col (+ col (mew-fancy-summary-do-highlight entry nil numend)))
 	       (setq form (cdr form))))
 	   (looking-at "[^\r\n]*")
@@ -368,7 +392,7 @@ mark line is done by mew-fancy-summary."
 	  ;; Marked line
 	  ((and (not (mew-in-decode-syntax-p))
 		(looking-at mew-regex-msg-mark))
-	   (let ((mark (string-to-char (mew-match 2))))
+	   (let ((mark (string-to-char (match-string 2))))
 	     (looking-at "[^\r\n]*")
 	     (setq lineend (match-end 0))
 	     (put-text-property linebeg lineend
@@ -481,14 +505,15 @@ mark line is done by mew-fancy-summary."
 		      (eq (string-match mew-scan-form-from-me-prefix entry)
 			  0)))
 	    ;; Mail to me
-	    (if (mew-fancy-summary-special-p entry)
+	    (if (mew-fancy-summary-special-p entry range)
 		(list (cons elen (mew-fancy-summary-get-face 'special t))
 		      (cons range nil))
 	      (list (cons elen face) (cons range nil)))
 	  ;; Mail from me
 	  (setq face (mew-fancy-summary-get-face 'to t))
 	  (if (and prelen mew-fancy-summary-special-to
-		   (mew-fancy-summary-special-p (substring entry prelen)))
+		   (mew-fancy-summary-special-p (substring entry prelen)
+						(- range prelen)))
 	      (list (cons prelen face)
 		    (cons elen (mew-fancy-summary-get-face 'special t))
 		    (cons range nil))
