@@ -7,7 +7,7 @@
 
 (require 'mew)
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
 ;; goodies
 ;;
@@ -24,7 +24,7 @@ To compose a multipart message, you should execute this command first."
       (if (mew-attach-p)
 	  (message "Attachments already exist")
 	(mew-attach-set)
-	(unless (file-exists-p attachdir) (mew-make-directory attachdir))
+	(mew-check-directory attachdir)
 	(unless (file-exists-p cp) (write-region "" nil cp 'append 'no-msg))
 	(unless mew-encode-syntax
 	  (setq mew-encode-syntax (mew-encode-syntax-initial dir)))
@@ -81,7 +81,7 @@ attachments region.
 (defun mew-attach-directory-p ()
   (save-excursion
     (mew-attach-move-onto-file)
-    (looking-at (concat "[^ ]+" mew-path-separator))))
+    (looking-at (file-name-as-directory "[^ ]+"))))
 
 (defun mew-attach-dot-p ()
   (save-excursion
@@ -160,7 +160,7 @@ attachments region.
       (re-search-forward (concat "^....." numreg " ") nil t)
       (mew-attach-move-onto-file))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
 ;; commands
 ;;
@@ -261,7 +261,7 @@ attachments region.
           (if (file-exists-p fullname)
               (if dirp
                   (mew-delete-directory-recursively fullname)
-                (delete-file fullname)))
+                (mew-delete-file fullname)))
 	  (message "Deleting %s...done" ename)
 	  (setq mew-encode-syntax
 		(mew-syntax-remove-entry mew-encode-syntax nums))
@@ -310,7 +310,7 @@ attachments region.
       (while (or (null frompath)
 		 (file-directory-p (file-chase-links frompath)))
 	(setq frompath (mew-draft-input-file-name
-			(concat (mew-capitalize action) " from : ") default)))
+			(concat (mew-capitalize action) " from: ") default)))
       (while (or (null efile)
 		 (file-exists-p (expand-file-name efile attachdir)))
 	(setq tofile (mew-input-string 
@@ -320,8 +320,7 @@ attachments region.
 	(setq efile (concat subdir tofile)))
       (setq FILE (expand-file-name efile attachdir))
       (funcall func frompath FILE)
-      (if (and setmode (/= mew-file-mode (mew-file-get-mode FILE)))
-	  (set-file-modes FILE mew-file-mode))
+      (if setmode (mew-set-file-modes FILE))
       (setq ct (mew-ctdb-ct (mew-ctdb-by-file tofile)))
       (unless ct
 	(setq ct (mew-content-type (mew-tinfo-get-case)))
@@ -345,14 +344,14 @@ attachments region.
   "Link a file with a symbolic link on '.' in attachments.
 FROM must be absolute path. TO must be a file name."
   (interactive)
-  (mew-attach-duplicate (function mew-symbolic-link) "link" nil from to))
+  (mew-attach-duplicate 'mew-symbolic-link "link" nil from to))
 
 (defun mew-attach-copy (&optional from to)
   "Copy a file (via networks) on '.' in attachments.
 FROM must be absolute path. TO must be a file name.
 To copy a remote file, use the \"/[user@]hostname:/filepath\" syntax."
   (interactive)
-  (mew-attach-duplicate (function copy-file) "copy" t from to))
+  (mew-attach-duplicate 'copy-file "copy" t from to))
 
 (defun mew-attach-toggle ()
   "Toggle text/binary in attachments."
@@ -366,7 +365,7 @@ To copy a remote file, use the \"/[user@]hostname:/filepath\" syntax."
 	   (ct (mew-syntax-get-value ctl 'cap))
 	   cte)
       (if (not (or (string= ct mew-ct-txt) (string= ct mew-ct-apo)))
-	  (message "Can't toggle %s" ct)
+	  (message "Cannot toggle %s" ct)
 	(if (string= ct mew-ct-txt)
 	    (setq ct mew-ct-apo)
 	  (setq ct mew-ct-txt))
@@ -386,8 +385,8 @@ on '.' in attachments."
 	default)
     (if fld (setq default (mew-expand-folder fld msg)))
     (if mew-use-symbolic-link-for-forwarding
-	(mew-attach-duplicate (function mew-symbolic-link) "link" nil nil nil default)
-      (mew-attach-duplicate (function copy-file) "copy" nil nil nil default))))
+	(mew-attach-duplicate 'mew-symbolic-link "link" nil nil nil default)
+      (mew-attach-duplicate 'copy-file "copy" nil nil nil default))))
 
 (defun mew-attach-find-new-file ()
   "Open a new file into a buffer on '.' in attachments."
@@ -414,7 +413,7 @@ on '.' in attachments."
 					 nil nil nil nil)))
 	(mew-encode-syntax-print mew-encode-syntax)
 	;;
-	(find-file filepath)
+	(switch-to-buffer (mew-find-file-noselect2 filepath))
 	(text-mode)
 	;; buffer switched
 	(local-set-key "\C-c\C-q" 'mew-kill-buffer)))))
@@ -427,7 +426,6 @@ on '.' in attachments."
     (let* ((nums (mew-syntax-nums))
 	   (subdir (mew-attach-expand-path mew-encode-syntax nums))
 	   (attachdir (mew-attachdir))
-	   (buf (generate-new-buffer mew-buffer-prefix))
 	   file filepath recorded)
       ;; attachdir / {subdir/} dir
       (unless (string= subdir "")
@@ -437,9 +435,7 @@ on '.' in attachments."
       (if (null filepath)
 	  (message "Cannot make a file for audio, sorry")
 	(setq file (file-name-nondirectory filepath))
-	(save-excursion
-	  (set-buffer buf)
-	  (mew-erase-buffer)
+	(with-temp-buffer
 	  (condition-case nil
 	      (progn
 		(if (not (y-or-n-p "Are you ready? "))
@@ -448,7 +444,7 @@ on '.' in attachments."
 		   (substitute-command-keys
 		    "Type '\\[keyboard-quit]' to finish recording..."))
 		  (mew-plet
-		   (apply (function call-process)
+		   (apply 'call-process
 			  (nth 0 mew-prog-audio2)
 			  nil t nil
 			  (nth 1 mew-prog-audio2)))))
@@ -459,7 +455,6 @@ on '.' in attachments."
 	     (setq recorded t)
 	     (mew-flet
 	      (write-region (point-min) (point-max) filepath nil 'no-msg)))))
-	(mew-remove-buffer buf)
 	(if recorded
 	    (setq mew-encode-syntax ;; buffer local
 		  (mew-syntax-insert-entry
@@ -579,7 +574,7 @@ on '.' in attachments."
 	   (file (read-string "Filename: " oldfile)))
       (if (string= file "")
 	  (mew-syntax-set-cdp syntax nil)
-	(mew-syntax-set-cdp syntax (mew-syntax-cdp-format ct file)))
+	(mew-syntax-set-cdp syntax (mew-syntax-cdp-format ct file nil 'force-file)))
       (mew-encode-syntax-print mew-encode-syntax))))
 
 (defun mew-attach-base64 ()
@@ -702,7 +697,7 @@ Input decryptors' addresses."
 	   efile FILE)
       (setq efile (if (string= subdir "") file (concat subdir file)))
       (setq FILE (expand-file-name efile attachdir))
-      (find-file FILE)
+      (switch-to-buffer (mew-find-file-noselect2 FILE))
       ;; buffer switched
       (local-set-key "\C-c\C-q" 'mew-kill-buffer)
       (if (file-symlink-p FILE)
@@ -712,7 +707,7 @@ Input decryptors' addresses."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2002 Mew developing team.
+;; Copyright (C) 1996-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
