@@ -8,8 +8,10 @@
 (require 'mew)
 
 (defun mew-auth-select (grt lst)
-  (let ((auths (mew-split grt 32))
-	auth n preference strongest)
+  (mew-auth-select2 (mew-split grt 32) lst))
+
+(defun mew-auth-select2 (auths lst)
+  (let (auth n preference strongest)
     (while auths
       (setq auth (car auths))
       (setq auths (cdr auths))
@@ -74,7 +76,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2000-2002 Mew developing team.
+;; Copyright (C) 2000-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
