@@ -20,6 +20,7 @@
 ;; Inevitable due to mode-line.
 (defvar mew-summary-buffer-process nil)
 (defvar mew-summary-buffer-process-status nil)
+(defvar mew-summary-buffer-secure-process nil)
 (defvar mew-summary-buffer-left-msgs "-")
 (defvar mew-summary-buffer-raw t)
 
@@ -38,6 +39,7 @@
 (mapcar 'make-variable-buffer-local
 	(list 'mew-summary-buffer-process
 	      'mew-summary-buffer-process-status
+	      'mew-summary-buffer-secure-process
 	      'mew-summary-buffer-left-msgs
 	      'mew-summary-buffer-raw
 	      'mew-decode-syntax
@@ -55,7 +57,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2000-2002 Mew developing team.
+;; Copyright (C) 2000-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
