@@ -12,15 +12,16 @@
 ;;; Cache info
 ;;;
 
-(defvar mew-cinfo-list '("fld" "msg" "time" "size"))
+(defvar mew-cinfo-list '("fld" "msg" "time" "size" "decode-broken"))
 
 (mew-blinfo-defun 'mew-cinfo mew-cinfo-list)
 
-(defun mew-cinfo-set (fld msg time size)
+(defun mew-cinfo-set (fld msg time size decode-broken)
   (mew-cinfo-set-fld fld)
   (mew-cinfo-set-msg msg)
   (mew-cinfo-set-time time)
-  (mew-cinfo-set-size size))
+  (mew-cinfo-set-size size)
+  (mew-cinfo-set-decode-broken decode-broken))
 
 (defun mew-cinfo-equal (fld msg time size)
   (and (string= (mew-cinfo-get-fld) fld)
@@ -28,6 +29,12 @@
        (equal (mew-cinfo-get-time) time)
        (eq (mew-cinfo-get-size) size)))
 
+(defun mew-cache-dinfo-get-decode-broken (buf)
+  (when buf
+    (save-excursion
+      (set-buffer buf)
+      (mew-cinfo-get-decode-broken))))
+
 (defvar mew-xinfo-list
   '("decode-err" "warning" "info" "action" "multi-form" "icon-spec"
     "pri-result" "not-decrypted" "text-body"))
@@ -77,8 +84,8 @@ If MUST-HIT is non-nil and no valid cache entry is found, an error occurs."
 	  nil)
       (setq cache (mew-cache-buffer-get entry))
       (setq file (mew-expand-folder fld msg))
-      (if (not (file-readable-p file))
-	  (error "%s%s does not exist" (file-name-as-directory fld) msg))
+      (unless (file-readable-p file)
+	(error "%s does not exist" (mew-concat-folder fld msg)))
       (setq time (mew-file-get-time file))
       (setq size (mew-file-get-size file))
       (save-excursion
@@ -157,7 +164,7 @@ if decode fails."
 	 (mew-cache-delete)
 	 (setq errormsg (nth 1 errmsg))
 	 (throw 'return (setq cache nil))))
-      (mew-cinfo-set fld msg (car tim-siz) (cdr tim-siz))
+      (mew-cinfo-set fld msg (car tim-siz) (cdr tim-siz) mew-decode-broken)
       (mew-dinfo-set nil t t mew-use-alternative)
       (mew-decode-syntax-clear)
       (mew-xinfo-set-text-body mew-use-text-body)
@@ -198,7 +205,7 @@ if decode fails."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2002 Mew developing team.
+;; Copyright (C) 1997-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
