@@ -7,11 +7,6 @@
 
 (require 'mew)
 
-;; hoping the functions here are free from marker problem
-;; because it inserts any chars before \n. It is important
-;; to use the position's color.
-(defalias 'mew-complete-insert 'insert)
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Low level functions
@@ -37,7 +32,7 @@
     (while (and (< (point-min) (point))	(looking-at mew-lwsp))
       (forward-line -1))
     (if (looking-at "\\([^:]*:\\)")
-	(mew-field-get-func (mew-match 1) switch)
+	(mew-field-get-func (match-string 1) switch)
       nil))) ;; what a case reachs here?
   
 ;;
@@ -88,9 +83,17 @@
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Completion function: C-i
+;;; Completion function for a draft only
 ;;;
 
+(defun mew-draft-set-completion-ignore-case (case)
+  ;; Need to set the global variable "completion-ignore-case",
+  ;; since clicking a candidate on a completion buffer checks
+  ;; the global variable.
+  ;; Yes, this has side-effect.
+  (when (mew-draft-or-header-p)
+    (setq completion-ignore-case case)))
+
 (defun mew-draft-header-comp ()
   "Complete and expand address short names.
 First, a short name is completed. When completed solely or the @ character
@@ -111,71 +114,168 @@ is inserted before the cursor, the short name is expanded to its address."
 	(mew-complete-window-show mew-fields)
       (mew-complete
        word
-       (mapcar (function (lambda (x) (list (concat (mew-capitalize x) " "))))
-	       mew-fields)
+       (mapcar (lambda (x) (list (concat (mew-capitalize x) " "))) mew-fields)
        "field"
        nil))))
 
-(defun mew-complete-folder ()
-  "Folder complete function for Fcc:."
+(defun mew-complete-newsgroups ()
   (interactive)
-  (when (mew-draft-or-header-p)
-    (setq completion-ignore-case mew-complete-folder-ignore-case))
-  (let ((word (mew-delete-backward-char))
-	(completion-ignore-case mew-complete-folder-ignore-case))
+  (let ((word (mew-delete-backward-char)))
     (if (null word)
-	(mew-complete-window-show (list "+" "="))
-      (mew-complete word mew-folder-alist "folder" nil))))
+	(tab-to-tab-stop)
+      (mew-complete
+       word
+       (mew-nntp-folder-alist2 (mew-tinfo-get-case))
+       "newsgroup"
+       nil))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Completion function for both a draft and the minibuffer
+;;;
 
 (defun mew-complete-address ()
   "Complete and expand an address short name.
 First alias key is completed. When completed solely or the @ character
 is inserted before the cursor, the short name is expanded to its address."
   (interactive)
-  (when (mew-draft-or-header-p)
-    (setq completion-ignore-case mew-complete-address-ignore-case))
+  (mew-draft-set-completion-ignore-case mew-complete-address-ignore-case)
   (let ((word (mew-delete-backward-char))
     	(completion-ignore-case mew-complete-address-ignore-case))
     (if (null word)
 	(tab-to-tab-stop)
       (if (string-match "@." word)
-	  (mew-complete-insert
-	   (or (mew-addrbook-alias-next word mew-addrbook-alist) word))
+	  (insert (or (mew-addrbook-alias-next word mew-addrbook-alist) word))
 	(mew-complete
 	 word mew-addrbook-alist "alias" ?@ nil nil
-	 (function mew-addrbook-alias-get) 
-	 (function mew-addrbook-alias-hit))))))
+	 'mew-addrbook-alias-get
+	 'mew-addrbook-alias-hit)))))
 
-(defun mew-complete-newsgroups ()
+(defun mew-complete-local-folder ()
+  "Local folder complete function."
   (interactive)
-  (when (mew-draft-or-header-p)
-    (setq completion-ignore-case mew-complete-address-ignore-case))
-  (let ((word (mew-delete-backward-char))
-    	(completion-ignore-case mew-complete-address-ignore-case))
-    (if (null word)
-	(tab-to-tab-stop)
-      (mew-complete
-       word
-       mew-newsgroups-alist
-       "Newsgroups"
-       nil))))
+  (if mew-input-folder-search-direction
+      (mew-input-folder-search-complete)
+    (mew-draft-set-completion-ignore-case mew-complete-folder-ignore-case)
+    (let ((word (mew-delete-backward-char))
+	  (completion-ignore-case mew-complete-folder-ignore-case))
+      (if (null word)
+	  (mew-complete-window-show (list "+"))
+	(if (and (mew-folder-absolutep word)
+		 (not (mew-draft-or-header-p)))
+	    (mew-complete word (mew-complete-directory-alist word) "directory" nil)
+	  (mew-complete word (mew-local-folder-alist) "folder" nil))))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Completion function for the minibuffer only
+;;;
+
+(defun mew-complete-folder ()
+  "Folder complete function."
+  (interactive)
+  (if mew-input-folder-search-direction
+      (mew-input-folder-search-complete)
+    (mew-complete-folder2)))
+
+(defun mew-input-folder-search-complete ()
+  (let (keys)
+    (save-excursion
+      (set-buffer mew-input-folder-search-buf)
+      (save-excursion
+	(goto-char (point-min))
+	(while (search-forward (or mew-input-folder-search-key "\n") nil t)
+	  (setq keys
+		(cons (buffer-substring (progn (beginning-of-line) (point))
+					(progn (end-of-line) (point)))
+		      keys)))))
+    (mew-complete-window-show (nreverse (delete "" keys)))
+    (mew-highlight-folder-comp-search-window)))
+
+(defun mew-complete-folder2 ()
+  (let ((word (mew-delete-backward-char nil ", \t\n"))
+	(completion-ignore-case mew-complete-folder-ignore-case)
+	case fld case-fld)
+    (cond
+     ((null word)
+      (mew-complete-window-show mew-config-cases2))
+     ((setq case-fld (mew-folder-case word))
+      (setq case (car case-fld))
+      (setq fld (cdr case-fld))
+      (cond
+       ((mew-folder-localp fld)
+	(mew-complete2 fld (mew-local-folder-alist) case))
+       ((mew-folder-popp fld)
+	(mew-complete2 fld (mew-pop-folder-alist) case))
+       ((mew-folder-nntpp fld)
+	(mew-complete2 fld (mew-nntp-folder-alist case) case))
+       ((mew-folder-imapp fld)
+	(mew-complete2 fld (mew-imap-folder-alist case) case))
+       ((mew-folder-virtualp fld)
+	(mew-complete
+	 word (mew-buffer-list "^\\*" t 'mew-virtual-mode) "folder" nil))
+       ((string= fld "")
+	(insert word)
+	(mew-complete-window-show
+	 (mapcar (lambda (x) (concat case ":" x)) mew-folder-prefixes)))
+       (t
+	(insert word)
+	(if (window-minibuffer-p (get-buffer-window (current-buffer)))
+	    (mew-temp-minibuffer-message " [No matching folder]")
+	  (message "No matching folder")))))
+     (t
+      (cond
+       ((mew-folder-localp word)
+	(mew-complete word (mew-local-folder-alist) "folder" nil))
+       ((mew-folder-popp word)
+	(mew-complete word (mew-pop-folder-alist) "folder" nil))
+       ((mew-folder-nntpp word)
+	(mew-complete word (mew-nntp-folder-alist nil) "newsgroup" nil))
+       ((mew-folder-imapp word)
+	(mew-complete word (mew-imap-folder-alist nil) "mailbox" nil))
+       ((mew-folder-virtualp word)
+	(mew-complete
+	 word (mew-buffer-list "^\\*" t 'mew-virtual-mode) "folder" nil))
+       ((mew-folder-absolutep word)
+	(mew-complete word (mew-complete-directory-alist word) "directory" nil))
+       (t
+	(mew-complete
+	 word
+	 (mapcar (lambda (x) (list (concat x ":")))  mew-config-cases)
+	 "case"
+	 nil)))))))
+
+;; called by mew-input-refile-folders.
+;; case is specified by mew-inherit-case.
+(defun mew-complete-imap-folder ()
+  "IMAP folder complete function."
+  (interactive)
+  (if mew-input-folder-search-direction
+      (mew-input-folder-search-complete)
+    (let ((word (mew-delete-backward-char))
+	  (completion-ignore-case mew-complete-folder-ignore-case))
+      (if (null word)
+	  (mew-complete-window-show (list "%"))
+	(mew-complete
+	 word
+	 (mew-imap-folder-alist mew-inherit-case) ;; ie mew-sinfo-get-case
+	 "mailbox"
+	 nil)))))
 
 (defun mew-complete-case ()
   "Complete function for cases."
   (interactive)
-  (when (mew-draft-or-header-p)
-    (setq completion-ignore-case mew-complete-case-ignore-case))
   (let ((word (or (mew-delete-backward-char) ""))
     	(completion-ignore-case mew-complete-case-ignore-case))
     (mew-complete
      word
-     (mapcar (function (lambda (x) (cons x x))) mew-config-cases)
-     "Case"
+     (mapcar 'list mew-config-cases)
+     "case"
      nil)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Circular completion: C-cC-i
+;;; Circular completion function for a draft only
 ;;;
 
 (defun mew-draft-circular-comp ()
@@ -193,21 +293,21 @@ mew-mail-domain-list is inserted. If exists, the next value of
 mew-mail-domain-list concerned with the string between @ and 
 the cursor is inserted."
   (interactive)
-  (when (mew-draft-or-header-p)
-    (setq completion-ignore-case mew-circular-complete-domain-ignore-case))
+  (mew-draft-set-completion-ignore-case
+   mew-circular-complete-domain-ignore-case)
   (let ((word (mew-delete-backward-char "@"))
 	(completion-ignore-case mew-circular-complete-domain-ignore-case))
     (cond
      ((eq word nil) ;; @ does not exist.
       (if (null mew-mail-domain-list)
 	  (message "For domain circular completion, set mew-mail-domain-list")
-	(mew-complete-insert "@")
-	(mew-complete-insert (car mew-mail-domain-list))
+	(insert "@")
+	(insert (car mew-mail-domain-list))
 	(mew-complete-window-delete)))
      ((eq word t) ;; just after @
       (if (null mew-mail-domain-list)
 	  (message "For domain circular completion, set mew-mail-domain-list")
-	(mew-complete-insert (car mew-mail-domain-list))
+	(insert (car mew-mail-domain-list))
 	(mew-complete-window-delete)))
      (t
       ;; cannot use mew-get-next since completion is necessary sometime.
@@ -215,8 +315,7 @@ the cursor is inserted."
        word
        (mew-slide-pair mew-mail-domain-list)
        "domain"
-       t)) ;; use cdr
-     )))
+       t))))) ;; use cdr
 
 (defun mew-circular-complete (msg clist cname &optional here)
   "General circular complete function to call mew-complete."
@@ -224,7 +323,7 @@ the cursor is inserted."
   (let ((str (mew-delete-value here)))
     (if (null str)
 	(if (car clist)
-	    (mew-complete-insert (car clist))
+	    (insert (car clist))
 	  (message "For circular completion, set %s" cname))
       (mew-complete
        str
@@ -239,7 +338,50 @@ the cursor is inserted."
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Expansion : M-C-i
+;;; Circular completion function for the minibuffer only
+;;;
+
+(defun mew-circular-complete-case ()
+  (interactive)
+  (cond
+   ((null mew-input-folder-circular-case)
+    ())
+   ((eq mew-input-complete-function 'mew-complete-local-folder)
+    ())
+   (mew-input-folder-search-direction
+    (mew-input-folder-self-insert))
+   (t
+    (let (cases oldcase newcase insert-:)
+      (save-excursion
+	(if (search-backward "," nil t)
+	    (forward-char 1)
+	  (beginning-of-line))
+	(if (looking-at mew-regex-case2)
+	    (progn
+	      (setq oldcase (mew-match-string 1))
+	      (delete-region (match-beginning 1) (match-end 1)))
+	  (setq oldcase mew-case-default)
+	  (setq insert-: t))
+	(if (setq cases (member oldcase mew-config-cases))
+	    (if (> (length cases) 1)
+		(setq newcase (nth 1 cases))
+	      (setq newcase (car mew-config-cases)))
+	  (setq newcase mew-case-default))
+	(if (string= newcase mew-case-default)
+	    (unless insert-: (delete-char 1))
+	  (insert newcase)
+	  (if insert-: (insert ":"))))
+      (if (or (= (point) (mew-minibuf-point-min))
+	      (save-excursion
+		(forward-char -1)
+		(looking-at "[:,]")))
+	  (if (search-forward "," nil t)
+	      (forward-char -1)
+	    (goto-char (point-max))))))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Expansion for a draft only
 ;;;
 
 (defun mew-draft-expand ()
@@ -260,9 +402,9 @@ the name exists."
 	(message "No address here")
       (setq func (mew-addrbook-func mew-addrbook-for-address-expansion))
       (if (null func)
-	  (mew-complete-insert word)
+	  (insert word)
 	(setq name (funcall func word))
-	(mew-complete-insert (if name (format "%s <%s>" name word) word))))))
+	(insert (if name (format "%s <%s>" name word) word))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -276,7 +418,7 @@ the name exists."
 (defun mew-complete-rfile ()
   "Complete a remote file."
   (interactive)
-  (let* ((path-file (mew-delete-file))
+  (let* ((path-file (mew-delete-file-name))
 	 (path (car path-file))
 	 (file (cdr path-file))
 	 rpath)
@@ -286,8 +428,8 @@ the name exists."
      rpath
      "remote file"
      nil
-     (function mew-ext-file-name-completion)
-     (function mew-ext-file-name-all-completions))))
+     'mew-ext-file-name-completion
+     'mew-ext-file-name-all-completions)))
 
 (defun mew-complete-pick-pattern ()
   "Complete pick patterns."
@@ -295,12 +437,12 @@ the name exists."
   (let* ((pat (mew-delete-pattern))
 	 (clist (append '("(" "!")
 			mew-pick-field-list
-			(mapcar (function car) mew-pick-macro-alist))))
+			(mapcar 'car mew-pick-macro-alist))))
     (if (null pat)
 	(mew-complete-window-show clist)
       (mew-complete
        pat
-       (mapcar (function list) clist)
+       (mapcar 'list clist)
        "pick pattern"
        nil))))
 
@@ -314,12 +456,37 @@ the name exists."
 	  ;; If WORD contains ':', change alist for completion.
 	  (setq field (car (mew-split word ?:)))
 	  (setq alist 
-		(mapcar (function (lambda (str) (list (concat field ":" str))))
-			mew-sort-modes)))
+		(mapcar (lambda (str) (list (concat field ":" str))) mew-sort-modes)))
       ;; Otherwise, alist is mew-sort-key-alist itself.
       (setq alist mew-sort-key-alist))
     (mew-complete word alist "sort key" nil)))
 
+(defun mew-complete-directory-alist (dir)
+  "Return alist of directories for completion."
+  (let ((odir dir) odir1 dirs1 sub dirs2)
+    (setq dir (mew-file-chase-links (expand-file-name dir)))
+    (when (file-directory-p dir)
+      (setq odir1 (file-name-as-directory odir))
+      (setq dirs1 (mapcar
+		   (lambda (x)
+		     (when (file-directory-p (expand-file-name x dir))
+		       (cons (concat odir1 (file-name-as-directory x)) x)))
+		   (directory-files dir nil "[^.]" 'nosort))))
+    (setq sub (file-name-nondirectory dir))
+    (setq odir (file-name-directory odir))
+    (setq dir (file-name-directory dir))
+    (when (and dir odir sub (not (string= sub "")))
+      (setq odir (file-name-as-directory odir))
+      (setq dirs2 (mapcar
+		   (lambda (x)
+		     (when (file-directory-p (expand-file-name x dir))
+		       (cons (concat odir (file-name-as-directory x)) x)))
+		   (directory-files dir nil
+				    (concat "^" (regexp-quote sub))
+				    'nosort))))
+    (sort (delq nil (append dirs2 dirs1))
+	  (lambda (x y) (string< (car x) (car y))))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Hart function for completions
@@ -331,10 +498,10 @@ the name exists."
   (cdr (mew-complete-hit key alist)))
 
 (defun mew-complete (WORD ALIST MSG EXPAND-CHAR &optional TRY ALL GET HIT)
-  (let* ((ftry (or TRY (function try-completion)))
-	 (fall (or ALL (function all-completions)))
-	 (fget (or GET (function mew-complete-get)))
-	 (fhit (or HIT (function mew-complete-hit)))
+  (let* ((ftry (or TRY 'try-completion))
+	 (fall (or ALL 'all-completions))
+	 (fget (or GET 'mew-complete-get))
+	 (fhit (or HIT 'mew-complete-hit))
 	 (cmp (funcall ftry WORD ALIST))
 	 (all (funcall fall WORD ALIST))
 	 (len (length WORD))
@@ -343,40 +510,72 @@ the name exists."
      ;; already completed
      ((eq cmp t)
       (if EXPAND-CHAR ;; may be "t"
-	  (mew-complete-insert (funcall fget WORD ALIST)) ;; use cdr
-	(mew-complete-insert WORD)) ;; use car
+	  (insert (funcall fget WORD ALIST)) ;; use cdr
+	(insert WORD)) ;; use car
       (mew-complete-window-delete))
      ;; EXPAND
      ((and (mew-characterp EXPAND-CHAR)
 	   (char-equal (aref WORD (1- len)) EXPAND-CHAR)
 	   (setq subkey (substring WORD 0 (1- len)))
 	   (funcall fhit subkey ALIST))
-      (mew-complete-insert (funcall fget subkey ALIST)) ;; use cdr
+      (insert (funcall fget subkey ALIST)) ;; use cdr
       (mew-complete-window-delete))
      ;; just one candidate
      ((= 1 (length all))
-      (mew-complete-insert cmp)
+      (insert cmp)
       (if (window-minibuffer-p (get-buffer-window (current-buffer)))
 	  (mew-temp-minibuffer-message " [Sole completion]")
 	(message "Sole completion"))
       (mew-complete-window-delete))
      ;; two or more candidates
      ((stringp cmp) ;; (length all) > 1
-      (mew-complete-insert cmp)
+      (insert cmp)
       (mew-complete-window-show all)
-      (if (and EXPAND-CHAR (funcall fhit cmp ALIST))
+      (if (and (mew-characterp EXPAND-CHAR) (funcall fhit cmp ALIST))
 	  (message
 	   (substitute-command-keys
 	    "To expand '%s', type '%c' then '\\<mew-draft-header-map>\\[mew-draft-header-comp]'.")
 	   cmp EXPAND-CHAR)))
      ;; no candidate
      (t
-      (mew-complete-insert WORD)
+      (insert WORD)
       ;;(mew-complete-window-delete)
       (if (window-minibuffer-p (get-buffer-window (current-buffer)))
 	  (mew-temp-minibuffer-message (format " [No matching %s]" MSG))
 	(message "No matching %s" MSG))))))
 
+(defun mew-complete2-insert (case word)
+  (if case
+      (insert case ":" word)
+    (insert word)))
+
+(defun mew-complete2 (word alist case)
+  (let* ((cmp (try-completion word alist))
+	 (all (all-completions word alist)))
+    (cond
+     ;; already completed
+     ((eq cmp t)
+      (mew-complete2-insert case word) ;; use car
+      (mew-complete-window-delete))
+     ;; just one candidate
+     ((= 1 (length all))
+      (mew-complete2-insert case cmp)
+      (if (window-minibuffer-p (get-buffer-window (current-buffer)))
+	  (mew-temp-minibuffer-message " [Sole completion]")
+	(message "Sole completion"))
+      (mew-complete-window-delete))
+     ;; two or more candidates
+     ((stringp cmp) ;; (length all) > 1
+      (mew-complete2-insert case cmp)
+      (mew-complete-window-show all))
+     ;; no candidate
+     (t
+      (mew-complete2-insert case word)
+      ;;(mew-complete-window-delete)
+      (if (window-minibuffer-p (get-buffer-window (current-buffer)))
+	  (mew-temp-minibuffer-message " [No matching folder]")
+	(message "No matching folder"))))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Minibuf magic
@@ -386,7 +585,7 @@ the name exists."
   (let ((savemax (point-max)))
     (save-excursion
       (goto-char (point-max))
-      (mew-complete-insert m))
+      (insert m))
     (let ((inhibit-quit t))
       (mew-let-user-read)
       (delete-region savemax (point-max))
@@ -396,17 +595,16 @@ the name exists."
 ;; Extracting completion key
 ;;
 
-(defun mew-delete-backward-char (&optional here)
+(defun mew-delete-backward-char (&optional here sep)
   "Delete appropriate preceding word and return it."
   (interactive)
   (let ((case-fold-search t)
         (start nil)
         (end (point))
-        (regex (concat "[^" mew-address-separator "]")))
+        (regex (concat "[^" (or sep mew-address-separator) "]")))
     (save-excursion
       (while (and (not (bobp))
-                  (string-match regex (mew-buffer-substring
-                                       (1- (point)) (point))))
+                  (string-match regex (mew-buffer-substring (1- (point)) (point))))
         (forward-char -1))
       (if (and here (not (re-search-forward (regexp-quote here) end t)))
           nil ;; "here" does not exist.
@@ -417,14 +615,12 @@ the name exists."
                 (mew-buffer-substring start end)
               (delete-region start end)))))))
 
-(defun mew-delete-file ()
+(defun mew-delete-file-name ()
   (if (search-backward mew-path-separator nil t)
       (forward-char 1)
     (beginning-of-line))
   (prog1
-      (cons (mew-buffer-substring (save-excursion (beginning-of-line)
-						  (point))
-				  (point))
+      (cons (mew-buffer-substring (mew-minibuf-point-min) (point))
 	    (mew-buffer-substring (point) (point-max)))
     (delete-region (point) (point-max))))
 
@@ -458,7 +654,7 @@ the name exists."
     (goto-char (match-end 0))
     (if (looking-at "[ \t]")
 	(forward-char 1)
-      (mew-complete-insert " "))
+      (insert " "))
     (if (eolp)
 	nil
       (let ((start (point)) ret)
@@ -493,7 +689,7 @@ the name exists."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2002 Mew developing team.
+;; Copyright (C) 1997-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
