@@ -158,14 +158,14 @@
 (defun mew-smtp-mail-from (&optional case)
   (mew-cfent-value case "smtp-mail-from" mew-smtp-mail-from))
 
-(defun mew-smtp-msgid-domain (&optional case)
-  (or (mew-cfent-value case "smtp-msgid-domain" mew-smtp-msgid-domain)
-      (mew-mail-domain case)))
-
 (defun mew-smtp-msgid-user (&optional case)
   (or (mew-cfent-value case "smtp-msgid-user" mew-smtp-msgid-user)
       (mew-user case)))
 
+(defun mew-smtp-msgid-domain (&optional case)
+  (or (mew-cfent-value case "smtp-msgid-domain" mew-smtp-msgid-domain)
+      (mew-mail-domain case)))
+
 (defun mew-smtp-message-id (&optional case)
   (let* ((random (format "%08d" (mew-random)))
 	 (domain (mew-smtp-msgid-domain case))
@@ -285,6 +285,20 @@
 (defun mew-nntp-header-only (&optional case)
   (mew-cfent-value case "nntp-header-only" mew-nntp-header-only))
 
+(defun mew-nntp-msgid-user (&optional case)
+  (or (mew-cfent-value case "nntp-msgid-user" mew-nntp-msgid-user)
+      (mew-user case)))
+
+(defun mew-nntp-msgid-domain (&optional case)
+  (or (mew-cfent-value case "nntp-msgid-domain" mew-nntp-msgid-domain)
+      (mew-mail-domain case)))
+
+(defun mew-nntp-message-id (&optional case)
+  (let* ((random (format "%08d" (mew-random)))
+	 (domain (mew-nntp-msgid-domain case))
+	 (user (mew-nntp-msgid-user case))
+	 (time (mew-time-ctz-to-msgid (current-time))))
+    (concat "<" time "." random "." user "@" domain ">")))
 ;;
 
 (defun mew-inbox-folder (&optional case)
@@ -470,7 +484,7 @@ fields in the header according to the new value."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2002 Mew developing team.
+;; Copyright (C) 1999-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
