@@ -49,6 +49,7 @@
 (defconst mew-reply-to:     "Reply-To:")
 (defconst mew-organization: "Organization:")
 (defconst mew-newsgroups:   "Newsgroups:")
+(defconst mew-distribution: "Distribution:")
 (defconst mew-followup-to:  "Followup-To:")
 
 (defconst mew-apparently-to: "Apparently-To:")
@@ -63,6 +64,7 @@
 
 (defconst mew-x-mailer:    "X-Mailer:")
 (defconst mew-x-face:      "X-Face:")
+(defconst mew-face:        "Face:")
 (defconst mew-x-mew:       "X-Mew:")
 (defconst mew-x-mew-uidl:  "X-Mew-Uidl:")
 
@@ -81,6 +83,7 @@
 (defconst mew-ct-txt "Text/Plain")
 (defconst mew-ct-htm "Text/Html")
 (defconst mew-ct-xml "Text/Xml")
+(defconst mew-ct-trh "Text/Rfc822-Headers")
 (defconst mew-ct-msg "Message/Rfc822")
 (defconst mew-ct-ext "Message/External-Body")
 (defconst mew-ct-sts "Message/Delivery-Status")
@@ -99,7 +102,10 @@
 (defconst mew-ct-sme "application/x-pkcs7-mime")
 
 (defconst mew-us-ascii "us-ascii")
+(defconst mew-utf-8 "utf-8")
+(defconst mew-iso-2022-jp-2 "iso-2022-jp-2")
 (defconst mew-cs-unknown 'unknown)
+(defconst mew-cs-m17n-list `(,mew-utf-8 ,mew-iso-2022-jp-2))
 
 (defconst mew-type-txt `(,mew-ct-txt ("charset" ,mew-us-ascii)))
 (defconst mew-type-msg `(,mew-ct-msg))
@@ -143,9 +149,10 @@
           (if (string-match "XEmacs" emacs-version) "XEmacs" "Emacs")
           (if (boundp 'emacs-program-version)
               (format " %s " emacs-program-version)
-	    (format " %d.%d " emacs-major-version emacs-minor-version))
+	    (string-match "^\\([.0-9]+\\)\\.[0-9]+$" emacs-version)
+	    (concat " " (match-string 1 emacs-version) " "))
           (and (boundp 'xemacs-codename)
-	       (string-match "^[A-Za-z0-9 ]+$" xemacs-codename)
+	       (string-match "^[a-zA-Z0-9 ]+$" xemacs-codename)
 	       (concat "(" xemacs-codename ")"))
           (and (boundp 'mule-version) (concat "/ Mule " mule-version)))
   "*A value inserted into X-Mailer: field in Draft mode if *non-nil*.")
@@ -179,6 +186,7 @@
 (defconst mew-queue-work-suffix ".wrk")
 (defconst mew-queue-info-suffix ".mqi")
 (defconst mew-draft-info-suffix ".mdi")
+(defconst mew-imapq-info-suffix ".iqi")
 (defconst mew-backup-suffix     ".old")
 (defconst mew-backup-prefix     "#")
 
@@ -188,24 +196,32 @@
 ;;;
 
 (defconst mew-regex-files "^[^.]\\|^.[^.]") ;; xxx "[^.]" is enough?
-(defconst mew-regex-message-files "^[1-9][0-9]*$")
+(defconst mew-regex-message-files  "^[1-9][0-9]*$")
 (defconst mew-regex-message-files2 "^[1-9][0-9]*")
+(defconst mew-regex-message-files3 "^[0-9][0-9]*$")
+(defconst mew-regex-message-files4 "^0[1-9][0-9]*$")
 (defconst mew-regex-mark "\\([^ 0-9]\\)")
 
-(defconst mew-regex-msg      "^ *\\([0-9]+\\)")
-(defconst mew-regex-msg-icon   "^\\([0-9]+\\) ")
-(defconst mew-regex-part "^.....\\([.0-9]+\\) +")
+(defconst mew-regex-msg         "^ *\\([0-9]+\\)")
+(defconst mew-regex-msg-valid   "^ *\\([1-9][0-9]*\\)")
+(defconst mew-regex-msg-invalid "^ *\\(0[1-9][0-9]*\\)")
+
+(defconst mew-regex-msg-icon  "^\\([0-9]+\\) ")
+(defconst mew-regex-part      "^.....\\([.0-9]+\\) +")
 (defconst mew-regex-part-icon "<\\([.0-9]+\\)>")
 
-(defconst mew-regex-virtual "[^\r\n]+\r \\([-+%=/~][^ ]*\\) \\([0-9]+\\)")
-(defconst mew-regex-thread
-  "^ *\\([0-9]+\\)[^\r\n]+\r <\\([^>]*\\)> <\\([^>]*\\)>$")
-(defconst mew-regex-virtual-thread
-  "[^\r\n]+\r\\( \\([-+%=/~][^ ]*\\) \\([0-9]+\\)\\)? <\\([^>]*\\)> <\\([^>]*\\)>$")
+(defconst mew-regex-summary
+  "[^\r\n]+\r <\\([^>]*\\)> <\\([^>]*\\)>")
+(defconst mew-regex-summary2
+  "[^\r\n]+\r <\\([^>]*\\)> <\\([^>]*\\)> \\([^ ]+\\) \\([0-9]*\\)")
+(defconst mew-regex-virtual
+  "[^\006\n]+\006 \\([^ ]*\\) \\([0-9]+\\)$")
 
 ;; [\033\200-\377] does not work due to the multibyte
 (defconst mew-regex-esc-or-nonascii "[^\000-\032\034-\177]") ;; \033 = esc
 (defconst mew-regex-nonascii "[^\000-\177]")
+(defconst mew-regex-singlebyte-nonascii
+  "\\(\\`\\|[\000-\177]\\)[^\000-\177]\\([\000-\177]\\|\\'\\)")
 
 ;; The followings MUST contain "+" for conversion.
 ;; \011 = tab, \012 = lf
@@ -221,6 +237,23 @@
 (defsubst mew-regex-jmp-msg (msg)
   (concat  "^[ ]*" msg "[^0-9]"))
 
+(defconst mew-regex-case "^\\([^-+$%*,:][^,:]+\\):\\(.*\\)")
+(defconst mew-regex-case2 "\\([^-+$%*,:][^,:]+\\):")
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Folder
+;;;
+
+(defconst mew-folder-local   "+")
+(defconst mew-folder-pop     "$")
+(defconst mew-folder-imap    "%")
+(defconst mew-folder-nntp    "-")
+(defconst mew-folder-virtual "*")
+(defconst mew-folder-prefixes
+  `(,mew-folder-local ,mew-folder-pop ,mew-folder-imap
+    ,mew-folder-nntp ,mew-folder-virtual))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Misc
@@ -233,12 +266,26 @@
 
 (defconst mew-draft-coverpage "Cover.txt")
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Inherit
+;;;
+
+(defconst mew-inherit-case nil)
+(defconst mew-inherit-exec-case nil)
+(defconst mew-inherit-alt nil)
+(defconst mew-inherit-7bit nil)
+(defconst mew-inherit-offline nil)
+(defconst mew-inherit-decode-signer nil)
+(defconst mew-inherit-encode-signer nil)
+(defconst mew-inherit-prefetching nil)
+(defconst mew-inherit-ct nil)
 
 (provide 'mew-const)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2000-2002 Mew developing team.
+;; Copyright (C) 2000-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
