@@ -280,11 +280,13 @@ Return the existence MIME-Version: and the value of Subject:."
     ;; for recenter in Message mode
     (put-text-property beg (point) 'mew-visible t)
     (put-text-property beg (point) 'mew-noncontents t)
+    (mew-rear-nonsticky beg (point))
     (when mew-field-other-visible
       (setq beg (point))
       (mapcar 'insert (nreverse others))
       (put-text-property beg (point) 'mew-others t)
-      (put-text-property beg (point) 'mew-noncontents t))
+      (put-text-property beg (point) 'mew-noncontents t)
+      (mew-rear-nonsticky beg (point)))
     ;; the beginning of the content header
     (save-excursion (mapcar 'insert (nreverse contents)))
     ;; 'mew-contents does not work due to PGP/MIME
@@ -789,7 +791,7 @@ Return a part syntax after moving the beginning of the content body."
     ;;
     (if (and func existp (file-exists-p file3))
 	(mew-flet 
-	 (insert-file-contents file3)
+	 (mew-insert-file-contents file3)
 	 (put-text-property (point-min) (point-max) 'mew-noncontents nil)
 	 ;; because of RICH functionality of RFC1847... Gee dirty!
 	 (mew-decode-crlf-magic))
