@@ -22,7 +22,7 @@ Mew -- Messaging in the Emacs World
 
 %s
 
-Copyright (C) 1994-2002 Kazu Yamamoto
+Copyright (C) 1994-2003 Kazu Yamamoto
 
 Please send comments to Kazu@Mew.org.
 
@@ -59,7 +59,6 @@ Please send comments to Kazu@Mew.org.
 	(fill-column (window-width))
 	indent)
     (if (not (integerp time)) (setq time 0))
-    (mew-erase-buffer)
     (insert (format mew-hello-message mew-version))
     (center-region (point-min) (point-max))
     (if (null mew-demo-picture)
@@ -89,22 +88,16 @@ Please send comments to Kazu@Mew.org.
 	(insert-image mew-logo)
 	(goto-char (point-min))
 	(mew-redraw time))
-       ((featurep 'bitmap)
+       ((and (featurep 'bitmap) window-system)
 	(or
 	 (condition-case nil
-	     (insert-file-contents
-	      (expand-file-name mew-icon-mew-mule-bitmap-image
-				mew-icon-directory))
+	     (mew-insert-file-contents
+	      (expand-file-name
+	       mew-icon-mew-mule-bitmap-image mew-icon-directory))
 	   (file-error nil))
 	 (condition-case nil
-	     (let ((buf (generate-new-buffer mew-buffer-prefix)))
-	       (save-excursion
-		 (set-buffer buf)
-		 (mew-erase-buffer)
-		 (insert-file-contents
-		  (expand-file-name mew-icon-mew-mono mew-icon-directory)))
-	       (bitmap-insert-xbm-buffer buf)
-	       (mew-remove-buffer buf))
+	     (bitmap-insert-xbm-file
+	      (expand-file-name mew-icon-mew-mono mew-icon-directory))
 	   (file-error nil)))
 	(center-region (point-min) (point-max))
 	(mew-redraw time))
@@ -138,7 +131,7 @@ Please send comments to Kazu@Mew.org.
   (let ((end (point)))
     (beginning-of-line)
     (delete-region (point) end))
-  (insert (concat prefix string suffix)))
+  (insert prefix string suffix))
 
 (defun mew-demo-loop ()
   (mew-redraw 0.02))
@@ -147,7 +140,7 @@ Please send comments to Kazu@Mew.org.
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2002 Mew developing team.
+;; Copyright (C) 1996-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
