@@ -117,7 +117,7 @@ Execute mew-dinfo-set before calling this."
 	      (narrow-to-region (point) (point))
 	      (mew-frwlet
 	       mew-cs-text-for-read mew-cs-text-for-write
-	       (insert-file-contents (mew-tinfo-get-hdr-file)))
+	       (mew-insert-file-contents (mew-tinfo-get-hdr-file)))
 	      (goto-char (point-min))
 	      (mew-header-goto-end)
 	      (mew-header-set mew-header-separator)
@@ -161,7 +161,7 @@ Execute mew-dinfo-set before calling this."
 	      (narrow-to-region (point) (point))
 	      (mew-frwlet
 	       mew-cs-text-for-read mew-cs-text-for-write
-	       (insert-file-contents (mew-tinfo-get-hdr-file)))
+	       (mew-insert-file-contents (mew-tinfo-get-hdr-file)))
 	      (goto-char (point-min))
 	      (mew-header-goto-end)
 	      (mew-header-set mew-header-separator)
@@ -197,10 +197,7 @@ Execute mew-dinfo-set before calling this."
 you want to save, the entire message or its body. If the target is
 a non-message part, the part is saved (with line delimiter conversion
 if it is a text object). When executed with '\\[universal-argument]', coding-system for
-a text object to be saved can be specified.
-If 'binary' is specified to coding-system, the text object is treated as
-if it were binary. That is, no character set conversion nor line
-delimiter coversion occur."
+a text object to be saved can be specified."
   (interactive "P")
   (mew-summary-msg-or-part
    ;; need to make a cache or a message buffer.
@@ -214,49 +211,46 @@ delimiter coversion occur."
 	  (cache (or cbuf (mew-buffer-message)))
 	  (syntax (mew-syntax-get-entry (mew-cache-decode-syntax cache) nums))
 	  (action "Save")
-	  (buf (generate-new-buffer mew-buffer-prefix))
 	  PLUS1P limit have-hdrp bodyp beg end cdpl file ct ctl
 	  doit append-p)
-     (save-excursion
-       ;; Due to mew-decode-broken, the filename may be changed.
-       ;; So, save it here.
-       (and syntax (setq cdpl (mew-syntax-get-cdp syntax)))
-       (setq ctl (mew-syntax-get-ct syntax))
-       (setq file (mew-syntax-get-filename cdpl ctl))
-       ;; First of all, we should determine which part the user want to
-       ;; save due to the ambiguity.
-       ;; "y" on Message/Rfc822
-       ;;     - msg/txt      the entire msg or its body?
-       ;;     - msg/mul/txt  the entire msg or its part 1?
-       ;; We have to make use of mew-decode-syntax
-       ;; in the cache buffer due to the PGP/MIME dilemma.
-       ;; We need the correct LIMIT.
-       (if (mew-syntax-message-p syntax)
-	   (let ((bodyname "the body") body bct plus1p)
-	     (setq body (mew-syntax-get-part syntax))
-	     (when (mew-syntax-multipart-p body)
-	       (setq plus1p t)
-	       (setq bodyname "the part 1 text")
-	       (setq body (mew-syntax-get-entry body '(1))))
-	     (setq bct (mew-syntax-get-value (mew-syntax-get-ct body) 'cap))
-	     (if (mew-ct-textp bct)
-		 (if (y-or-n-p (format "Save the entire message (y) or %s (n) " bodyname))
-		     (setq have-hdrp t)
-		   (if plus1p
-		       (setq nums (nreverse (cons 1 (nreverse nums))))
-		     (setq bodyp t))
-		   (setq PLUS1P (mew-syntax-get-privacy body)))
-	       (setq have-hdrp t))))
-       ;; Now, let's analyze the message in the burst buffer.
-       ;; This is lengthy, though, avoidable.
-       (setq limit (length nums))
-       (if (or PLUS1P bodyp) ;; VERY important for PGP/MIME
-	   (setq limit (1+ limit)))
-       ;;
-       (set-buffer buf)
-       (mew-erase-buffer)
+     ;; Due to mew-decode-broken, the filename may be changed.
+     ;; So, save it here.
+     (and syntax (setq cdpl (mew-syntax-get-cdp syntax)))
+     (setq ctl (mew-syntax-get-ct syntax))
+     (setq file (mew-syntax-get-filename cdpl ctl))
+     ;; First of all, we should determine which part the user want to
+     ;; save due to the ambiguity.
+     ;; "y" on Message/Rfc822
+     ;;     - msg/txt      the entire msg or its body?
+     ;;     - msg/mul/txt  the entire msg or its part 1?
+     ;; We have to make use of mew-decode-syntax
+     ;; in the cache buffer due to the PGP/MIME dilemma.
+     ;; We need the correct LIMIT.
+     (if (mew-syntax-message-p syntax)
+	 (let ((bodyname "the body") body bct plus1p)
+	   (setq body (mew-syntax-get-part syntax))
+	   (when (mew-syntax-multipart-p body)
+	     (setq plus1p t)
+	     (setq bodyname "the part 1 text")
+	     (setq body (mew-syntax-get-entry body '(1))))
+	   (setq bct (mew-syntax-get-value (mew-syntax-get-ct body) 'cap))
+	   (if (mew-ct-textp bct)
+	       (if (y-or-n-p (format "Save the entire message (y) or %s (n) " bodyname))
+		   (setq have-hdrp t)
+		 (if plus1p
+		     (setq nums (nreverse (cons 1 (nreverse nums))))
+		   (setq bodyp t))
+		 (setq PLUS1P (mew-syntax-get-privacy body)))
+	     (setq have-hdrp t))))
+     ;; Now, let's analyze the message in the burst buffer.
+     ;; This is lengthy, though, avoidable.
+     (setq limit (length nums))
+     (if (or PLUS1P bodyp) ;; VERY important for PGP/MIME
+	 (setq limit (1+ limit)))
+     ;;
+     (with-temp-buffer
        (mew-insert-message fld msg mew-cs-text-for-read nil)
-       (mew-dinfo-set limit 'no-cs-ld-conv t alt)
+       (mew-dinfo-set limit 'no-cs-conv t alt)
        (mew-decode-for-edit)
        ;;
        (setq syntax (mew-syntax-get-entry mew-decode-syntax nums))
@@ -268,6 +262,7 @@ delimiter coversion occur."
        (setq ctl (mew-syntax-get-ct syntax))
        ;;
        (if (and mew-use-samba-encoding
+		file
 		(string-match mew-regex-nonascii file))
 	   (setq file (mew-samba-encoding file)))
        (setq file (mew-summary-input-file-name nil file))
@@ -282,17 +277,20 @@ delimiter coversion occur."
        ;;
        (if (not doit)
 	   (message "Did not save anything")
-	 (let (linebasep fromcs tocs charset)
+	 (let (linebasep fromcs tocs charset writecs)
 	   (save-restriction
 	     (narrow-to-region beg end)
 	     (goto-char (point-min))
 	     (setq ct (mew-syntax-get-value ctl 'cap))
 	     ;; Allow Message/Rfc822. It's user's risk.
 	     (setq linebasep (or (mew-ct-textp ct) (mew-ct-linebasep ct)))
+	     (if linebasep
+		 (setq writecs mew-cs-text-for-write)
+	       (setq writecs mew-cs-binary))
 	     (when (and askcs linebasep mew-mule-p)
 	       (setq tocs (read-coding-system "Coding-system: "))
-	       (if (equal tocs mew-cs-binary)
-		   (setq tocs nil linebasep nil)))
+	       (if (mew-cs-raw-p tocs)
+		   (setq writecs tocs tocs nil)))
 	     (when have-hdrp
 	       (goto-char (point-min))
 	       (mew-header-delete-lines mew-field-delete-common)
@@ -311,11 +309,9 @@ delimiter coversion occur."
 		(point-min) (point-max) (or fromcs mew-cs-autoconv))
 	       (mew-cs-encode-region (point-min) (point-max) tocs))
 	     (mew-frwlet
-	      mew-cs-dummy
-	      (if linebasep mew-cs-text-for-write mew-cs-binary)
+	      mew-cs-dummy writecs
 	      ;; do not specify 'no-msg
-	      (write-region (point-min) (point-max) file append-p))))))
-     (mew-remove-buffer buf))))
+	      (write-region (point-min) (point-max) file append-p)))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -425,11 +421,8 @@ If executed with '\\[universal-argument]', coding-system is asked."
 ;;;
 
 (defun mew-summary-burst-body (fld msg folder &optional num)
-  (save-excursion
-    (let ((buf (generate-new-buffer mew-buffer-prefix))
-	  ret errmsg mstr m)
-      (set-buffer buf)
-      (mew-erase-buffer)
+  (let (ret errmsg mstr m)
+    (with-temp-buffer
       (mew-insert-message fld msg mew-cs-text-for-read nil)
       (mew-dinfo-set 1 nil nil nil) ;; xxx
       (mew-decode-for-edit)
@@ -445,7 +438,6 @@ If executed with '\\[universal-argument]', coding-system is asked."
 	    (message "Nothing was processed")
 	  (mew-touch-folder folder)
 	  (setq ret (list mstr (int-to-string (1- m)))))))
-      (mew-remove-buffer buf)
       (if errmsg (error errmsg))
       ret)))
 
@@ -540,11 +532,8 @@ If called with '\\[universal-argument]', the current part is de-capsulated."
 	 (sep1 (make-string 70 ?-))
 	 (sep2 (make-string 30 ?-))
 	 (folder (mew-input-burst-folder))
-	 (buf (get-buffer-create mew-buffer-prefix))
 	 start mstr m done)
-     (save-excursion
-       (set-buffer buf)
-       (mew-erase-buffer)
+     (with-temp-buffer
        (mew-insert-message fld msg mew-cs-text-for-read nil)
        (mew-set-buffer-multibyte t)
        (if (not (re-search-forward sep1 nil t))
@@ -574,7 +563,6 @@ If called with '\\[universal-argument]', the current part is de-capsulated."
 	 (message "Bursting...done")
 	 (mew-touch-folder folder)
 	 (setq done t)))
-     (mew-remove-buffer buf)
      (if (and done (y-or-n-p (format "Go to %s? " folder)))
 	 (mew-summary-visit-folder folder 'goend)))))
 
@@ -768,13 +756,10 @@ mew-summary-reedit."
 	  (msg (mew-summary-message-number))
 	  (nfld mew-temp-dir)
 	  (nmsg (mew-folder-new-message nfld 'num-only))
-	  (buf (generate-new-buffer mew-buffer-prefix))
 	  nfile)
-     (save-excursion
+     (with-temp-buffer
        ;; First, let's remove garbage and make a valid message
        ;; into a file.
-       (set-buffer buf)
-       (mew-erase-buffer)
        (mew-insert-message fld msg mew-cs-text-for-read nil)
        (mew-set-buffer-multibyte t)
        (goto-char (point-min))
@@ -791,8 +776,7 @@ mew-summary-reedit."
 	 ;; A new message is stored in the temporary directory
 	 (mew-summary-reedit-for-message nfld nmsg nil nil nil)
 	 (run-hooks 'mew-draft-mode-edit-again-hook)
-	 (mew-delete-file nfile)))
-     (mew-remove-buffer buf))))
+	 (mew-delete-file nfile))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -980,7 +964,7 @@ mew-summary-reedit."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2002 Mew developing team.
+;; Copyright (C) 1999-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
