@@ -113,37 +113,13 @@
   (when (mew-header-end)
     (save-excursion
       (save-restriction
-	(goto-char (mew-header-end))
-	(if (not (bolp)) (insert "\n"))
 	(narrow-to-region (point-min) (mew-header-end))
-	(let (beg med str)
-	  (mew-elet
-	   ;; removing null lines
-	   (goto-char (point-min))
-	   (while (and (re-search-forward "^$" nil t)
-		       (not (eobp)))
-	     (delete-char 1))
-	   ;; removing fields which do not have value.
-	   (goto-char (point-min))
-	   (while (not (eobp))
-	     (if (not (looking-at mew-keyval))
-		 (forward-line)
-	       (setq beg (match-beginning 0))
-	       (setq med (match-end 0))
-	       (forward-line)
-	       (mew-header-goto-next)
-	       (setq str (buffer-substring med (1- (point))))
-	       ;; str may consists of multiple lines
-	       ;; So, "$" does not work. We need to use "[^ ]".
-	       (unless (string-match "[^ \t\n]" str)
-		 (delete-region beg (point)))))))
 	(let ((fields mew-multiple-field-error-list))
 	  (while fields
 	    (if (> (mew-header-count-field (car fields)) 1)
 		(mew-encode-error (format "Multiple %s!" (car fields))))
 	    (setq fields (cdr fields))))
-	(mapcar (function mew-header-combine-field)
-		mew-multiple-field-combine-list)))))
+	(mapcar 'mew-header-combine-field mew-multiple-field-combine-list)))))
 
 (defun mew-encode-ask-subject ()
   (when (and mew-ask-subject (not (mew-header-existp mew-subj:)))
@@ -161,19 +137,13 @@
 	(setq folder (mew-canonicalize-folder (car folders)))
 	(setq folders (cdr folders))
 	(when (and mew-ask-fcc
-		   (not (file-directory-p (mew-expand-folder folder)))
-		   (not (y-or-n-p (format "%s does not exist. Create it? " folder))))
-	  (error "Edit Fcc:"))
+		   (not (file-directory-p (mew-expand-folder folder))))
+	  (if (y-or-n-p (format "%s does not exist. Create it? " folder))
+	      (mew-local-folder-check folder)
+	    (mew-encode-error "Edit Fcc:")))
 	(setq ret (cons folder ret))))
     (nreverse ret)))
 
-;; xxx NetNews: ""?
-(defun mew-encode-ask-newsgroups ()
-  (when (and mew-ask-newsgroups (mew-header-existp mew-newsgroups:))
-    (unless (y-or-n-p "Do you want to post to NetNews? ")
-      (save-excursion
-	(mew-header-delete-lines (list mew-newsgroups:))))))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; To:, Cc:
@@ -189,7 +159,7 @@
 	(narrow-to-region (point-min) (mew-header-end))
 	(goto-char (point-min))
 	(while (re-search-forward regex nil t)
-	  (setq key (mew-match 1))
+	  (setq key (match-string 1))
 	  (setq start (match-end 0))
 	  (forward-line)
 	  (while (looking-at mew-lwsp)
@@ -199,8 +169,7 @@
 	  (delete-region start (1- (point)))
 	  (backward-char 1)
 	  ;;
-	  (setq vals (mapcar (function mew-chop)
-			     (mew-split-quoted val ?, ?: ?\;)))
+	  (setq vals (mapcar 'mew-chop (mew-split-quoted val ?, ?: ?\;)))
 	  (while vals
 	    (setq val (car vals))
 	    (setq vals (cdr vals))
@@ -209,27 +178,27 @@
 	     ((string-match "[^:]+:[^;]+;" val)
 	      (let ((mew-alias-expand-prefix nil))
 		(setq addrs (mew-alias-expand val mew-addrbook-alist 0))
-		(setq addrs (mapcar (function mew-addrstr-append-domain) addrs))
+		(setq addrs (mapcar 'mew-addrstr-append-domain addrs))
 		(setq ins (list (concat mew-alias-expand-prefix ":;")))))
 	     ((and (setq addr (mew-addrstr-parse-address val))
 		   (string-match (concat "\\(.*<\\)" (regexp-quote addr) "\\(>.*\\)")
 				 val))
-	      (setq prefix (mew-match 1 val))
-	      (setq suffix (mew-match 2 val))
+	      (setq prefix (match-string 1 val))
+	      (setq suffix (match-string 2 val))
 	      (setq addr (mew-addrstr-append-domain addr))
 	      (setq addrs (list addr))
 	      (setq ins (list (concat prefix addr suffix))))
 	     (t
 	      (let ((mew-alias-expand-prefix nil))
 		(setq addrs (mew-alias-expand val mew-addrbook-alist 0))
-		(setq addrs (mapcar (function mew-addrstr-append-domain) addrs))
+		(setq addrs (mapcar 'mew-addrstr-append-domain addrs))
 		(if mew-alias-expand-prefix
 		    (setq ins (list (concat mew-alias-expand-prefix ":;")))
 		  (setq ins (copy-sequence addrs))))))
 	    (setq insl (nconc insl ins))
 	    (unless (string-match "bcc" key) ;; Removing Bcc:
 	      (setq ret (nconc ret addrs))))
-	  (insert " " (mapconcat (function identity) insl ", "))
+	  (insert " " (mapconcat 'identity insl ", "))
 	  (setq insl nil)
 	  (forward-line))))
     ret))
@@ -259,11 +228,8 @@
 ;;; Message-Id:, Date:
 ;;;
 
-(defun mew-encode-id-date (pnm case resentp)
-  (let ((msgid (mew-smtp-message-id case))
-	(time (current-time)))
-    (mew-smtp-set-msgid pnm msgid)
-    (mew-smtp-set-logtime pnm (mew-time-ctz-to-logtime time))
+(defun mew-encode-id-date (pnm msgid &optional resentp)
+  (let ((time (current-time)))
     (cond
      (resentp
       (mew-header-delete-lines (list mew-resent-date: mew-resent-message-id:))
@@ -276,45 +242,28 @@
       (save-excursion
 	(goto-char (point-min))
 	(mew-header-insert mew-date: (mew-time-ctz-to-rfc time))
-	(mew-header-insert mew-message-id: msgid))))))
+	(mew-header-insert mew-message-id: msgid))))
+    (list msgid (mew-time-ctz-to-logtime time))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; From:, Sender:
 ;;;
 
-(defun mew-encode-from-sender (pnm case resentp)
+(defun mew-encode-check-sender (resentp)
   (if (< 1 (length (mew-addrstr-parse-address-list
 		    (mew-header-get-value
 		     (if resentp mew-resent-sender: mew-sender:)))))
-      (mew-encode-error "Sender: must contain one address!"))
-  (let* ((mail-from (mew-addrstr-parse-address (mew-smtp-mail-from case)))
-	 (from (mew-addrstr-parse-address-list
-		(mew-header-get-value
-		 (if resentp mew-resent-from: mew-from:))))
-	 (nfrom (length from))
-	 insert-from)
-    (cond
-     ((= nfrom 0)
-      (setq insert-from (mew-from case))
-      (setq from (mew-addrstr-parse-address insert-from))
-      (if (null mail-from) (setq mail-from from)))
-     ((= nfrom 1)
-      (setq from (car from))
-      (if (null mail-from) (setq mail-from from)))
-     (t
-      (if (null mail-from)
-	  ;; Which address is suitable form MAIL FROM?
-	  (setq mail-from (mew-addrstr-parse-address (mew-from case))))))
-    ;; MUST be an address only (excluding phrase)
-    (mew-smtp-set-from pnm mail-from)
-    ;;
-    (when insert-from
+      (mew-encode-error "Sender: must contain one address!")))
+
+(defun mew-encode-from (case resentp)
+  (unless (mew-header-existp mew-from:)
+    (let ((from (mew-from case)))
       (save-excursion
 	(goto-char (point-min))
 	(if resentp
-	    (mew-header-insert mew-resent-from: insert-from)
-	  (mew-header-insert mew-from: insert-from))))))
+	    (mew-header-insert mew-resent-from: from)
+	  (mew-header-insert mew-from: from))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -326,7 +275,7 @@
   (if (and mew-use-auto-alias mew-addrbook-append-domain-p)
       ;; If mew-addrbook-append-domain-p is nil, automatic
       ;; short names would be conflicted to local users.
-      (mapcar (function mew-addrbook-alias-add)
+      (mapcar 'mew-addrbook-alias-add
 	      (mew-header-parse-address-list
 	       (if resentp
 		   (list mew-resent-to: mew-resent-cc:)
@@ -358,11 +307,11 @@
       (mew-syntax-set-decrypters
        mew-encode-syntax (mew-smtp-get-recipients pnm)))))
 
-(defun mew-encode-make-header (&optional addsep)
+(defun mew-encode-make-header (&optional addsep resentp)
   (unless (mew-header-existp mew-mv:)
     (goto-char (mew-header-end))
     (mew-header-insert mew-mv: mew-mv:-num))
-  (mew-header-encode-region (point-min) (mew-header-end))
+  (mew-header-encode-region (point-min) (mew-header-end) resentp)
   (cond
    (addsep ;; reedit
     ;; To:
@@ -388,9 +337,7 @@
    mew-cs-dummy mew-cs-text-for-write
    (write-region (point-min) (point-max) (buffer-file-name) nil 'no-msg)
    (clear-visited-file-modtime)
-   (if (and buffer-auto-save-file-name
-	    (file-exists-p buffer-auto-save-file-name))
-       (delete-file buffer-auto-save-file-name))
+   (mew-delete-file buffer-auto-save-file-name)
    (set-buffer-modified-p nil)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -398,29 +345,28 @@
 ;;; Encoding a message
 ;;;
 
-(defun mew-encode (pnm case resentp &optional privacy signer headerp)
+(defun mew-smtp-encode (pnm case resentp fcc &optional privacy signer headerp)
   (let ((ret t))
     (if (mew-debug 'encode)
 	(let ((debug-on-error t))
-	  (mew-encode-message pnm case resentp privacy signer headerp))
+	  (mew-smtp-encode-message pnm case resentp fcc privacy signer headerp))
       (condition-case nil
-	  (mew-encode-message pnm case resentp privacy signer headerp)
+	  (mew-smtp-encode-message pnm case resentp fcc privacy signer headerp)
 	(error
 	 (mew-encode-undo)
 	 (message (mew-tinfo-get-encode-err))
 	 (setq ret nil))))
     ret))
 
-(defun mew-encode-message (pnm case resentp &optional privacy signer headerp)
+(defun mew-smtp-encode-message (pnm case resentp fcc &optional privacy signer headerp)
   (mew-set-buffer-multibyte t)
   (if (buffer-modified-p) (save-buffer)) ;; to make backup
   (widen)
-  (run-hooks 'mew-make-message-hook)
-  (let (multip recipients)
-    (mew-smtp-set-raw-header
-     pnm (mew-buffer-substring (point-min) (mew-header-end)))
+  (let (multip recipients msgid-logtime)
     (goto-char (point-min))
     (mew-encode-remove-illegal-fields)
+    (mew-smtp-set-raw-header
+     pnm (mew-buffer-substring (point-min) (mew-header-end)))
     ;; Destination check
     (setq recipients (mew-encode-canonicalize-address resentp))
     ;; Bcc: is not included.
@@ -430,16 +376,8 @@
       ;; Let's backup
       (if (mew-attach-p)
 	  (progn
-	    (mew-attach-clear)
-	    (cond
-	     ((mew-encode-syntax-have-multiple-parts)
-	      (setq multip t))
-	     ((mew-encode-syntax-have-one-part)
-	      ;; A user may want to do any MIME stuff to the body
-	      (setq mew-encode-syntax (mew-syntax-get-part mew-encode-syntax)))
-	     (t
-	      ;; xxx error but ...
-	      )))
+	    (setq multip t)
+	    (mew-attach-clear))
 	(setq mew-encode-syntax
 	      (mew-encode-syntax-single "text.txt" (list mew-ct-txt))))
       (mew-encode-make-backup))
@@ -449,15 +387,14 @@
     (mew-header-delete-lines (list mew-fcc: mew-resent-fcc:)) ;; anyway
     (mew-smtp-set-dcc pnm (mew-encode-delete-dcc resentp))
     (mew-smtp-set-bcc pnm (mew-encode-delete-bcc resentp))
-    (mew-encode-id-date pnm case resentp)
-    (mew-encode-from-sender pnm case resentp)
-    (mew-encode-learn-aliases resentp)
+    (mew-smtp-set-fcc pnm fcc)
     ;;
-    (mew-smtp-set-user pnm (mew-smtp-user case))
-    (mew-smtp-set-auth pnm (mew-smtp-auth case))
-    (mew-smtp-set-auth-list pnm (mew-smtp-auth-list case))
-    (mew-smtp-set-helo-domain pnm (mew-smtp-helo-domain case))
-    (mew-smtp-set-queue pnm (mew-queue-folder case))
+    (mew-encode-check-sender resentp)
+    (mew-encode-from case resentp)
+    ;;
+    (setq msgid-logtime (mew-encode-id-date pnm (mew-smtp-message-id case) resentp))
+    (mew-smtp-set-msgid pnm (nth 0 msgid-logtime))
+    (mew-smtp-set-logtime pnm (nth 1 msgid-logtime))
     ;;
     (goto-char (mew-header-end))
     (forward-line) ;; necessary for PGP
@@ -467,10 +404,78 @@
     (unless headerp
       (mew-encode-set-privacy pnm privacy)
       (let ((mew-inherit-encode-signer (or signer (mew-get-my-address))))
+	(goto-char (mew-header-end)) ;; due to illegal null lines in the header
+	(forward-line)
 	(if multip
 	    (mew-encode-make-multi)
 	  (mew-encode-make-single))))
-    (mew-encode-make-header headerp)
+    (mew-encode-make-header headerp resentp)
+    ;; Learn aliases after no error occurred
+    (mew-encode-learn-aliases resentp)
+    (mew-encode-save-draft)
+    (mew-overlay-delete-buffer)
+    (message "Making a message...done")))
+
+(defun mew-nntp2-encode (pnm case fcc &optional privacy signer headerp)
+  (let ((ret t))
+    (if (mew-debug 'encode)
+	(let ((debug-on-error t))
+	  (mew-nntp2-encode-message pnm case fcc privacy signer headerp))
+      (condition-case nil
+	  (mew-nntp2-encode-message pnm case fcc privacy signer headerp)
+	(error
+	 (mew-encode-undo)
+	 (message (mew-tinfo-get-encode-err))
+	 (setq ret nil))))
+    ret))
+
+(defun mew-nntp2-encode-message (pnm case fcc &optional privacy signer headerp)
+  (mew-set-buffer-multibyte t)
+  (if (buffer-modified-p) (save-buffer)) ;; to make backup
+  (widen)
+  (let (multip newsgroups msgid-logtime)
+    (goto-char (point-min))
+    (mew-encode-remove-illegal-fields)
+    (mew-nntp2-set-raw-header
+     pnm (mew-buffer-substring (point-min) (mew-header-end)))
+    ;;
+    (setq newsgroups (mew-header-get-value mew-newsgroups:))
+    (mew-nntp2-set-newsgroups pnm newsgroups)
+    ;; Let's backup
+    (unless headerp
+      (if (mew-attach-p)
+	  (progn
+	    (setq multip t)
+	    (mew-attach-clear))
+	(setq mew-encode-syntax
+	      (mew-encode-syntax-single "text.txt" (list mew-ct-txt))))
+      (mew-encode-make-backup))
+    ;; Errors can be caused from here.
+    (unless newsgroups (mew-encode-error "No newsgroups!"))
+    ;; Header modifications which are not remained.
+    (mew-header-delete-lines (list mew-fcc: mew-resent-fcc:)) ;; anyway
+    (mew-nntp2-set-fcc pnm fcc)
+    ;;
+    (mew-encode-from case nil)
+    ;;
+    (setq msgid-logtime (mew-encode-id-date pnm (mew-nntp-message-id case)))
+    (mew-nntp2-set-msgid pnm (nth 0 msgid-logtime))
+    (mew-nntp2-set-logtime pnm (nth 1 msgid-logtime))
+    ;;
+    (goto-char (mew-header-end))
+    (forward-line) ;; necessary for PGP
+    ;;
+    (message "Making a message...")
+    ;; save syntax before setting privacy
+    (unless headerp
+      (mew-encode-set-privacy pnm privacy)
+      (let ((mew-inherit-encode-signer (or signer (mew-get-my-address))))
+	(goto-char (mew-header-end)) ;; due to illegal null lines in the header
+	(forward-line)
+	(if multip
+	    (mew-encode-make-multi)
+	  (mew-encode-make-single))))
+    (mew-encode-make-header nil)
     (mew-encode-save-draft)
     (mew-overlay-delete-buffer)
     (message "Making a message...done")))
@@ -481,9 +486,8 @@
 ;;;
 
 (defun mew-encode-make-single ()
-  (goto-char (mew-header-end)) ;; due to illegal null lines in the header
-  (forward-line)
-  (mew-encode-singlepart mew-encode-syntax nil nil t))
+  ;; Just after the header
+  (mew-encode-singlepart mew-encode-syntax nil nil t 'cover))
 
 (defun mew-encode-charset-8bitp (charset)
   (let ((case-fold-search t))
@@ -531,7 +535,7 @@
 		     (symbol-name cs)))))
 	(mew-frwlet
 	 (or cs mew-cs-autoconv) mew-cs-dummy
-	 (insert-file-contents file)))
+	 (mew-insert-file-contents file)))
       ;; A user may specify charset to convery JIS X 0201 Katakana.
       ;; So, we need to avoid the sanity check.
       (unless charset
@@ -579,7 +583,7 @@
       (unless textp
 	(mew-frwlet
 	 mew-cs-text-for-read mew-cs-dummy
-	 (insert-file-contents file)))
+	 (mew-insert-file-contents file)))
       (when (and (string= ct mew-ct-msg)
 		 (or mew-inherit-7bit (not mew-use-8bit))
 		 (mew-case-equal cte mew-8bit))
@@ -592,11 +596,10 @@
 	(mew-frwlet
 	 (if linebasep mew-cs-text-for-read mew-cs-binary)
 	 mew-cs-dummy
-	 (let (format-alist)
-	   (insert-file-contents file))))
+	 (mew-insert-file-contents file)))
       (when linebasep
 	(goto-char beg)
-	(while (search-forward "\n" nil t) (replace-match "\r\n" nil t)))
+	(mew-lf-to-crlf))
       (base64-encode-region beg (point-max))
       (goto-char (point-max))
       (insert "\n"))
@@ -613,17 +616,17 @@
 	  (delete-region beg (point-max)))
 	(mew-piolet
 	 mew-cs-text-for-read mew-cs-dummy
-	 (apply (function call-process) mew-prog-mime-encode
-		file t nil opt))
-	(if (and textp (file-exists-p file))
-	    (delete-file file))))
+	 (apply 'call-process mew-prog-mime-encode file t nil opt))
+	(if textp (mew-delete-file file))))
      (t
       (mew-encode-error (concat mew-prog-mime-encode " does not exist"))))
     (list (if charset (list "charset" charset)) cte)))
 
-(defun mew-encode-singlepart (syntax &optional path depth buffered)
+(defun mew-encode-singlepart (syntax &optional path depth buffered coverp)
   ;; path is nil if called make-single or security multipart
   ;; buffered is t if called make-single
+  (run-hook-with-args 'mew-encode-singlepart-hook
+                      syntax path depth buffered)
   (let* ((file (expand-file-name (mew-syntax-get-file syntax) path))
 	 (ctl (mew-syntax-get-ct syntax))
          (ct (mew-syntax-get-value ctl 'cap))
@@ -633,7 +636,7 @@
 	 (privacy (mew-syntax-get-privacy syntax))
 	 (mew-inherit-7bit (mew-encode-limit-7bitp privacy))
 	 (beg (point))
-	 charset-cte charset bodybeg)
+	 charset-cte charset bodybeg cst ask-cst)
     (setq charset-cte (mew-encode-mime-body ctl cte (or buffered file)))
     (goto-char beg)
     (setq charset (nth 0 charset-cte))
@@ -660,6 +663,20 @@
       (mew-encode-security-multipart
        beg privacy depth (mew-syntax-get-decrypters syntax)))
     (goto-char (point-max))
+    (when (and coverp (setq cst (nth 1 charset)))
+      (cond
+       ((null mew-ask-charset)
+	;; not ask
+	)
+       ((eq mew-ask-charset t)
+	(if (mew-member-case-equal cst mew-cs-m17n-list)
+	    (setq ask-cst t)))
+       ((listp mew-ask-charset)
+	(unless (mew-member-case-equal cst mew-ask-charset)
+	  (setq ask-cst t))))
+      (if (and ask-cst
+	       (not (y-or-n-p (format "%s is used. OK? " cst))))
+	  (mew-encode-error "Modify body")))
     (mew-case-equal cte mew-8bit)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -668,39 +685,46 @@
 ;;;
 
 (defun mew-encode-make-multi ()
-  ;; delete delimiter
-  (goto-char (mew-header-end)) ;; due to illegal null lines in the header
-  (forward-line)
-  (let ((beg (point))
-	(path (mew-expand-folder mew-attach-folder))
-	(multip t)
-	buffered)
-    ;; Just after the header
-    ;; See if a cover page is empty or not
-    (save-excursion
-      (while (and (looking-at "^$") (not (eobp)))
-	(forward-line))
-      (if (not (eobp))
+  ;; Just after the header
+  (let (beg path multip buffered privacy first ct)
+    (if (mew-encode-syntax-have-one-part)
+	;; A user may want to do any MIME stuff to the body
+	(progn
+	  (setq mew-encode-syntax (mew-syntax-get-part mew-encode-syntax))
+	  (setq buffered t))
+      ;; See if a cover page is empty or not
+      (setq beg (point))
+      (save-excursion
+	(while (and (looking-at "^$") (not (eobp)))
+	  (forward-line))
+	(unless (eobp)
 	  ;; The cover page exists.
-	  (setq buffered t)
+	  (setq buffered t)))
+      (setq path (mew-expand-folder mew-attach-folder))
+      (if buffered
+	  ;; The cover page exists.
+	  (setq multip t)
 	;; The cover page does not exist.
 	(delete-region beg (point-max))
 	;; Remove the cover page entry from the syntax.
 	(setq mew-encode-syntax
 	      (mew-syntax-remove-entry mew-encode-syntax '(1)))
-	(when (and (mew-encode-syntax-have-one-part)
-		   (mew-ct-textp 
-		    (mew-syntax-get-value
-		     (mew-syntax-get-ct
-		      (mew-syntax-get-part mew-encode-syntax)))))
-	  (setq path (expand-file-name
-		      (mew-syntax-get-file mew-encode-syntax) path))
-	  (setq mew-encode-syntax (mew-syntax-get-part mew-encode-syntax))
-	  (setq multip nil))))
+	;; After removing the over page, see if this message has
+	;; only one text part.
+	(if (not (mew-encode-syntax-have-one-part))
+	    (setq multip t)
+	  (setq first (mew-syntax-get-part mew-encode-syntax))
+	  (setq ct (mew-syntax-get-value (mew-syntax-get-ct first) 'cap))
+	  (if (not (mew-ct-textp ct))
+	      (setq multip t)		   
+	    (setq path (expand-file-name
+			(mew-syntax-get-file mew-encode-syntax) path))
+	    (setq privacy (mew-syntax-get-privacy mew-encode-syntax))
+	    (setq mew-encode-syntax (mew-syntax-get-part mew-encode-syntax))
+	    (mew-syntax-set-privacy mew-encode-syntax privacy)))))
     (if multip
 	(mew-encode-multipart mew-encode-syntax path 0 buffered)
-      (goto-char beg)
-      (mew-encode-singlepart mew-encode-syntax path nil nil))))
+      (mew-encode-singlepart mew-encode-syntax path nil nil 'cover))))
 
 (defvar mew-default-boundary "--%s(%s_%s)--")
 
@@ -726,21 +750,29 @@
 	 (beg (point))
 	 (cnt mew-syntax-magic)
 	 (8bit-cnt 0)
-	 8bitp cte-pos)
+	 8bitp cte-pos cover)
     (mew-header-insert mew-ct: (list ct (list "boundary" boundary)))
     (setq cte-pos (point))
     (and cd (mew-header-insert mew-cd: cd))
     (while (< cnt len)
-      (insert (concat "\n--" boundary "\n"))
+      (insert "\n--" boundary "\n")
       (if (mew-syntax-multipart-p (aref syntax cnt))
 	  (setq 8bitp (mew-encode-multipart 
-		       (aref syntax cnt) fullname (1+ depth)))
+		       (aref syntax cnt)
+		       fullname
+		       (1+ depth)))
+	(if (and (= depth 0) (= cnt mew-syntax-magic ))
+	    (setq cover t)
+	  (setq cover nil))
 	(setq 8bitp (mew-encode-singlepart
-		     (aref syntax cnt) fullname (1+ depth)
-		     (if (eq cnt mew-syntax-magic) buffered nil))))
+		     (aref syntax cnt)
+		     fullname
+		     (1+ depth)
+		     (if (eq cnt mew-syntax-magic) buffered nil)
+		     cover)))
       (if 8bitp (setq 8bit-cnt (1+ 8bit-cnt)))
       (setq cnt (1+ cnt)))
-    (insert (concat "\n--" boundary "--\n"))
+    (insert "\n--" boundary "--\n")
     (save-excursion
       (goto-char cte-pos)
       (mew-header-insert mew-cte: (if (> 8bit-cnt 0) mew-8bit mew-7bit)))
@@ -787,7 +819,7 @@
 	  (setq sbeg (point-min) send (point-max))))
     (unless mew-cs-text-for-net
       (goto-char sbeg) ;; just in case
-      (while (search-forward "\n" nil t) (replace-match "\r\n" nil t))
+      (mew-lf-to-crlf)
       (setq send (point-max)))
     (setq file (mew-make-temp-name))
     (mew-frwlet
@@ -814,7 +846,7 @@
     (condition-case nil
 	(setq fc (funcall func file1 decrypters))
       (error
-       (if (file-exists-p file1) (delete-file file1))
+       (mew-delete-file file1)
        (mew-encode-error
 	(format "unknown error for %s. Check %s, anyway" 
 		mew-ct-mle mew-temp-dir))))
@@ -822,9 +854,9 @@
     (setq errmsg (nth 4 fc))
     (if errmsg
 	(progn
-	  (if (file-exists-p file1) (delete-file file1))
-	  (if (file-exists-p file2) (delete-file file2))
-	  (if (file-exists-p file3) (delete-file file3))
+	  (mew-delete-file file1)
+	  (mew-delete-file file2)
+	  (mew-delete-file file3)
 	  (mew-tinfo-set-privacy-err t)
 	  (mew-tinfo-set-privacy-type nil)
 	  (mew-draft-mode-name)
@@ -844,9 +876,9 @@
        (mew-encode-syntax-single file3 mew-type-apo cte3))
       (insert (format "\n--%s--\n" boundary))
       ;; Throw away the garbage 
-      (if (file-exists-p file1) (delete-file file1))
-      (if (file-exists-p file2) (delete-file file2))
-      (if (file-exists-p file3) (delete-file file3)))))
+      (mew-delete-file file1)
+      (mew-delete-file file2)
+      (mew-delete-file file3))))
 
 (defun mew-encode-multipart-signed (ct proto depth)
   ;; called in the narrowed region
@@ -860,7 +892,7 @@
     (condition-case nil
 	(setq fmc (funcall func file1))
       (error
-       (if (file-exists-p file1) (delete-file file1))
+       (mew-delete-file file1)
        (mew-encode-error
 	(format "unknown error for %s. Check %s, anyway" 
 		mew-ct-mls mew-temp-dir))))
@@ -869,8 +901,8 @@
     (setq ct2 (nth 4 fmc) cdp2 (nth 5 fmc))
     (if errmsg
 	(progn
-	  (if (file-exists-p file1) (delete-file file1))
-	  (if (file-exists-p file2) (delete-file file2))
+	  (mew-delete-file file1)
+	  (mew-delete-file file2)
 	  (mew-tinfo-set-privacy-err t)
 	  (mew-tinfo-set-privacy-type nil)
 	  (mew-draft-mode-name)
@@ -891,8 +923,8 @@
        (mew-encode-syntax-single file2 (or ct2 (list proto)) cte2 nil nil cdp2))
       (insert (format "\n--%s--\n" boundary))
       ;; Throw away the garbage 
-      (if (file-exists-p file1) (delete-file file1))
-      (if (file-exists-p file2) (delete-file file2)))))
+      (mew-delete-file file1)
+      (mew-delete-file file2))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -910,8 +942,7 @@
        ((and (mew-case-equal cte mew-b64) (fboundp 'base64-encode-region))
 	(when linebasep
 	  (goto-char (point-min))
-	  (while (search-forward "\n" nil t)
-	    (replace-match "\r\n" nil t)))
+	  (mew-lf-to-crlf))
 	(base64-encode-region (point-min) (point-max))
 	(goto-char (point-max))
 	(insert "\n"))
@@ -928,9 +959,8 @@
 	   (delete-region (point-min) (point-max)))
 	  (mew-piolet
 	   mew-cs-text-for-read mew-cs-dummy
-	   (apply (function call-process) mew-prog-mime-encode
-		  file t nil opt))
-	  (if (file-exists-p file) (delete-file file))))
+	   (apply 'call-process mew-prog-mime-encode file t nil opt))
+	  (mew-delete-file file)))
        (t
 	(mew-encode-error (concat mew-prog-mime-encode " does not exist")))))))
 
@@ -938,7 +968,7 @@
   ;; called on the beginning of a header
   (let ((case-fold-search t)
 	(buf (current-buffer))
-	tmpbuf hd-end mimep charset cte body-beg body-end)
+	hd-end mimep charset cte body-beg body-end)
     (unless (re-search-forward mew-eoh nil t)
       (goto-char (point-max)))
     (setq hd-end (point-marker))
@@ -957,14 +987,10 @@
       (forward-line)
       (setq body-beg (point))
       (setq body-end (point-max))
-      (setq tmpbuf (generate-new-buffer mew-buffer-prefix))
-      (save-excursion
-	(set-buffer tmpbuf)
-	(mew-erase-buffer)
+      (with-temp-buffer
 	(mew-insert-buffer-substring buf body-beg body-end)
 	(mew-cs-decode-region (point-min) (point-max) mew-cs-autoconv)
 	(setq charset (mew-charset-guess-region (point-min) (point-max))))
-      (mew-remove-buffer tmpbuf)
       (when (mew-encode-charset-8bitp charset)
 	(setq cte (mew-charset-to-cte charset))
 	(if (null cte) (setq cte mew-b64))
@@ -1022,7 +1048,8 @@
 		  (mew-case-equal cte mew-8bit)
 		  (mew-case-equal cte mew-bin))
 	      (setq cte mew-b64))
-	  (mew-convert-mime-body ctbody-beg (point-max) cte 'linebasep))
+	  (unless (mew-case-equal cte mew-7bit)
+	    (mew-convert-mime-body ctbody-beg (point-max) cte 'linebasep)))
 	 (t
 	  ;; rare case
 	  (setq cte (mew-ctdb-cte (mew-ctdb-by-ct ct)))
@@ -1030,8 +1057,9 @@
 		  (mew-case-equal cte mew-8bit)
 		  (mew-case-equal cte mew-bin))
 	      (setq cte mew-b64))
-	  (mew-convert-mime-body
-	   ctbody-beg (point-max) cte (mew-ct-linebasep ct)))
+	  (unless (mew-case-equal cte mew-7bit)
+	    (mew-convert-mime-body
+	     ctbody-beg (point-max) cte (mew-ct-linebasep ct))))
 	 (goto-char (point-min)))
 	(mew-header-delete-lines (list mew-cte:))
 	(goto-char cthd-end)
@@ -1056,7 +1084,7 @@
       (while t
 	(unless (re-search-forward bregex nil t)
 	  (mew-encode-error (format "No last boundary for %s" ct)))
-	(setq break (string= (regexp-quote (mew-match 0)) ebound))
+	(setq break (string= (regexp-quote (match-string 0)) ebound))
 	(forward-line) ;; the beginning of the next part
 	(save-excursion
 	  (forward-line -1)
@@ -1112,7 +1140,7 @@
     (when (file-exists-p back)
       (mew-frwlet
        mew-cs-m17n mew-cs-dummy
-       (insert-file-contents back)))))
+       (mew-insert-file-contents back)))))
 
 (defun mew-encode-undo ()
   "Get back to the draft before making MIME message."
@@ -1133,7 +1161,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2001 Mew developing team.
+;; Copyright (C) 1996-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
