@@ -509,14 +509,12 @@ the queue, type '\\[mew-summary-send-message]' in the queue online."
 	       (imapq (mew-folder-case-folder
 		       case (mew-imap-queue-folder case)))
 	       (imapq-dir (mew-expand-folder imapq))
-	       file file-info buf ref)
+	       file file-info ref)
 	  (unless (file-directory-p imapq-dir) (mew-make-directory imapq-dir))
 	  (setq file (mew-folder-new-message imapq))
 	  (setq file-info (concat file mew-imapq-info-suffix))
 	  (mew-lisp-save file-info (cons fld info) 'nobackup 'unlimit)
-	  (setq buf (find-file-noselect file))
-	  (save-excursion
-	    (set-buffer buf)
+	  (with-temp-buffer
 	    (mew-header-insert mew-subj: (format "IMAP jobs for %s" fld))
 	    (mew-header-insert mew-date: (mew-time-ctz-to-rfc (current-time)))
 	    (mew-header-insert mew-from: "Mew IMAP manager")
@@ -539,9 +537,8 @@ the queue, type '\\[mew-summary-send-message]' in the queue online."
 	    (set-buffer-modified-p nil)
 	    (mew-frwlet
 	     mew-cs-dummy nil
-	     (write-region (point-min) (point-max) (buffer-file-name) nil 'no-msg)))
+	     (write-region (point-min) (point-max) file nil 'no-msg)))
 	  (mew-touch-folder imapq)
-	  (mew-remove-buffer buf)
 	  (if (mew-imap-trash-folder case)
 	      (let* ((mew-inherit-exec-case case)
 		     (mew-msg-rm-policy 'trashonly)
