@@ -518,7 +518,7 @@
   (mew-flet
    (let* ((file (mew-syntax-get-param params "name")))
      (if (file-exists-p file)
-	 (insert-file-contents file)))))
+	 (mew-insert-file-contents file)))))
 
 (provide 'mew-ext)
 
