@@ -12,34 +12,29 @@
 (defun mew-fib-split (str)
   (let (ret match)
     (while (string-match "[\t \n]*\\([^,]+\\)" str)
-      (setq  match (substring str (match-beginning 1) (match-end 1))
-	     str (substring str (match-end 0))
-	     match (if (string-match "[\t \n]+$" match)
-		       (substring match 0 (match-beginning 0))
-		     match)
-	     ret (cons match ret)))
-    (reverse ret)))
+      (setq match (substring str (match-beginning 1) (match-end 1)))
+      (setq str (substring str (match-end 0)))
+      (setq match (if (string-match "[\t \n]+$" match)
+		      (substring match 0 (match-beginning 0))
+		    match))
+      (setq ret (cons match ret)))
+    (nreverse ret)))
 
 (defun mew-fib-make-alist ()
   (let ((fib-file (expand-file-name mew-fib-item-file))
-	(buf (generate-new-buffer mew-buffer-prefix))
 	item val ret tmp-val)
     (setq mew-fib-item-alist nil)
-    (save-excursion
-      (set-buffer buf)
-      (mew-erase-buffer)
+    (with-temp-buffer
       (if (file-exists-p fib-file)
-	  (insert-file-contents fib-file))
+	  (mew-insert-file-contents fib-file))
       (goto-char (point-min))
       (delete-matching-lines "^[ \t]*[;#%]")
       (while (re-search-forward "^\\([^:]+\\):[ \t]*\\(.*\\)$" nil t)
 	(setq item    (mew-buffer-substring (match-beginning 1) (match-end 1))
 	      tmp-val (mew-buffer-substring (match-beginning 2) (match-end 2))
 	      val (if (string= tmp-val "") val tmp-val)
-	      ret (append ret (mapcar (function (lambda (arg)
-						  (cons (downcase arg) val)))
+	      ret (append ret (mapcar (lambda (arg) (cons (downcase arg) val))
 				      (mew-fib-split item))))))
-    (mew-remove-buffer buf)
     ret))
 
 (defun mew-fib-fill-default ()
@@ -55,8 +50,7 @@
 	      str (mew-buffer-substring begin end))
 	(delete-region begin end)
 	(backward-char 2)
-	(insert (or (cdr (assoc (downcase str) mew-fib-item-alist)) str)))
-    )))
+	(insert (or (cdr (assoc (downcase str) mew-fib-item-alist)) str))))))
 
 (defun mew-fib-delete-frame ()
   "Delete all quotations, '|>' and '<|'. This is the finishing stroke."
@@ -98,7 +92,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1994-2002 Mew developing team.
+;; Copyright (C) 1994-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
