@@ -64,6 +64,17 @@ If FOLDER is a sub-folder of KEY or KEY itself, t is returned."
 	(setq list (cdr list))
 	(setq n (1+ n))))))
 
+(defun mew-member* (x list)
+  "Member in a nested list."
+  (catch 'found
+    (while list
+      (if (consp (car list))
+	  (if (mew-member* x (car list))
+	      (throw 'found t))
+	(if (equal x (car list))
+	    (throw 'found t)))
+      (setq list (cdr list)))))
+
 (defun mew-member-match (str list &optional ignore-case)
   "Return the position matched to STR in LIST. If
 IGNORE-CASE is t, matching is performed by ignoring case."
@@ -75,6 +86,14 @@ IGNORE-CASE is t, matching is performed by ignoring case."
 	(setq list (cdr list))
 	(setq n (1+ n))))))
 
+(defun mew-member-match2 (regex list &optional ignore-case)
+  (let ((case-fold-search ignore-case))
+    (catch 'member
+      (while list
+	(if (string-match regex (car list))
+	    (throw 'member (car list)))
+	(setq list (cdr list))))))
+
 (defun mew-uniq-list (lst)
   "Destructively uniqfy elements of LST.
 This is O(N^2). So, do not use this function with a large LST."
@@ -84,7 +103,7 @@ This is O(N^2). So, do not use this function with a large LST."
 
 (defun mew-uniq-alist (alst)
   "Destructively uniqfy elements of ALST."
-  (let ((vec (make-vector 511 0))
+  (let ((vec (make-vector 511 0)) ;; hash
 	ent str ret)
     (while alst
       (setq ent (car alst))
@@ -188,12 +207,12 @@ to the list itself."
 (defsubst mew-assoc-member (key lol nth)
   "Return a list member of LoL whose NTH list contains 
 a member equal to KEY."
-  (mew-assoc-member-base key lol nth (function member)))
+  (mew-assoc-member-base key lol nth 'member))
 
 (defsubst mew-assoc-member-case-equal (key lol nth)
   "Return a list member of LoL whose NTH list contains 
 a member equal to KEY ignoring case."
-  (mew-assoc-member-base key lol nth (function mew-member-case-equal)))
+  (mew-assoc-member-base key lol nth 'mew-member-case-equal))
 
 (defun mew-assoc-member-base (key lol nth func)
   "Return a list member of LoL whose NTH list contains KEY
@@ -235,6 +254,10 @@ in the context of FUNC."
 (defmacro mew-add-first (variable value)
   `(setq ,variable (cons ,value ,variable)))
 
+(defmacro mew-addq (variable value)
+  `(if (and ,value (not (member ,value ,variable)))
+       (setq ,variable (cons ,value ,variable))))
+
 (defmacro mew-insert-after (variable value key)
   `(let ((var ,variable))
      (catch 'loop
@@ -287,14 +310,6 @@ in the context of FUNC."
     (setq string (replace-match " " nil t string)))
   string)
 
-(defsubst mew-match (pos &optional string)
-  "Substring POSth matched from STRING or the current buffer."
-  (cond 
-   ((stringp string)
-    (substring string (match-beginning pos) (match-end pos)))
-   (t 
-    (mew-buffer-substring (match-beginning pos) (match-end pos)))))
-
 (defsubst mew-capitalize (ostr)
   "Syntax table independent version of capitalize.
 Words are separated by '/' and '-'."
@@ -330,22 +345,22 @@ Words are separated by '/' and '-'."
   (let ((file (mew-expand-folder fld msg)))
     (cond
      ((not (file-readable-p file))
-      (error "%s%s does not exist" (file-name-as-directory fld) msg))
+      (error "%s does not exist" (mew-concat-folder fld msg)))
      ((= (mew-file-get-size file) 0)
-      (error "The size of %s%s is 0" (file-name-as-directory fld) msg))
+      (error "The size of %s is 0" (mew-concat-folder fld msg)))
      ((file-readable-p file)
       (let ((old-cs (and (boundp 'buffer-file-coding-system)
 			 buffer-file-coding-system)))
 	(mew-frwlet
 	 rcs mew-cs-dummy
-	 (insert-file-contents file nil 0 size))
+	 (mew-insert-file-contents file nil 0 size))
 	(if (boundp 'buffer-file-coding-system)
 	    (setq buffer-file-coding-system old-cs)))
       ;; return physical size
       (cons (mew-file-get-time file) (mew-file-get-size file))))))
 
 (defsubst mew-insert-manual (&rest args)
-  (insert (substitute-command-keys (apply (function concat) args))))
+  (insert (substitute-command-keys (apply 'concat args))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -353,19 +368,19 @@ Words are separated by '/' and '-'."
 ;;;
 
 (defsubst mew-join (separator string-list)
-  (mapconcat (function identity) string-list separator))
+  (mapconcat 'identity string-list separator))
 
 (defun mew-split (str sepchar)
   "Return a list of strings splitting STR with SEPCHAR."
- (let ((len (length str)) (start 0) (i 0) ret)
-   (while (< i len)
-     (when (char-equal (aref str i) sepchar)
-       (setq ret (cons (substring str start i) ret))
-       (setq start (1+ i)))
-     (setq i (1+ i)))
-   (if (/= start len)
-       (setq ret (cons (substring str start) ret)))
-   (nreverse ret)))
+  (let ((len (length str)) (start 0) (i 0) ret)
+    (while (< i len)
+      (when (char-equal (aref str i) sepchar)
+	(setq ret (cons (substring str start i) ret))
+	(setq start (1+ i)))
+      (setq i (1+ i)))
+    (if (/= start len)
+	(setq ret (cons (substring str start) ret)))
+    (nreverse ret)))
 
 (defun mew-split-quoted (str sepchar &optional qopen qclose)
   "Return a list of strings splitting STR with SEPCHAR.
@@ -413,6 +428,19 @@ also ignored."
 	  (throw 'loop nil))))
     (substring str i (1+ j))))
 
+(defun mew-quote-string (str qchar targets)
+  (let* ((len (length str))
+	(ret (make-string (* len 2) 32))
+	(i 0) (j 0) c)
+    (while (< i len)
+      (setq c (aref str i))
+      (when (member c targets)
+	(aset ret j qchar)
+	(setq j (1+ j)))
+      (aset ret j c)
+      (setq i (1+ i) j (1+ j)))
+    (substring ret 0 j)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Folder
@@ -421,28 +449,86 @@ also ignored."
 (defsubst mew-folder-regex (folder)
   (concat "^" (regexp-quote folder)))
 
-(defsubst mew-folder-mailp (folder)
-  (string-match "^+" folder))
+;;
 
-(defsubst mew-folder-newsp (folder)
+(defun mew-folder-case (folder)
+  ;; "case" must be distinguished from drive
+  (if (string-match mew-regex-case folder)
+      (cons (mew-match-string 1 folder) (mew-match-string 2 folder))))
+
+(defun mew-folder-folder (folder)
+  (let ((case-folder (mew-folder-case folder)))
+    (if case-folder
+	(cdr case-folder)
+      folder)))
+
+(defun mew-folder-case-folder (case folder)
+  (if (and case
+	   (not (string= case mew-case-default))
+	   (mew-folder-remotep folder))
+      (concat case ":" folder)
+    folder))
+
+;;
+
+(defsubst mew-folder-absolutep (folder)
+  (or (string-match mew-regex-file-absolute folder)
+      (and (featurep 'mew-win32)
+	   (string-match mew-regex-drive-letter folder))))
+
+(defsubst mew-folder-localp (folder)
+  (string-match "^\\+" folder))
+
+(defsubst mew-folder-remotep (folder)
+  (string-match "^[-%$]" folder))
+
+(defsubst mew-folder-popp (folder)
+  (string-match "^\\$" folder))
+
+(defsubst mew-folder-nntpp (folder)
   (string-match "^-" folder))
 
-(defsubst mew-folder-local-newsp (folder)
-  (string-match "^=" folder))
-
 (defsubst mew-folder-imapp (folder)
   (string-match "^%" folder))
 
 (defsubst mew-folder-virtualp (folder)
-  (string-match "^\\+[+=]" folder))
+  (string-match "^\\*" folder))
 
-(defsubst mew-folder-to-dir (folder)
-  (if (string-match "^[+=]" folder)
+(defsubst mew-virtual-thread-p (vfolder)
+  (save-excursion  
+    (set-buffer vfolder)
+    (mew-thread-p)))
+
+;;
+
+(defsubst mew-folder-prefix (folder)
+  (substring folder 0 1))
+
+(defsubst mew-folder-string (folder)
+  (substring folder 1))
+
+(defsubst mew-string-to-local (folder)
+  (concat mew-folder-local folder))
+
+;;
+
+(defsubst mew-folder-to-virtual (folder)
+  (concat mew-folder-virtual folder mew-folder-virtual))
+
+(defsubst mew-folder-to-thread (folder)
+  (concat mew-folder-virtual folder))
+
+(defsubst mew-thread-to-folder (folder)
+  (mew-folder-string folder))
+
+;;
+
+(defsubst mew-local-to-dir (folder)
+  (if (string-match "^[+]" folder)
       (substring folder 1 nil)
     folder))
 
-(defsubst mew-folder-localp (folder)
-  (or (mew-folder-mailp folder) (mew-folder-local-newsp folder)))
+;;
 
 (defsubst mew-folder-inboxp (folder)
   (member folder mew-inbox-folders))
@@ -450,75 +536,132 @@ also ignored."
 (defsubst mew-folder-queuep (folder)
   (member folder mew-queue-folders))
 
-(defsubst mew-folder-mdropp (folder)
-  (member folder mew-mdrop-folders))
+(defsubst mew-folder-postqp (folder)
+  (member folder mew-postq-folders))
 
 (defsubst mew-folder-draftp (folder)
   (equal folder mew-draft-folder))
 
+;;
+
+(defsubst mew-folder-imap-queuep ()
+  (string= (mew-sinfo-get-folder)
+	   (mew-imap-queue-folder (mew-sinfo-get-case))))
+
+;;
+
 (defun mew-canonicalize-folder (folder)
-  (if (or (mew-folder-mailp folder)
-	  (mew-folder-local-newsp folder)
+  (if (or (mew-folder-localp folder)
 	  (file-name-absolute-p folder))
       folder
-    (concat "+" folder)))
+    (mew-string-to-local folder)))
+
+(defun mew-path-to-folder (path)
+  (let ((regex (concat "^" (regexp-quote (file-name-as-directory (expand-file-name mew-mail-path))))))
+    (if (string-match regex path)
+	(mew-string-to-local (substring path (match-end 0)))
+      path)))
+
+(defun mew-canonicalize-case-folder (case:folder)
+  (let* ((case-folder (mew-folder-case case:folder))
+	 (case (car case-folder))
+	 (folder (cdr case-folder)))
+    (unless folder (setq folder case:folder))
+    (cond
+     ((mew-folder-localp folder) folder)
+     ((mew-folder-absolutep folder) folder)
+     ((mew-folder-virtualp folder) folder)
+     (case
+      (if (string-equal case mew-case-default)
+	  folder
+	case:folder))
+     (t case:folder))))
+
+(defun mew-folder-remote-to-local (folder)
+  (let ((case-folder (mew-folder-case folder))
+	case fld)
+    (when case-folder
+      (setq case (car case-folder))
+      (setq folder (cdr case-folder)))
+    (setq fld (mew-folder-string folder))
+    (cond
+     ((mew-folder-localp folder) folder)
+     ((mew-folder-popp folder)
+      (mew-concat-folder (mew-pop-folder case) fld))
+     ((mew-folder-imapp folder)
+      (mew-concat-folder (mew-imap-folder case) fld))
+     ((mew-folder-nntpp folder)
+      (mew-concat-folder (mew-nntp-folder case) fld)))))
+
+;;
+
+(defun mew-expand-folder2 (folder)
+  (let ((case-folder (mew-folder-case folder))
+	case subdir)
+    ;; The length of "case" must be longer than or equal to 2.
+    (when case-folder
+      (setq case (car case-folder))
+      (setq folder (cdr case-folder)))
+    (setq subdir (mew-folder-string folder))
+    (cond
+     ((mew-folder-popp folder)
+      (mew-concat-folder (mew-pop-folder case) subdir))
+     ((mew-folder-nntpp folder)
+      (mew-concat-folder (mew-nntp-folder case) subdir))
+     ((mew-folder-imapp folder)
+      (mew-concat-folder (mew-imap-folder case) subdir case))
+     (t folder))))
 
 (defun mew-expand-folder (folder &optional message)
-  (let ((subdir (substring folder 1 nil))
-	dir)
-    (cond
-     ((mew-folder-mailp folder)
-      (setq dir (expand-file-name subdir mew-mail-path)))
-     ((mew-folder-local-newsp folder)
-      (setq dir (expand-file-name subdir mew-news-path)))
-     ((file-name-absolute-p folder)
-      (setq dir (expand-file-name folder))))
+  (when (stringp folder)
+    (let (dir)
+    (setq folder (mew-expand-folder2 folder))
+    (if (mew-folder-localp folder)
+	(setq dir (expand-file-name (mew-folder-string folder) mew-mail-path))
+      ;; absolute path
+      ;; "C:/" -> t, "C:" -> nil , "CC:/" -> nil
+      (setq dir (expand-file-name folder)))
     (if message
 	(expand-file-name message dir)
-      dir)))
+      dir))))
 
-(defun mew-folder-check (folder &optional force-to-create)
-  "A function to see if FOLDER exists.
-Return t if exists or created. Otherwise, return nil."
-  (when (stringp folder)
-    (let ((absdir (mew-expand-folder folder))  ;; /home/Mail/foo
-	  (create-it force-to-create))
-      (when absdir
-	(if (file-exists-p absdir)
-	    (if (file-directory-p absdir)
-		(progn
-		  ;; The folder does not exist in mew-folder-list if
-		  ;; a user creates it by hand...
-		  (mew-folder-insert folder) ;; just in case
-		  t) ;; exists
-	      (message "%s is a file" folder)
-	      nil) ;; xxx exists but a file
-	  (unless create-it
-	    (if (y-or-n-p (format "%s does not exist. Create it? " folder))
-		(setq create-it t)))
-	  (if (not create-it)
-	      nil ;; not created
-	    (mew-make-directory absdir)
-	    (mew-folder-insert folder)
-	    (mew-lisp-save mew-folder-list-file mew-folder-list)
-	    (mew-lisp-save mew-folder-alist-file mew-folder-alist)
-	    (message "%s has been created" folder)
-	    t)))))) ;; created
+(defun mew-concat-folder (folder subfolder &optional case)
+  (if (mew-folder-imapp folder)
+      (let* ((sep (mew-imap-separator case))
+	     (regex (concat (regexp-quote sep) "$")))
+	(if (string-match regex folder)
+	    (concat folder subfolder)
+	  (concat folder sep subfolder)))
+    (concat (file-name-as-directory folder) subfolder)))
 
-(defsubst mew-dir-messages (dir)
-  ;; (default-file-name-coding-system nil)
-  ;; (file-name-coding-system nil)
-  ;; This makes scan faster but makes non-ASCII directories
-  ;; unavailable.
-  (directory-files dir nil mew-regex-message-files 'no-sort))
+(defsubst mew-dir-messages (dir &optional regex)
+  ;; directory_files uses ENCODE_FILE() for DIR and
+  ;;                      DECODE_FILE() for results(FILES).
+  ;; Both ENCODE_FILE() and DECODE_FILE() are quite slow.
+  ;; Since DECODE_FILE() is used many times, directory_files is
+  ;; too slow when {default-,}file-name-coding-system are non-nil.
+  ;; If {default-,}file-name-coding-system are bound to nil, 
+  ;; ENCODE_FILE() and DECODE_FILE() are skipped, resulting speed up.
+  ;; So, we need to encode DIR by ourselves.
+  (let ((edir (expand-file-name dir))
+	;; XEmacs 21.1.14 does not have default-file-name-coding-system
+	(cs (or (and (boundp 'default-file-name-coding-system)
+		     default-file-name-coding-system)
+		(and (boundp 'file-name-coding-system)
+		     file-name-coding-system))))
+    (setq edir (mew-cs-encode-string edir cs))
+    (or regex (setq regex mew-regex-message-files))
+    (mew-alet
+     (directory-files edir nil regex 'no-sort))))
 
-(defun mew-folder-new-message (folder &optional num-only)
+(defun mew-folder-new-message (folder &optional num-only cache)
   (let* ((dir (mew-expand-folder folder))
-	 (max 0) cur maxfile maxpath msgs)
+	 (regex (if cache mew-regex-message-files3 mew-regex-message-files))
+	 (max 0)
+	 cur maxfile maxpath msgs)
     ;; xxx create if there is no directory?
-    (if (not (file-directory-p dir))
-	nil
-      (setq msgs (mew-dir-messages dir))
+    (when (file-directory-p dir)
+      (setq msgs (mew-dir-messages dir regex))
       (while msgs
 	(setq cur (string-to-int (car msgs)))
 	(if (> cur max) (setq max cur))
@@ -526,12 +669,12 @@ Return t if exists or created. Otherwise, return nil."
       (setq max (1+ max))
       (setq maxfile (int-to-string max))
       (setq maxpath (mew-expand-folder folder maxfile))
-      (when (file-exists-p maxpath)
+      (while (file-exists-p maxpath) ;; xxx
 	;; If NFS is used, readdir() may fail. Emacs 20 does not
 	;; rescan the directory again. So, we need to rescan in
 	;; the Emacs level.
-	(mew-touch-folder folder);; can clear cache?
-	(setq msgs (mew-dir-messages dir))
+	(mew-touch-folder folder) ;; can clear cache?
+	(setq msgs (mew-dir-messages dir regex))
 	(setq max 0)
 	(while msgs
 	  (setq cur (string-to-int (car msgs)))
@@ -540,15 +683,15 @@ Return t if exists or created. Otherwise, return nil."
 	(setq max (1+ max))
 	(setq maxfile (int-to-string max))
 	(setq maxpath (mew-expand-folder folder maxfile)))
-      (when (get-file-buffer maxpath)
+      (while (get-file-buffer maxpath) ;; xxx
 	;; file not exist but there is a buffer.
 	(setq max (1+ max))
 	(setq maxfile (int-to-string max))
 	(setq maxpath (mew-expand-folder folder maxfile)))
       (while (file-exists-p maxpath)
-	(setq maxfile (read-string (format "%s/%s exists. Input a message number : " max folder)))
+	(setq maxfile (read-string (format "%s/%s exists. Input a message number: " max folder)))
 	(while (not (string-match mew-regex-message-files maxfile))
-	  (setq maxfile (read-string "Input NUMBER : ")))
+	  (setq maxfile (read-string "Input NUMBER: ")))
 	(setq maxpath (mew-expand-folder folder maxfile)))
       (if num-only
 	  maxfile
@@ -561,7 +704,44 @@ Return t if exists or created. Otherwise, return nil."
 	       (setq file (mew-expand-folder fld mew-summary-touch-file))
 	       (file-writable-p file))
       (write-region "touched by Mew." nil file nil 'no-msg)
-      (set-file-modes file mew-file-mode))))
+      (mew-set-file-modes file))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Folder spec
+;;;
+
+(defconst mew-folder-spec-type
+  '((regex     . string-match)
+    (recursive . mew-folder-recursive-match)
+    (string    . string=)))
+
+(defsubst mew-folder-spec-func (type)
+  (or (cdr (assq type mew-folder-spec-type)) 'string=))
+
+(defun mew-folder-spec (folder lst str-type lst-type)
+  (let ((str-func (mew-folder-spec-func str-type))
+	(lst-func (mew-folder-spec-func lst-type))
+	key keys values ret)
+    (setq folder (mew-folder-folder folder))
+    (catch 'loop
+      (while lst
+	(setq keys (car (car lst)))
+	(setq values (cdr (car lst)))
+	(setq lst (cdr lst))
+	(cond
+	 ((eq keys t)
+	  (throw 'loop (setq ret values)))
+	 ((stringp keys)
+	  (if (funcall str-func keys folder)
+	      (throw 'loop (setq ret values))))
+	 ((listp keys)
+	  (while keys
+	    (setq key (car keys))
+	    (setq keys (cdr keys))
+	    (if (funcall lst-func key folder)
+		(throw 'loop (setq ret values))))))))
+    ret))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -594,15 +774,10 @@ Return t if exists or created. Otherwise, return nil."
     (unless (directory-files dir 'full mew-regex-files 'no-sort)
       (delete-directory dir))))
 
-(defun mew-directory-dirs (dir)
-  (let ((files (directory-files dir 'full "^[^.]" 'no-sort))
-	file dirs)
-    (while files
-      (setq file (car files))
-      (setq files (cdr files))
-      (if (file-directory-p file)
-	  (setq dirs (cons file dirs))))
-    dirs))
+(defun mew-check-directory (dir)
+  (unless (file-directory-p dir)
+    (unless (file-exists-p dir)
+      (mew-make-directory dir))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -687,6 +862,47 @@ Does not examine containing directories for links."
 	(setq i (1- i)))
       (rename-file file (format "%s.0" file)))))
 
+(defun mew-remove-drive-letter (file)
+  (if (string-match mew-regex-drive-letter file)
+      (substring file 2)
+    file))
+
+(defun mew-set-file-modes (path)
+  (let* ((dir (file-name-directory path))
+	 (dirmode (mew-file-get-mode dir))
+	 (mode (logand dirmode mew-file-mode-mask)))
+    (set-file-modes path mode)))
+
+(defun mew-delete-file (file)
+  (if (and (stringp file) (file-exists-p file)) (delete-file file)))
+
+(defun mew-insert-file-contents (&rest args)
+  "A safe version of insert-file-contents.
+This checks -*-coding:ctext;-*- internally when including a file."
+  (let ((after-insert-file-functions nil)
+	(auto-image-file-mode nil))
+    ;; preventing after-insert-file-set-buffer-file-coding-system
+    (apply 'insert-file-contents args)))
+
+(defun mew-find-file-noselect (&rest args)
+  "A safe version of find-file-noselect.
+This checks -*-coding:ctext;-*- internally when including a file.
+But this does not set buffer-file-coding-system."
+  (let ((after-insert-file-functions nil))
+    ;; preventing after-insert-file-set-buffer-file-coding-system
+    (apply 'mew-find-file-noselect2 args)))
+
+(defun mew-find-file-noselect2 (&rest args)
+  "A safe version of find-file-noselect.
+This checks -*-coding:ctext;-*- internally when including a file
+and set buffer-file-coding-system."
+  (let ((auto-image-file-mode nil)
+	(format-alist nil)
+	(auto-mode-alist nil)
+        (enable-local-variables nil)
+        (find-file-hooks nil))
+    (apply 'find-file-noselect args)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; temp name
@@ -701,7 +917,7 @@ Does not examine containing directories for links."
 	       (not (file-exists-p (expand-file-name fname mew-temp-dir))))
 	  (expand-file-name fname mew-temp-dir)	
 	(if (string-match "\\.[ -~]+$" fname)
-	    (concat (make-temp-name mew-temp-file) (mew-match 0 fname))
+	    (concat (make-temp-name mew-temp-file) (mew-match-string 0 fname))
 	  (make-temp-name mew-temp-file)))
     (make-temp-name mew-temp-file)))
 
@@ -775,15 +991,19 @@ Does not examine containing directories for links."
 (defmacro mew-elet (&rest body)
   `(let ((buffer-read-only nil)
 	 (inhibit-read-only t)
+	 (after-change-functions nil)
 	 (zmacs-regions nil)) ;; for XEmacs
      ,@body))
 
 (defsubst mew-region-bytes (beg end buf)
-  (if (fboundp 'string-bytes)
-      (save-excursion
-	(set-buffer buf)
-	(string-bytes (mew-buffer-substring beg end)))
-    (- end beg)))
+  ;; string-bytes() acts differently on each Emacs.
+  ;; set-buffer-multibyte is also buggy.
+  ;; So, use this way.
+  (save-excursion
+    (set-buffer buf)
+    (if (fboundp 'string-as-unibyte)
+	(length (string-as-unibyte (mew-buffer-substring beg end)))
+      (- end beg))))
 
 (defun mew-count-lines (beg end &optional skip-regex)
   "Return number of lines between BEG and END."
@@ -800,10 +1020,22 @@ Does not examine containing directories for links."
 	    lines)
 	(- (buffer-size) (forward-line (buffer-size)))))))
 
-(defalias 'mew-buffer-substring 'buffer-substring-no-properties)
-
-(defsubst mew-insert-buffer-substring (buf beg end)
-  (insert (save-excursion (set-buffer buf) (mew-buffer-substring beg end))))
+(defun mew-buffer-list (regex &optional listp mode)
+  (let ((bufs (mapcar 'buffer-name (buffer-list)))
+	buf ret)
+    (while bufs
+      (setq buf (car bufs))
+      (setq bufs (cdr bufs))
+      (when (and (string-match regex buf)
+		 (or (not mode)
+		     (and mode (get-buffer buf)
+			  (save-excursion
+			    (set-buffer buf)
+			    (eq major-mode mode)))))
+	(if listp
+	    (setq ret (cons (list buf) ret))
+	  (setq ret (cons buf ret)))))
+    (nreverse ret)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -811,8 +1043,10 @@ Does not examine containing directories for links."
 ;;;
 
 (defmacro mew-filter (&rest body)
-  `(if (bufferp (process-buffer process)) ;; MUST use 'process'
-       (let ((obuf (buffer-name)))
+  `(let ((pbuf (process-buffer process)) ;; MUST use 'process'
+	 (obuf (buffer-name)))
+     (if (and (bufferp pbuf)
+	      (buffer-name pbuf)) ;; check a killed buffer
 	 ;; must use buffer-name instead of current-buffer
 	 ;; so that get-buffer can detect killed buffer.
 	 (unwind-protect
@@ -830,21 +1064,21 @@ Does not examine containing directories for links."
 		(cdr (assq 'display (frame-parameters)))))
         (process-environment (copy-sequence process-environment)))
     (if disp (setenv "DISPLAY" disp))
-    (apply (function start-process) name buffer program program-args)))
+    (apply 'start-process name buffer program program-args)))
 
 (defun mew-start-process-lang (name buffer program &rest program-args)
   (let ((process-environment (copy-sequence process-environment)))
     (setenv "LANGUAGE" "C")
     (setenv "LC_ALL" "C")
     (setenv "LANG" "C")
-    (apply (function start-process) name buffer program program-args)))
+    (apply 'start-process name buffer program program-args)))
 
 (defun mew-call-process-lang (prog &optional infile buffer display &rest args)
   (let ((process-environment (copy-sequence process-environment)))
     (setenv "LANGUAGE" "C")
     (setenv "LC_ALL" "C")
     (setenv "LANG" "C")
-    (apply (function call-process) prog infile buffer display args)))
+    (apply 'call-process prog infile buffer display args)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -858,7 +1092,7 @@ Does not examine containing directories for links."
     (if (or (not (intern-soft name))
 	    (not (boundp (intern name))))
 	(if (string-match "^mew-[^-]+-info-" name)
-	    (let* ((sym (intern (concat (mew-match 0 name) "list")))
+	    (let* ((sym (intern (concat (mew-match-string 0 name) "list")))
 		   (lst (symbol-value sym))
 		   (len (length lst)))
 	      (set (intern name) (make-vector len nil)))))
@@ -881,8 +1115,8 @@ Does not examine containing directories for links."
 		((vectorp arg) (aset arg ,i value)))))
       (setq i (1+ i)))))
 
-(defun mew-info-clean-up (arg)
-  (let ((i 0) vec len)
+(defun mew-info-clean-up (arg &optional start)
+  (let ((i (or start 0)) vec len)
     (cond
      ((stringp arg) (setq vec (mew-info arg)))
      ((vectorp arg) (setq vec arg)))
@@ -891,6 +1125,9 @@ Does not examine containing directories for links."
       (aset vec i nil)
       (setq i (1+ i)))))
 
+(defsubst mew-net-get-ssh-process (pnm)
+  (aref (mew-info pnm) 3))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Buffer local info
@@ -948,9 +1185,8 @@ from (mew-user), (mew-mail-address), and 'mew-mail-address-list'."
   (and from
        (let ((case-fold-search t))
 	 (catch (quote match)
-	   (car (mapcar (function (lambda (arg)
-				    (and (string-match arg from)
-					 (throw (quote match) t))))
+	   (car (mapcar (lambda (arg) (and (string-match arg from)
+					   (throw (quote match) t)))
 			addrs))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -971,7 +1207,7 @@ from (mew-user), (mew-mail-address), and 'mew-mail-address-list'."
 	  (mew-erase-buffer)
 	  (mew-frwlet
 	   mew-cs-m17n mew-cs-dummy
-	   (insert-file-contents fullname))
+	   (mew-insert-file-contents fullname))
 	  (setq lisp 
 		(condition-case nil
 		    (read (current-buffer))
@@ -979,7 +1215,7 @@ from (mew-user), (mew-mail-address), and 'mew-mail-address-list'."
     (mew-remove-buffer tmp-buf)
     lisp))
 
-(defun mew-lisp-save (filename lisp &optional nobackup)
+(defun mew-lisp-save (filename lisp &optional nobackup unlimit)
   "Save LISP to FILENAME. LISP is truncated to mew-lisp-max-length
 by side-effect."
   (let* ((fullname (if (file-name-absolute-p filename)
@@ -991,19 +1227,20 @@ by side-effect."
     (if (file-writable-p fullname)
 	(save-excursion
 	  (if nobackup
-	      (if (file-exists-p fullname)
-		  (delete-file fullname))
+	      (mew-delete-file fullname)
 	    (if (file-exists-p fullname)
 		(rename-file fullname backname 'override)))
 	  (set-buffer tmp-buf)
 	  (mew-erase-buffer)
-	  (if (> (length lisp) mew-lisp-max-length)
+	  (if (and (not unlimit) (> (length lisp) mew-lisp-max-length))
 	      (setcdr (nthcdr (1- mew-lisp-max-length) lisp) nil))
-	  (pp lisp tmp-buf)
+	  (if (> (length lisp) mew-lisp-max-length)
+	      (print lisp tmp-buf)
+	    (pp lisp tmp-buf))
 	  (mew-frwlet
 	   mew-cs-dummy mew-cs-m17n
 	   (write-region (point-min) (point-max) fullname nil 'no-msg))
-	  (set-file-modes fullname mew-file-mode)))
+	  (mew-set-file-modes fullname)))
     (mew-remove-buffer tmp-buf)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1024,22 +1261,19 @@ by side-effect."
 ;;; Time Zone
 ;;;
 
+;; Define symbol time zone defined in RFC822 only.
 (defvar mew-time-tmzn-alist
-  '(("PST" . -8) ("PDT" . -7) ("MST" . -7) ("MDT" . -6)
-    ("CST" . -6) ("CDT" . -5) ("EST" . -5) ("EDT" . -4)
-    ("AST" . -4) ("NST" . -3) ("UT"  . +0) ("GMT" . +0)
-    ("BST" . +1) ("MET" . +1) ("EET" . +2) ("JST" . +9)))
+  '(("UT"  .  0) ("GMT" .  0)
+    ("EST" . -5) ("EDT" . -4)
+    ("CST" . -6) ("CDT" . -5) 
+    ("MST" . -7) ("MDT" . -6)
+    ("PST" . -8) ("PDT" . -7)))
 
 (defsubst mew-time-tmzn-str-to-int (str)
   (cdr (assoc (upcase str) mew-time-tmzn-alist)))
 
-(defsubst mew-time-tmzn-int ()
-  (let ((tz (car (current-time-zone))))
-    (if (< tz 0)
-	(format "-%02d%02d" (/ (- tz) 3600) (/ (% (- tz) 3600) 60))
-      (format "+%02d%02d" (/ tz 3600) (/ (% tz 3600) 60)))))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
 ;;; RFC 822
 ;;; Date: Wed, 26 Jul 2000 21:18:35 +0900 (JST)
 ;;;
@@ -1060,31 +1294,41 @@ by side-effect."
   '(string-to-int (substring s (match-beginning 4) (match-end 4))))
 
 (defmacro mew-time-rfc-min  ()
-  '(substring s (match-beginning 5) (match-end 5)))
+  '(string-to-int (substring s (match-beginning 5) (match-end 5))))
 
 (defmacro mew-time-rfc-sec  ()
   '(if (match-beginning 7)
-       (substring s (match-beginning 7) (match-end 7))
-     "00"))
+       (string-to-int (substring s (match-beginning 7) (match-end 7)))
+     0))
 
+;; returns seconds
 (defmacro mew-time-rfc-tmzn ()
   '(if (match-beginning 9)
        (let ((tmzn (substring s (match-beginning 9) (match-end 9)))
 	     int)
 	 (cond
 	  ((string-match "^[-+][0-9]+$" tmzn)
-	   (/ (string-to-int tmzn) 100))
+	   (setq int (string-to-int tmzn))
+	   (+ (* (/ int 100) 3600) (* (% int 100) 60)))
 	  ((setq int (mew-time-tmzn-str-to-int tmzn))
-	   int)
+	   (* int 3600))
 	  (t 0)))
      0))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;;
+;;; Sort key for Date:
 ;;;
 
-;; "20000726231835"
+;; "20000726121835"
+(defsubst mew-time-ctz-to-sortkey (time)
+  (let ((system-time-locale "C"))
+    (format-time-string "%Y%m%d%H%M%S" time)))
+
+(defsubst mew-time-ctz-to-sortkey-invalid (sec min hour day mon year)
+  (format "%04d%02d%02d%02d%02d%02d" year mon day hour min sec))
+
+;; "20000726121835"
 (defsubst mew-time-rfc-to-sortkey (s)
   (if (string-match mew-time-rfc-regex s)
       (let ((year (mew-time-rfc-year))
@@ -1099,36 +1343,22 @@ by side-effect."
 	  (setq year (+ year 2000)))
 	 ((< year 100)
 	  (setq year (+ year 1900))))
-	(setq hour (- hour tmzn))
-	(cond
-	 ((< hour 0)
-	  (setq hour (+ hour 24) day (1- day))
-	  (when (< day 1)
-	    (if (< (setq mon (1- mon)) 1)
-		(setq year (1- year) mon (+ mon 12)))
-	    (setq day (mew-last-day year mon))))
-	 ((> hour 23)
-	  (setq hour (- hour 24) day (1+ day))
-	  (if (> day (mew-last-day year mon))
-		(if (> (setq day 1 mon (1+ mon)) 12)
-		    (setq year (1+ year) mon (- mon 12))))))
-	(format "%4d%02d%02d%02d%s%s" year mon day hour min sec))))
+	(if (or (< year 1970) (>= year 2038))
+	    ;; invalid data
+	    (mew-time-ctz-to-sortkey-invalid sec min hour day mon year)
+	  (setq sec (- sec tmzn))
+	  (mew-time-ctz-to-sortkey (encode-time sec min hour day mon year))))))
 
-(defvar mew-last-day-list '(31 nil 31 30 31 30 31 31 30 31 30 31))
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; User friendly time
+;;;
 
-(defsubst mew-last-day (year mon)
-  (let ((day (nth (1- mon) mew-last-day-list)))
-    (cond
-     (day day)
-     ((and (= (% year 4) 0)
-	   (or (= (% year 400) 0)
-	       (/= (% year 100) 0)))
-      29)
-     (t 28))))
-
-(defsubst mew-time-ctz-to-sortkey (time)
-  (let ((system-time-locale "C"))
-    (format-time-string "%Y%m%d%H%M%S" time)))
+(defsubst mew-time-tmzn-int ()
+  (let ((tz (car (current-time-zone))))
+    (if (< tz 0)
+	(format "-%02d%02d" (/ (- tz) 3600) (/ (% (- tz) 3600) 60))
+      (format "+%02d%02d" (/ tz 3600) (/ (% tz 3600) 60)))))
 
 ;; Wed, 26 Jul 2000 21:18:35 +0900 (JST)
 (defsubst mew-time-ctz-to-rfc (time)
@@ -1190,6 +1420,27 @@ by side-effect."
       (string-to-int port)
     port))
 
+(defsubst mew-dot-insert ()
+  (goto-char (point-min))
+  (while (re-search-forward "^\\." nil t)
+    (insert ".")
+    (forward-line)))
+
+(defsubst mew-dot-delete ()
+  (goto-char (point-max))
+  (forward-line -1)
+  (delete-region (point) (point-max))
+  (goto-char (point-min))
+  (while (re-search-forward "^\\." nil t)
+    (delete-char -1)
+    (forward-line)))
+
+(defsubst mew-crlf-to-lf ()
+  (while (search-forward "\r\n" nil t) (replace-match "\n" nil t)))
+
+(defsubst mew-lf-to-crlf ()
+  (while (search-forward "\n" nil t) (replace-match "\r\n" nil t)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; 
 ;;; sit-for
@@ -1213,7 +1464,7 @@ by side-effect."
   (sit-for 1.5))
 
 (defsubst mew-warn (&rest msg)
-  (apply (function message) msg)
+  (apply 'message msg)
   (ding)
   (mew-let-user-read))
 
@@ -1224,7 +1475,7 @@ by side-effect."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2002 Mew developing team.
+;; Copyright (C) 1997-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
