@@ -266,96 +266,101 @@
        (throw 'max nil))))
 
 (defun mew-addrstr-parse-syntax-list (str sep addrp &optional depth)
-  (let* ((i 0) (len (length str))
-	 (par-cnt 0) (tmp-cnt 0) (sep-cnt 0)
-	 (tmp (make-string len ?x))
-	 c ret)
-    (catch 'max
-      (while (< i len)
-	(setq c (aref str i))
-	(cond
-	 ((char-equal c ?\")
-	  (aset tmp tmp-cnt c)
-	  (setq tmp-cnt (1+ tmp-cnt))
-	  (setq i (1+ i))
-	  (catch 'quote
-	    (while (< i len)
-	      (setq c (aref str i))
-	      (cond
-	       ((char-equal c ?\")
-		(aset tmp tmp-cnt c)
-		(setq tmp-cnt (1+ tmp-cnt))
-		(throw 'quote nil))
-	       ((char-equal c ?\n)
-		(setq i (1+ i))
-		(catch 'fold-quote
-		  (while (< i len)
-		    (setq c (aref str i))
-		    (cond
-		     ;; ((or (char-equal c ?\t) (char-equal c 32))
-		     ;; (setq i (1+ i)))
-		     ((char-equal c ?\")
-		      (aset tmp tmp-cnt c)
-		      (setq tmp-cnt (1+ tmp-cnt))
-		      (throw 'quote nil))
-		     (t
-		      (aset tmp tmp-cnt c)
-		      (setq tmp-cnt (1+ tmp-cnt))
-		      (throw 'fold-quote nil))))))
-	       (t
-		(aset tmp tmp-cnt c)
-		(setq tmp-cnt (1+ tmp-cnt))))
-	      (setq i (1+ i)))))
-	 ((char-equal c ?\()
-	  (setq par-cnt 1)
-	  (setq i (1+ i))
-	  (catch 'comment
-	    (while (< i len)
-	      (setq c (aref str i))
-	      (cond
-	       ((char-equal c ?\()
-		(setq par-cnt (1+ par-cnt)))
-	       ((char-equal c ?\))
-		(setq par-cnt (1- par-cnt))
-		(if (eq par-cnt 0) (throw 'comment nil))))
-	      (setq i (1+ i)))))
-	 ((char-equal c ?<)
+  (when str
+    (if (featurep 'mew-temacs)
+	(setq str (string-as-unibyte str))) ;; A bug of Emacs 20.7
+    (let* ((i 0) (len (length str))
+	   (par-cnt 0) (tmp-cnt 0) (sep-cnt 0)
+	   (tmp (make-string len ?x))
+	   c ret)
+      (catch 'max
+	(while (< i len)
+	  (setq c (aref str i))
 	  (cond
-	   (addrp
-	    (let (rbeg rend)
-	      (setq i (1+ i))
-	      (setq rbeg i)
-	      (while (and (< i len) (not (char-equal (aref str i) ?>)))
+	   ((char-equal c ?\")
+	    (aset tmp tmp-cnt c)
+	    (setq tmp-cnt (1+ tmp-cnt))
+	    (setq i (1+ i))
+	    (catch 'quote
+	      (while (< i len)
+		(setq c (aref str i))
+		(cond
+		 ((char-equal c ?\")
+		  (aset tmp tmp-cnt c)
+		  (setq tmp-cnt (1+ tmp-cnt))
+		  (throw 'quote nil))
+		 ((char-equal c ?\n)
+		  (setq i (1+ i))
+		  (catch 'fold-quote
+		    (while (< i len)
+		      (setq c (aref str i))
+		      (cond
+		       ;; ((or (char-equal c ?\t) (char-equal c 32))
+		       ;; (setq i (1+ i)))
+		       ((char-equal c ?\")
+			(aset tmp tmp-cnt c)
+			(setq tmp-cnt (1+ tmp-cnt))
+			(throw 'quote nil))
+		       (t
+			(aset tmp tmp-cnt c)
+			(setq tmp-cnt (1+ tmp-cnt))
+			(throw 'fold-quote nil))))))
+		 (t
+		  (aset tmp tmp-cnt c)
+		  (setq tmp-cnt (1+ tmp-cnt))))
+		(setq i (1+ i)))))
+	   ((char-equal c ?\()
+	    (setq par-cnt 1)
+	    (setq i (1+ i))
+	    (catch 'comment
+	      (while (< i len)
+		(setq c (aref str i))
+		(cond
+		 ((char-equal c ?\()
+		  (setq par-cnt (1+ par-cnt)))
+		 ((char-equal c ?\))
+		  (setq par-cnt (1- par-cnt))
+		  (if (eq par-cnt 0) (throw 'comment nil))))
+		(setq i (1+ i)))))
+	   ((char-equal c ?<)
+	    (cond
+	     (addrp
+	      (let (rbeg rend)
+		(setq i (1+ i))
+		(setq rbeg i)
+		(while (and (< i len) (not (char-equal (aref str i) ?>)))
+		  (setq i (1+ i)))
+		(when (> i rbeg)
+		  (setq rend i) ;; note: to be used for substring, so not 1-.
+		  ;; should not be nested but easy to implement...
+		  (setq ret (cons (car (mew-addrstr-parse-syntax-list
+					(substring str rbeg rend) sep t))
+				  ret))))
+	      (while (and (< i len) (not (char-equal (aref str i) sep)))
 		(setq i (1+ i)))
-	      (when (> i rbeg)
-		(setq rend i) ;; note: to be used for substring, so not 1-.
-		;; should not be nested but easy to implement...
-		(setq ret (cons (car (mew-addrstr-parse-syntax-list
-				      (substring str rbeg rend) sep t))
-				ret))))
-	    (while (and (< i len) (not (char-equal (aref str i) sep)))
-	      (setq i (1+ i)))
+	      (setq tmp-cnt 0)
+	      (mew-addrstr-parse-syntax-list-check-depth depth))
+	     (t
+	      ;; just ignore
+	      (while (and (< i len) (not (char-equal (aref str i) ?>)))
+		(setq i (1+ i))))))
+	   ((char-equal c ?\n))
+	   ((char-equal c ?\t))
+	   ((char-equal c 32))
+	   ((char-equal c sep)
+	    (if (> tmp-cnt 0)
+		(setq ret (cons (substring tmp 0 tmp-cnt) ret)))
 	    (setq tmp-cnt 0)
 	    (mew-addrstr-parse-syntax-list-check-depth depth))
-	   (t
-	    ;; just ignore
-	    (while (and (< i len) (not (char-equal (aref str i) ?>)))
-	      (setq i (1+ i))))))
-	 ((char-equal c ?\n))
-	 ((char-equal c ?\t))
-	 ((char-equal c 32))
-	 ((char-equal c sep)
-	  (if (> tmp-cnt 0)
-	      (setq ret (cons (substring tmp 0 tmp-cnt) ret)))
-	  (setq tmp-cnt 0)
-	  (mew-addrstr-parse-syntax-list-check-depth depth))
-	 (t 
-	  (aset tmp tmp-cnt c)
-	  (setq tmp-cnt (1+ tmp-cnt)))) ;; end of cond
-	(setq i (1+ i))) ;; end of while
-      (if (> tmp-cnt 0)
-	  (setq ret (cons (substring tmp 0 tmp-cnt) ret)))) ;; end of catch
-    (nreverse ret)))
+	   (t 
+	    (aset tmp tmp-cnt c)
+	    (setq tmp-cnt (1+ tmp-cnt)))) ;; end of cond
+	  (setq i (1+ i))) ;; end of while
+	(if (> tmp-cnt 0)
+	    (setq ret (cons (substring tmp 0 tmp-cnt) ret)))) ;; end of catch
+      (if (featurep 'mew-temacs)
+	  (mapcar 'string-as-multibyte (nreverse ret)) ;; A bug of Emacs 20.7
+	(nreverse ret)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
