@@ -169,57 +169,56 @@
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; X-Face:
+;;; X Face
 ;;;
 
-;; uncompface: ftp://ftp.cs.indiana.edu/pub/faces/xfaces/xfaces-<ver>.tar.Z
-;; icontopbm: ftp://ftp.sourceforge.net/pub/sourceforge/netpbm/netpbm-<ver>.tgz
-
 (defun mew-highlight-x-face (beg end)
-  "A function to display X-Face."
-  (if (and mew-use-highlight-x-face mew-use-highlight-x-face-function)
-      (funcall mew-use-highlight-x-face-function beg end)))
-
-(cond
- (mew-xemacs-p
-  ;;(autoload 'highlight-headers-x-face-to-pixmap "highlight-headers")
-  (if (mew-which-el "highlight-headers")
-      (require 'highlight-headers)) ;; due to the timing problem.
-  ;; now this is in the "mail-lib" package.
-  (defvar mew-use-highlight-x-face-function
-    (lambda (beg end)
-      (when (and window-system
-		 (mew-which-exec mew-prog-uncompface)
-		 mew-use-highlight-x-face)
-	(save-excursion
-	  (goto-char beg)
-	  (mew-elet
-	   (let ((regex1 "^X-Face: *\\(.*\\(\n[ \t].*\\)*\\)\n")
-		 (regex2 (concat "^\\(" mew-from: "\\).*"))
-		 overlay xface)
-	     (while (re-search-forward regex1 end t)
-	       (setq overlay (mew-overlay-make (match-beginning 0)
-					       (match-end 0)))
-	       (overlay-put overlay 'invisible t)
-	       (mew-plet
-		(setq xface (highlight-headers-x-face-to-pixmap
-			     (match-beginning 1)
-			     (match-end 1))))
-	       (save-excursion
-		 (goto-char beg)
-		 (when (re-search-forward regex2 end t)
-		   (setq overlay (mew-overlay-make
-				  (match-end 1) (match-end 1)))
-		   (set-extent-begin-glyph overlay xface)))))))))))
- (t
-  (defvar mew-use-highlight-x-face-function nil
-    "*On Text Emacs, this function is called if mew-use-highlight-x-face
-is *non-nil*. This is a temporary solution.")
+  "Display X-Face."
   (if (and mew-use-highlight-x-face
-	   (mew-which-exec mew-prog-uncompface)
-	   (mew-which-el "bitmap")
-	   (mew-which-el "mew-xface-mule"))
-      (require 'mew-xface-mule))))
+	   window-system
+	   (fboundp mew-highlight-x-face-function))
+      (funcall mew-highlight-x-face-function beg end)))
+
+(defvar mew-highlight-x-face-function 
+  (if mew-icon-p 'mew-highlight-x-face-original) ;; Emacs 21, XEmacs
+  "*A function to display-Xface")
+
+(defun mew-highlight-x-face-original (beg end)
+  (save-excursion
+    (goto-char beg)
+    (mew-elet
+     (let ((regex1 "^X-Face: *\\(.*\\(\n[ \t].*\\)*\\)\n")
+	   (buf (current-buffer))
+	   overlay xface beg0 end0 xbmp)
+       (while (re-search-forward regex1 end t)
+	 (setq beg0 (match-beginning 0))
+	 (setq end0 (match-end 0))
+	 (with-temp-buffer
+	   (mew-insert-buffer-substring
+	    buf (match-beginning 1) (match-end 1))
+	   (setq xbmp (mew-x-face-compface-to-xbm))
+	   (if xbmp (setq xface (mew-x-face-create))))
+	 (when xface
+	   (setq overlay (mew-overlay-make beg0 end0))
+	   (overlay-put overlay 'invisible t)
+	   (save-restriction
+	     (narrow-to-region beg end)
+	     (mew-x-face-display xface ))))))))
+
+(defun mew-x-face-compface-to-xbm ()
+  (when (and (mew-which-exec mew-prog-uncompface)
+	     (mew-which-exec mew-prog-icontopbm)
+	     (mew-which-exec mew-prog-pbmtoxbm))
+    (mew-set-buffer-multibyte nil)
+    (mew-flet
+     (call-process-region (point-min) (point-max) mew-prog-uncompface t t nil)
+     (goto-char (point-min))
+     (insert "/* Format_version=1, Width=48, Height=48, Depth=1, Valid_bits_per_item=16 */\n")
+     (call-process-region (point-min) (point-max) mew-prog-icontopbm t t nil)
+     (if mew-use-highlight-x-face-inversion
+	 (call-process-region (point-min) (point-max) mew-prog-pbminvert t t nil))
+     (call-process-region (point-min) (point-max) mew-prog-pbmtoxbm t t nil))
+    t)) ;; return value
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -313,7 +312,7 @@ is *non-nil*. This is a temporary solution.")
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2002 Mew developing team.
+;; Copyright (C) 1997-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
