@@ -21,7 +21,7 @@
 ;;;
 
 (defvar mew-imap-info-list
-  '("server" "port" "process" "ssh-process" "status"
+  '("server" "port" "process" "ssh-process" "ssl-process" "status"
     "directive" "bnm"
     "rtrs" "dels" "refs" "rmvs" "kils" "uidl" "range"
     "rttl" "rcnt" "dttl" "dcnt" "jcnt" "rfl" "hlds"
@@ -73,8 +73,14 @@
 ;;; Filters
 ;;;
 
+(defun mew-imap-secure-p (pnm)
+  (or (mew-imap-get-ssh-process pnm) (mew-imap-get-ssl-process pnm)))
+ 
 (defun mew-imap-command-capability (pro pnm)
-  (mew-net-status (mew-imap-get-bnm pnm) "Auth'ing")
+  (mew-net-status (mew-imap-get-bnm pnm)
+		  "Auth'ing"
+		  nil
+		  (mew-imap-secure-p pnm))
   (mew-imap-process-send-string pro pnm "CAPABILITY"))
 
 (defun mew-imap-command-authentication (pro pnm)
@@ -129,10 +135,13 @@
       (mew-imap-set-status pnm "list")
       (mew-imap-command-list pro pnm))
      (t
-      (mew-imap-process-send-string pro pnm "SELECT %s" mailbox)))))
+      (mew-imap-process-send-string pro pnm "SELECT \"%s\"" mailbox)))))
 
 (defun mew-imap-command-uid (pro pnm)
-  (mew-net-status (mew-imap-get-bnm pnm) "Checking")
+  (mew-net-status (mew-imap-get-bnm pnm)
+		  "Checking"
+		  nil
+		  (mew-imap-secure-p pnm))
   (let ((directive (mew-imap-get-directive pnm))
 	(refs (mew-imap-get-refs pnm)) ;; (uid siz del (+fld msg))
 	(rmvs (mew-imap-get-rmvs pnm))
@@ -314,7 +323,7 @@
       (mew-imap-set-rtrs pnm (cdr rtrs))
       (setq dst (mew-bnm-to-mailbox dst))
       (mew-imap-set-rfl pnm rtr)
-      (mew-imap-process-send-string pro pnm "UID COPY %s %s" uid dst))))
+      (mew-imap-process-send-string pro pnm "UID COPY %s \"%s\"" uid dst))))
 
 (defun mew-imap-command-create (pro pnm)
   (let* ((rtrs (mew-imap-get-rtrs pnm))
@@ -322,7 +331,7 @@
 	 (dst (nth 0 rtr))) ;; 'exec only
     (mew-imap-set-rtrs pnm (cons rtr rtrs))
     (setq dst (mew-bnm-to-mailbox dst))
-    (mew-imap-process-send-string pro pnm "CREATE %s" dst)))
+    (mew-imap-process-send-string pro pnm "CREATE \"%s\"" dst)))
 
 (defun mew-imap-command-wmbx (pro pnm)
   ;; xxx
@@ -346,7 +355,8 @@
 
 (defun mew-imap-command-dels (pro pnm)
   (mew-net-status1
-   (mew-imap-get-bnm pnm) (mew-imap-get-dttl pnm) (mew-imap-get-dcnt pnm))
+   (mew-imap-get-bnm pnm) (mew-imap-get-dttl pnm) (mew-imap-get-dcnt pnm)
+   (mew-imap-secure-p pnm))
   (let ((dels (mew-imap-get-dels pnm))
 	num)
     (if (null dels)
@@ -381,7 +391,8 @@
 		   (mew-imap-get-rttl pnm)
 		   (mew-imap-get-rcnt pnm)
 		   (nth 1 (car (mew-imap-get-rtrs pnm)))
-		   'zero)
+		   'zero
+		   (mew-imap-secure-p pnm))
   (let* ((directive (mew-imap-get-directive pnm))
 	 (rtrs (mew-imap-get-rtrs pnm))
 	 (dels (mew-imap-get-dels pnm))
@@ -494,7 +505,10 @@
 
 (defun mew-imap-command-list (pro pnm)
   (mew-imap-message pnm "Collecting mailbox list...")
-  (mew-net-status (mew-imap-get-bnm pnm) "Listing")
+  (mew-net-status (mew-imap-get-bnm pnm)
+		  "Listing"
+		  nil
+		  (mew-imap-secure-p pnm))
   (mew-imap-process-send-string pro pnm "LIST \"\" *"))
 
 (defun mew-imap-command-post-list (pro pnm)
@@ -649,6 +663,7 @@
 (defun mew-imap-process-send-string (pro pnm &rest args)
   (let ((str (apply 'format args))
 	(tag (mew-imap-tag)))
+    (mew-imap-debug "=SEND=" str)
     (mew-imap-set-tag pnm tag)
     (process-send-string pro (concat tag " " str mew-cs-eol))))
 
@@ -739,7 +754,7 @@
 	 (pnm (mew-imap-info-name case mailbox))
 	 (buf (get-buffer-create (mew-imap-buffer-name pnm)))
 	 (no-msg (eq directive 'biff))
-	 process sshpro sshname lport)
+	 process sshname sshpro sslname sslpro lport)
     (if (mew-imap-get-process pnm)
 	(message "Another IMAP process is running. Try later")
       (cond
@@ -751,7 +766,12 @@
 	  (when lport
 	    (setq process (mew-imap-open pnm "localhost" lport no-msg)))))
        (sslp
-	(setq process (mew-open-ssl-stream pnm nil server sslport)))
+ 	(setq sslpro (mew-open-ssl-stream server sslport))
+ 	(when sslpro
+ 	  (setq sslname (process-name sslpro))
+	  (setq lport (mew-ssl-pnm-to-lport sslname))
+ 	  (when lport
+ 	    (setq process (mew-imap-open pnm "localhost" lport no-msg)))))
        (t
 	(setq process (mew-imap-open pnm server port no-msg))))
       (if (null process)
@@ -760,7 +780,7 @@
 	    (mew-sinfo-set-refile
 	     (nconc (mew-sinfo-get-refile) (mew-sinfo-get-refile-back)))
 	    (mew-sinfo-set-refile-back nil))
-	(mew-summary-lock process "IMAPing")
+	(mew-summary-lock process "IMAPing" (or sshpro sslpro))
 	(mew-sinfo-set-scan-form (mew-summary-scan-form bnm))
 	(mew-sinfo-set-scan-id nil)
 	(mew-info-clean-up pnm)
@@ -768,6 +788,7 @@
 	(mew-imap-message pnm "Communicating with the IMAP server...")
 	(mew-imap-set-process pnm process)
 	(mew-imap-set-ssh-process pnm sshpro)
+	(mew-imap-set-ssl-process pnm sslpro)
 	(mew-imap-set-server pnm server)
 	(mew-imap-set-port pnm port)
 	(mew-imap-set-user pnm (mew-imap-user case))
@@ -838,7 +859,9 @@
        (mew-net-status2 (mew-imap-get-bnm pnm)
 			(mew-imap-get-rttl pnm)
 			(mew-imap-get-rcnt pnm)
-			(nth 1 (car (mew-imap-get-rtrs pnm)))))
+			(nth 1 (car (mew-imap-get-rtrs pnm)))
+			nil
+			(mew-imap-secure-p pnm)))
      (cond
       ((and (goto-char (point-max)) (forward-line -1) (looking-at  "\\*"))
        ;; untagged message
@@ -869,6 +892,7 @@
   (let* ((pnm (process-name process))
 	 (directive (mew-imap-get-directive pnm))
 	 (sshpro (mew-imap-get-ssh-process pnm))
+	 (sslpro (mew-imap-get-ssl-process pnm))
 	 (rttl (mew-imap-get-rttl pnm))
 	 (dttl (mew-imap-get-dttl pnm))
 	 (jcnt (mew-imap-get-jcnt pnm))
@@ -947,11 +971,14 @@
 	  (t
 	   (mew-imap-message pnm "%d message groups deleted" dttl))))))
      ;;
+     (mew-net-status-clear (mew-imap-get-bnm pnm))
      (mew-info-clean-up pnm)
      (set-buffer-modified-p nil)
      (mew-summary-unlock)
      (if (and (processp sshpro) (not mew-ssh-keep-connection))
 	 (process-send-string sshpro "exit\n"))
+     (if (and (processp sslpro) (not mew-ssl-keep-connection))
+ 	 (delete-process sslpro))
      (unless (eq directive 'biff)
        (run-hooks 'mew-imap-sentinel-non-biff-hook))
      (run-hooks 'mew-imap-sentinel-hook)
@@ -1122,7 +1149,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2002 Mew developing team.
+;; Copyright (C) 2002-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
