@@ -8,7 +8,7 @@
 (require 'mew)
 (if mew-xemacs-p (require 'easymenu))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Mode map
 ;;;
@@ -22,8 +22,9 @@
 (defvar mew-header-mode-map   nil)
 (defvar mew-addrbook-mode-map nil)
 (defvar mew-input-map         nil)
+(defvar mew-input-folder-map  nil)
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Summary mode
 ;;;
@@ -72,6 +73,7 @@
   (define-key mew-summary-mode-map "U"    'mew-summary-undo-all)
   (define-key mew-summary-mode-map "n"    'mew-summary-display-down)
   (define-key mew-summary-mode-map "p"    'mew-summary-display-up)
+  (define-key mew-summary-mode-map "M"    'mew-summary-mark-duplicated)
   (define-key mew-summary-mode-map "N"    'mew-summary-display-review-down)
   (define-key mew-summary-mode-map "P"    'mew-summary-display-review-up)
   (define-key mew-summary-mode-map "\en"  'mew-summary-find-keyword-down)
@@ -84,6 +86,7 @@
   (define-key mew-summary-mode-map "Z"    'mew-status-update)
   (define-key mew-summary-mode-map "#"    'mew-summary-print)
   (define-key mew-summary-mode-map "|"    'mew-summary-pipe-message)
+  (define-key mew-summary-mode-map "\\"   'mew-summary-cmd-msg)
   (define-key mew-summary-mode-map "q"    'mew-summary-suspend)
   (define-key mew-summary-mode-map "Q"    'mew-summary-quit)
   (define-key mew-summary-mode-map "C"    'mew-summary-set-case)
@@ -99,16 +102,17 @@
   (define-key mew-summary-mode-map "\C-c\C-r" 'mew-summary-isearch-backward)
   (define-key mew-summary-mode-map "\C-c\C-o" 
     'mew-summary-jump-to-draft-buffer)
-  (define-key mew-summary-mode-map "\ea"  'mew-summary-addrbook-edit)
+  (define-key mew-summary-mode-map "\ea"  'mew-summary-alias-edit)
   (define-key mew-summary-mode-map "\el"  'mew-summary-recenter)
   (define-key mew-summary-mode-map "\et"  'mew-summary-uudecode)
   (define-key mew-summary-mode-map "\es"  'mew-summary-unshar)
   (define-key mew-summary-mode-map "\eb"  'mew-summary-burst-multi)
+  (define-key mew-summary-mode-map "\e\\" 'mew-summary-cmd-msgs)
   (define-key mew-summary-mode-map "v"    'mew-summary-toggle-disp-msg)
   (define-key mew-summary-mode-map "\C-c\C-l" 'mew-summary-convert-local-cs)
   (define-key mew-summary-mode-map "\C-c\C-z" 'mew-summary-decode-pgp)
+  (define-key mew-summary-mode-map "\C-c\C-q" 'mew-summary-kill)
   (define-key mew-summary-mode-map "\C-c\C-x" 'mew-summary-x-face)
-  (define-key mew-summary-mode-map "\C-c\C-q" 'mew-kill-buffer)
   (define-key mew-summary-mode-map "\C-c\C-k" 'mew-summary-kill-subprocess)
   (define-key mew-summary-mode-map "^"    'mew-summary-thread-parent)
   (define-key mew-summary-mode-map "&"    'mew-summary-thread-child)
@@ -122,6 +126,8 @@
   (define-key mew-summary-mode-map "ma"   'mew-summary-mark-all)
   (define-key mew-summary-mode-map "mu"   'mew-summary-mark-undo-all)
   (define-key mew-summary-mode-map "mI"   'mew-summary-mark-retrieve-message)
+  (define-key mew-summary-mode-map "ml"	  (make-sparse-keymap))
+  (define-key mew-summary-mode-map "mlc"  'mew-summary-mark-local-copy)
   (define-key mew-summary-mode-map "V"    'mew-summary-virtual)
   (define-key mew-summary-mode-map "/"    'mew-summary-virtual)
   (define-key mew-summary-mode-map "?"    'mew-summary-search-mark)
@@ -134,10 +140,17 @@
   (define-key mew-summary-mode-map "t@"   'mew-thread-mark-multi)
   (define-key mew-summary-mode-map "td"   'mew-thread-mark-delete)
   (define-key mew-summary-mode-map "to"   'mew-thread-mark-refile)
+  (define-key mew-summary-mode-map "tc"   'mew-thread-mark-copy)
   (define-key mew-summary-mode-map "t\ed" 'mew-thread-mark-unlink)
   (define-key mew-summary-mode-map "tu"   'mew-thread-unmark)
   (define-key mew-summary-mode-map "z"    (make-sparse-keymap))
   (define-key mew-summary-mode-map "z8"   'mew-summary-toggle-8bit)
+  (define-key mew-summary-mode-map "zc"   'mew-summary-cache-clean-up)
+  (define-key mew-summary-mode-map "zd"   'mew-summary-toggle-debug)
+  (define-key mew-summary-mode-map "zp"   'mew-summary-toggle-policy)
+  (define-key mew-summary-mode-map "zw"   'mew-summary-toggle-warning)
+  (define-key mew-summary-mode-map "l"	  (make-sparse-keymap))
+  (define-key mew-summary-mode-map "lc"   'mew-summary-local-copy)
   ;;
   ;; not provided in Virtual mode
   ;;
@@ -148,8 +161,12 @@
   (define-key mew-summary-mode-map "\ed"  'mew-summary-unlink)
   (define-key mew-summary-mode-map "x"    'mew-summary-exec)
   (define-key mew-summary-mode-map "X"    'mew-summary-exec-one)
+  (define-key mew-summary-mode-map "lx"   'mew-summary-exec-offline)
+  (define-key mew-summary-mode-map "lX"   'mew-summary-exec-offline-one)
   (define-key mew-summary-mode-map "mo"   'mew-summary-mark-refile)
+  (define-key mew-summary-mode-map "mc"   'mew-summary-mark-copy)
   (define-key mew-summary-mode-map "md"   'mew-summary-mark-delete)
+  (define-key mew-summary-mode-map "m/"   'mew-summary-mark-virtual)
   (define-key mew-summary-mode-map "m\ed" 'mew-summary-mark-unlink)
   (define-key mew-summary-mode-map "mxd"  'mew-summary-exec-delete)
   (define-key mew-summary-mode-map "mx\ed" 'mew-summary-exec-unlink)
@@ -231,7 +248,7 @@
     ["Save"  mew-summary-save  t]
     ["Reply" mew-summary-reply t]))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Message mode
 ;;;
@@ -257,7 +274,7 @@
     ["Prev page"    mew-message-prev-page    t]
     ["Goto summary" mew-message-goto-summary t]))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Draft mode
 ;;;
@@ -390,7 +407,7 @@
      ["FIB fill default"  mew-fib-fill-default  (not (mew-attach-p))]
      ["FIB delete frame"  mew-fib-delete-frame  (not (mew-attach-p))])))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Header mode
 ;;;
@@ -412,7 +429,7 @@
     ["Set cases"       mew-draft-set-case      t]
     ["Kill Draft"      mew-draft-kill          t]))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Attach mode
 ;;;
@@ -449,7 +466,7 @@
   (define-key mew-draft-attach-map "\C-c\t" 'mew-draft-insert-signature)
   (mew-draft-share-keymap 'mew-draft-attach-map))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Addrbook mode
 ;;;
@@ -460,7 +477,7 @@
   (define-key mew-addrbook-mode-map "\C-c\C-c" 'mew-addrbook-register)
   (define-key mew-addrbook-mode-map "\C-c\C-q" 'mew-addrbook-kill))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Minibuffer
 ;;;
@@ -470,20 +487,41 @@
 	(if (boundp 'minibuffer-local-map)
 	    (copy-keymap minibuffer-local-map)
 	  (make-sparse-keymap)))
-  (define-key mew-input-map " "    'mew-input-complete)
-  (define-key mew-input-map "\t"   'mew-input-complete)
-  (define-key mew-input-map "?"    'mew-input-complete)
-;;  (define-key mew-input-map "\177" 'backward-delete-char)
-;;  (define-key mew-input-map [delete] 'backward-delete-char)
-;;  (define-key mew-input-map "\C-h" 'mew-complete-backscroll)
-  (define-key mew-input-map "\r"   'mew-exit-minibuffer)
-  (define-key mew-input-map "\n"   'mew-exit-minibuffer)
-  (define-key mew-input-map ","    'mew-input-comma)
-  (define-key mew-input-map "\C-g" 'abort-recursive-edit)
-  (define-key mew-input-map "\M-p" 'previous-history-element)
-  (define-key mew-input-map "\M-n" 'next-history-element))
+  (define-key mew-input-map "\r"     'mew-input-exit-minibuffer)
+  (define-key mew-input-map "\n"     'mew-input-exit-minibuffer)
+  (define-key mew-input-map [return] 'mew-input-exit-minibuffer)
+  (define-key mew-input-map ","      'mew-input-comma)
+  (define-key mew-input-map " "      'mew-input-complete)
+  (define-key mew-input-map "\t"     'mew-input-complete)
+  (define-key mew-input-map [tab]    'mew-input-complete)
+  (define-key mew-input-map "?"      'mew-input-complete)
+  (define-key mew-input-map "\C-c\t" 'mew-circular-complete-case))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+(unless mew-input-folder-map
+  (setq mew-input-folder-map (copy-keymap mew-input-map))
+  (let ((c 33)) ;; excluding SPC
+    (while (<= c 128) ;; including delete
+      (define-key mew-input-folder-map (char-to-string c)
+	'mew-input-folder-self-insert)
+      (setq c (1+ c))))
+  (define-key mew-input-folder-map [delete]    'mew-input-folder-self-insert)
+  (define-key mew-input-folder-map [backspace] 'mew-input-folder-self-insert)
+  (define-key mew-input-folder-map "\C-g"   'mew-input-folder-abort-minibuffer)
+  (define-key mew-input-folder-map "\r"     'mew-input-folder-exit-minibuffer)
+  (define-key mew-input-folder-map "\n"     'mew-input-folder-exit-minibuffer)
+  (define-key mew-input-folder-map [return] 'mew-input-folder-exit-minibuffer)
+  (define-key mew-input-folder-map "+"      'mew-input-folder-prefix)
+  (define-key mew-input-folder-map "%"      'mew-input-folder-prefix)
+  (define-key mew-input-folder-map "$"      'mew-input-folder-prefix)
+  (define-key mew-input-folder-map "-"      'mew-input-folder-prefix)
+  (define-key mew-input-folder-map "*"      'mew-input-folder-prefix)
+  (define-key mew-input-folder-map ","      'mew-input-folder-comma)
+  (define-key mew-input-folder-map "\C-s"   'mew-input-folder-search-forward)
+  (define-key mew-input-folder-map "\C-r"   'mew-input-folder-search-backward)
+  (define-key mew-input-folder-map "\C-n"   'mew-input-folder-search-forward)
+  (define-key mew-input-folder-map "\C-p"   'mew-input-folder-search-backward))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Toolbar
 ;;;
@@ -518,7 +556,7 @@
   '((mew-header-make-message . "mew-queue")
     (mew-header-send-message . "mew-send")))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Icons
 ;;;
@@ -557,7 +595,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2000-2002 Mew developing team.
+;; Copyright (C) 2000-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
