@@ -125,13 +125,14 @@
     result))
 
 (defun mew-zenkaku-katakana-region (beg end)
-  (save-restriction
-    (narrow-to-region beg end)
-    (save-excursion
-      (goto-char beg)
-      (while (re-search-forward "\\ck" nil t)
-	(let ((ch (preceding-char))
-	      (wk nil))
+  (let (ch wk)
+    (save-restriction
+      (narrow-to-region beg end)
+      (save-excursion
+	(goto-char (point-min))
+	(while (re-search-forward "\\ck" nil t)
+	  (setq ch (preceding-char))
+	  (setq wk nil)
 	  (cond
 	   ((= ch ?(I^(B)
 	    (save-excursion
@@ -162,25 +163,28 @@
 	    (progn
 	      (delete-char -1)
 	      (if mew-xemacs-p
-		  (insert (mew-make-char mew-lc-jp 37 (- wk 128)))
-		(insert (mew-make-char mew-lc-jp ?\245 wk)))))
+		  (insert (make-char mew-lc-jp 37 (- wk 128)))
+		(insert (make-char mew-lc-jp ?\245 wk)))))
 	   ((setq wk (mew-search-henkan-alist
 		      (char-to-string ch) mew-katakana-kigou-alist))
 	    (progn
 	      (delete-char -1)
 	      (if mew-xemacs-p
-		  (insert (mew-make-char mew-lc-jp 33 (- wk 128)))
-		(insert (mew-make-char mew-lc-jp ?\241 wk)))))))))))
+		  (insert (make-char mew-lc-jp 33 (- wk 128)))
+		(insert (make-char mew-lc-jp ?\241 wk)))))))))))
 
 ;;
 
 (defvar mew-fancy-thread-indent-strings ["$B(2(B" "$B(1(B" "$B(-(B" "$B!!(B"])
 
+(defvar mew-highlight-body-regex-cite
+  "^\\(\\([ \t]*\\([>:|$B!S!d!U"d!'!C(B]\\|\\w+\\(['._-]+\\w+\\)*>+\\)\\)+\\).*")
+
 (provide 'mew-lang-jp)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2002 Mew developing team.
+;; Copyright (C) 1997-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
