@@ -498,13 +498,14 @@ BEG and END."
     (save-excursion
       (goto-char (point-min))
       (while (re-search-forward regex nil t)
-	(when (looking-at "[^\n]*\r") ;; before thread info
-	  (mew-summary-mark-as mark)
-          (setq n (1+ n))
-	  (when (mew-virtual-p)
-	    (setq fld (mew-summary-folder-name))
-	    (setq msg (mew-summary-message-number))
-	    (mew-mark-alist-set alist fld msg)))
+	(unless (mew-summary-marked-p)
+	  (when (looking-at "[^\n]*\r") ;; before thread info
+	    (mew-summary-mark-as mark)
+	    (setq n (1+ n))
+	    (when (mew-virtual-p)
+	      (setq fld (mew-summary-folder-name))
+	      (setq msg (mew-summary-message-number))
+	      (mew-mark-alist-set alist fld msg))))
 	(forward-line)) ;; make search faster
       (set-buffer-modified-p nil))
     (mew-summary-mark-in-physical-alist alist mark)
