@@ -602,11 +602,12 @@
 
 (defun mew-mime-application/msoffice (prog cache begin end &optional parameter)
   (let ((doit t) file1 file2)
-    (condition-case nil
-	(require 'un-define)
-      (file-error
-       (setq doit nil)
-       (mew-elet (insert "To display this, install Mule-UCS for UTF-8.\n"))))
+    (unless mew-internal-utf-8p
+      (condition-case nil
+	  (require 'un-define)
+	(file-error
+	 (setq doit nil)
+	 (mew-elet (insert "To display this, install Mule-UCS for UTF-8.\n")))))
     (unless (mew-which-exec prog)
       (setq doit nil)
       (mew-elet (insert "To display this, install \"" prog "\".\n")))
@@ -632,11 +633,12 @@
 	   (if mew-use-old-wvhtml
 	       (call-process prog nil nil nil file1 file2)
 	     (call-process prog nil nil nil
+			   "--charset=utf-8"
 			   (concat "--targetdir=" (file-name-directory file2))
 			   file1 
 			   (file-name-nondirectory file2)))
 	   (let ((buffer-file-coding-system)) ;; to prevent the side effect
-	     (insert-file-contents file2)))
+	     (mew-insert-file-contents file2)))
 	(mew-frwlet
 	 'utf-8 mew-cs-dummy
 	 (call-process prog nil (current-buffer) nil file1)))
@@ -651,7 +653,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2002 Mew developing team.
+;; Copyright (C) 1997-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
