@@ -585,6 +585,9 @@ it is deleted automatically."
 	(mew-inherit-case case)
 	(mew-input-folder-search-multi t)
 	(mew-input-folder-refile t)
+	;; Emacs 21.1, 21.2 and 21.3 has a bug of inhibit-quit.
+	;; Set inhibit-quit to nil so that C-g can be used
+	(inhibit-quit nil)
 	default prompt init ret)
     (cond
      (singlep
