@@ -9,29 +9,32 @@
 ;; Charset
 ;;
 
+(defun mew-charset-m17n ()
+  (if (string= mew-charset-m17n mew-utf-8)
+      (if mew-internal-utf-8p
+	  mew-utf-8
+	(if (condition-case nil (require 'un-define) (file-error nil))
+	    mew-utf-8
+	  mew-iso-2022-jp-2))
+    mew-iso-2022-jp-2))
+
 (defun mew-charset-guess-string (str)
   (let ((ecsdb (mew-ecsdb-guess-string str)))
     (if ecsdb
 	(mew-cs-to-charset (mew-ecsdb-cs ecsdb)) ;; not hcs, take care
-      mew-charset-m17n)))
+      (mew-charset-m17n))))
 
 (defun mew-ecsdb-guess-string (str)
-  (let ((buf (generate-new-buffer mew-buffer-prefix))
-	ret)
-    (save-excursion
-      (set-buffer buf)
-      (mew-erase-buffer)
-      (insert-string str)
-      (setq ret (mew-ecsdb-guess-region (point-min) (point-max))))
-    (mew-remove-buffer buf)
-    ret))
+  (with-temp-buffer
+    (insert-string str)
+    (mew-ecsdb-guess-region (point-min) (point-max))))
 
 (defun mew-charset-guess-region (beg end)
   (interactive "r")
   (let ((ecsdb (mew-ecsdb-guess-region beg end))
 	ret)
     (if (null ecsdb)
-	(setq ret mew-charset-m17n)
+	(setq ret (mew-charset-m17n))
       (setq ret (mew-cs-to-charset (mew-ecsdb-cs ecsdb)))) ;; not hcs
     (if (interactive-p) (message ret)) ;; for debug
     ret))
@@ -64,15 +67,15 @@
 (defun mew-charset-sanity-check (beg end)
   "Eliminate illegal characters"
   (interactive "r")
-  (when (and (/= mew-mule-ver 0)
-	     (member mew-lc-kana (mew-find-cs-region beg end)))
-    (mew-zenkaku-katakana-region beg end)
-    (mew-warn "Converted Hankaku Kana to Zenkaku Kana!!")))
-
-
-(if (and (boundp 'current-language-environment)
-	 (string= current-language-environment "Japanese"))
-    (require 'mew-lang-jp))
+  (when (/= mew-mule-ver 0)
+    (let ((lcs (mew-find-cs-region beg end)))
+      (cond
+       ((member mew-lc-kana lcs)
+	(require 'mew-lang-jp)
+	(mew-zenkaku-katakana-region beg end))
+       ((and (memq 'latin-iso8859-1 lcs) (memq 'latin-iso8859-15 lcs)) ;; xxx
+	(require 'mew-lang-latin)
+	(mew-latin0-region beg end))))))
 
 ;;;
 ;;;
@@ -111,7 +114,7 @@
 (defun mew-cs-to-charset (cs)
   (let ((ecsdb (rassq cs mew-cs-database-for-decoding)))
     (if (null ecsdb)
-	mew-charset-m17n
+	(mew-charset-m17n)
       (mew-dcsdb-charset ecsdb))))
 
 (defun mew-charset-to-cs (charset)
@@ -131,7 +134,7 @@
 ;;
 
 (defvar mew-charset-list
-  (mapcar (function mew-dcsdb-charset) mew-cs-database-for-decoding))
+  (mapcar 'mew-dcsdb-charset mew-cs-database-for-decoding))
 
 ;;
 
@@ -145,7 +148,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1998-2002 Mew developing team.
+;; Copyright (C) 1998-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
