@@ -27,13 +27,17 @@
   (copy-coding-system 'no-conversion-unix 'mew-cs-text-lf)
   (copy-coding-system 'no-conversion-dos  'mew-cs-text-crlf)
   (copy-coding-system 'no-conversion-mac  'mew-cs-text-cr)
-  (copy-coding-system 'no-conversion-dos  'mew-cs-text-net))
+  (copy-coding-system 'no-conversion-dos  'mew-cs-text-net)
+  (defun mew-cs-raw-p (cs)
+    (memq cs '(no-conversion no-conversion-unix no-conversion-dos no-conversion-mac))))
  (t
   (define-coding-system-alias 'mew-cs-text      'raw-text)
   (define-coding-system-alias 'mew-cs-text-lf   'raw-text-unix)
   (define-coding-system-alias 'mew-cs-text-crlf 'raw-text-dos)
   (define-coding-system-alias 'mew-cs-text-cr   'raw-text-mac)
-  (define-coding-system-alias 'mew-cs-text-net  'raw-text-dos)))
+  (define-coding-system-alias 'mew-cs-text-net  'raw-text-dos)
+  (defun mew-cs-raw-p (cs)
+    (memq cs '(raw-text raw-text-unix raw-text-dos raw-text-mac)))))
 
 (defvar mew-cs-dummy          'binary)
 (defvar mew-cs-binary         'binary)
@@ -75,8 +79,11 @@
 ;; ISO-8859-14 -- Latin 8  Celtic
 ;; ISO-8859-15 -- Latin 9  New West European  (or Latin 0)
 
+(defvar mew-charset-utf-8-encoding "base64")
+(defvar mew-charset-utf-8-header-encoding "B")
+
 (defvar mew-cs-database-for-encoding
-  '(((ascii)                    nil         "7bit"             "7bit")
+  `(((ascii)                    nil         "7bit"             "7bit")
     ;; West European (Latin 1)
     ((ascii latin-iso8859-1)    iso-8859-1  "quoted-printable" "Q")
     ;; East European (Latin 2)
@@ -111,9 +118,9 @@
                               chinese-big5 "base64"           "B")
     ((ascii japanese-jisx0208 japanese-jisx0213-1 japanese-jisx0213-2)
                              iso-2022-jp-3 "7bit"             "B")
-    (nil                        utf-7      "7bit"             "Q") ;; xxx
-    (nil                        utf-8      "base64"           "B")
-    (nil                     iso-2022-jp-2 "7bit"             "B")))
+    (nil utf-7 "7bit" "Q") ;; xxx
+    (nil utf-8 ,mew-charset-utf-8-encoding ,mew-charset-utf-8-header-encoding)
+    (nil iso-2022-jp-2 "7bit" "B")))
 
 (defvar mew-cs-database-for-arg
   '((iso-2022-jp . euc-jp)
@@ -124,46 +131,58 @@
     (define-coding-system-alias 'chinese-big5 'big5))
 
 (defvar mew-cs-database-for-decoding
-  '(("us-ascii"        . nil)
-    ("iso-8859-1"      . iso-8859-1)
-    ("iso-8859-2"      . iso-8859-2)
-    ("iso-8859-3"      . iso-8859-3)
-    ("iso-8859-4"      . iso-8859-4)
-    ("iso-8859-5"      . iso-8859-5)
-    ("iso-8859-6"      . iso-8859-6)
-    ("iso-8859-7"      . iso-8859-7)
-    ("iso-8859-8"      . iso-8859-8)
-    ("iso-8859-9"      . iso-8859-9)
-    ("iso-8859-15"     . iso-8859-15)
-    ("iso-2022-cn"     . iso-2022-cn)
-    ("iso-2022-cn-ext" . iso-2022-cn-ext)
-    ("gb2312"          . cn-gb-2312) ;; should be before cn-gb
-    ("cn-gb"           . cn-gb-2312)
-    ("hz-gb-2312"      . hz-gb-2312)
-    ("big5"            . chinese-big5)
-    ("cn-big5"         . chinese-big5)
-    ("iso-2022-kr"     . iso-2022-kr)
-    ("euc-kr"          . euc-kr)
-    ("ks_c_5601-1987"  . euc-kr)
-    ("iso-2022-jp"     . iso-2022-jp)
-    ("iso-2022-jp-2"   . iso-2022-jp-2)
-    ("iso-2022-jp-3"   . iso-2022-jp-3)
-    ("euc-jp"          . euc-japan)
-    ("shift_jis"       . shift_jis)
-    ("koi8-r"          . koi8-r)
-    ("windows-1250"    . cp1250)
-    ("windows-1251"    . cp1251)
-    ("windows-1252"    . cp1252)
-    ("windows-1253"    . cp1253)
-    ("windows-1254"    . cp1254)
-    ("windows-1255"    . cp1255)
-    ("windows-1256"    . cp1256)
-    ("windows-1257"    . cp1257)
-    ("windows-1258"    . cp1258)
-    ("tis-620"         . tis-620)
-    ("iso-2022-int-1"  . iso-2022-int-1)
-    ("utf-7"           . utf-7)
-    ("utf-8"           . utf-8)))
+  `(("us-ascii"          . nil)
+    ("iso-8859-1"        . iso-8859-1)
+    ("iso-8859-2"        . iso-8859-2)
+    ("iso-8859-3"        . iso-8859-3)
+    ("iso-8859-4"        . iso-8859-4)
+    ("iso-8859-5"        . iso-8859-5)
+    ("iso-8859-6"        . iso-8859-6)
+    ("iso-8859-7"        . iso-8859-7)
+    ("iso-8859-8"        . iso-8859-8)
+    ("iso-8859-8-i"      . iso-8859-8) ;; temporary solution
+    ("iso-8859-9"        . iso-8859-9)
+    ("iso-8859-15"       . iso-8859-15)
+    ("iso-2022-cn"       . iso-2022-cn)
+    ("iso-2022-cn-ext"   . iso-2022-cn-ext)
+    ("gb2312"            . cn-gb-2312) ;; should be before cn-gb
+    ("cn-gb"             . cn-gb-2312)
+    ("hz-gb-2312"        . hz-gb-2312)
+    ("big5"              . chinese-big5)
+    ("cn-big5"           . chinese-big5)
+    ("iso-2022-kr"       . iso-2022-kr)
+    ("euc-kr"            . euc-kr)
+    ("ks_c_5601-1987"    . euc-kr)
+    ("iso-2022-jp"       . iso-2022-jp)
+    ("iso-2022-jp-2"     . iso-2022-jp-2)
+    ("iso-2022-jp-3"     . iso-2022-jp-3)
+    ("euc-jp"            . euc-japan)
+    ("shift_jis"         . shift_jis)
+    ("tis-620"           . tis-620)
+    ("koi8-r"            . koi8-r)
+    ("iso-2022-int-1"    . iso-2022-int-1)
+    ("utf-7"             . utf-7)
+    ("unicode-1-1-utf-7" . utf-7) ;; the old UTF-7 name, RFC 1642
+    ("utf-8"             . utf-8)
+    ,@(if (mew-coding-system-p 'windows-1251)
+	  '(("windows-1250" . windows-1250)
+	    ("windows-1251" . windows-1251)
+	    ("windows-1252" . windows-1252)
+	    ("windows-1253" . windows-1253)
+	    ("windows-1254" . windows-1254)
+	    ("windows-1255" . windows-1255)
+	    ("windows-1256" . windows-1256)
+	    ("windows-1257" . windows-1257)
+	    ("windows-1258" . windows-1258))
+	'(("windows-1250" . cp1250)
+	  ("windows-1251" . cp1251)
+	  ("windows-1252" . cp1252)
+	  ("windows-1253" . cp1253)
+	  ("windows-1254" . cp1254)
+	  ("windows-1255" . cp1255)
+	  ("windows-1256" . cp1256)
+	  ("windows-1257" . cp1257)
+	  ("windows-1258" . cp1258)))))
 
 ;;
 ;; CS
@@ -187,26 +206,20 @@
 (defun mew-cs-decode-string (str cs)
   (if (null cs)
       str
-    (let ((post-conv (coding-system-post-read-conversion cs))
-	  ret buf)
+    (let ((post-conv (coding-system-post-read-conversion cs)))
       (if (null post-conv)
 	  (decode-coding-string str cs)
 	;; Mule 4.1 (Emacs 20.7 + Mule-UCS-0.84) has a bug.
 	;; It applys post-conv just to a buffer, not to a string.
 	;; So, we should use a buffer.
-	(setq buf (generate-new-buffer mew-buffer-prefix))
-	(save-excursion
-	  (set-buffer buf)
-	  (mew-erase-buffer)
+	(with-temp-buffer
 	  ;; The old base64-decode-string() returns a multibyte string.
 	  ;; The new base64-decode-string() returns a unibyte string.
 	  (mew-set-buffer-multibyte (mew-multibyte-string-p str))
 	  (insert str)
 	  (or (mew-multibyte-string-p str) (mew-set-buffer-multibyte t))
 	  (decode-coding-region (point-min) (point-max) cs)
-	  (setq ret (buffer-substring (point-min) (point-max)))) ;; xxx
-	(mew-remove-buffer buf)
-	ret))))
+	  (buffer-substring (point-min) (point-max))))))) ;; xxx
 
 ;; to external
 (defun mew-cs-encode-string (str cs)
@@ -257,7 +270,7 @@
 ;;
 ;;
 
-(defun mew-substring (str width &optional cnt)
+(defun mew-substring (str width &optional cnt nopad)
   (let ((sw (if (null str) 0 (string-width str)))
 	(i 0) (w 0) cw safe-w)
     (condition-case nil
@@ -265,7 +278,9 @@
 	 ((= sw width)
 	  str)
 	 ((< sw width)
-	  (concat str (make-string (- width sw) 32)))
+          (if nopad
+              str
+            (concat str (make-string (- width sw) 32))))
 	 (t
 	  (if cnt (setq width (- width 2)))
 	  (catch 'loop
@@ -295,9 +310,8 @@
 ;; Language specific
 ;;
 
-(defvar mew-lc-kana  'katakana-jisx0201)
-(defvar mew-lc-jp    'japanese-jisx0208)
-(defalias 'mew-make-char 'make-char)
+(defvar mew-lc-kana 'katakana-jisx0201)
+(defvar mew-lc-jp   'japanese-jisx0208)
 
 ;;
 ;; Stolen from mule-cmd.el
@@ -350,7 +364,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2002 Mew developing team.
+;; Copyright (C) 1997-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
