@@ -7,6 +7,33 @@
 
 (require 'mew)
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Port DB
+;;;
+
+(defvar mew-port-db
+  '(("smtp"  .  "25")
+    ;; This is officially assigned to another service (urd) by IANA.
+    ;; IESG will never assign a port number to SMTP over SSL...
+    ("smtps" . "465")
+    ("pop3"  . "110")
+    ("pop3s" . "995")
+    ("nntp"  . "119")
+    ("nntps" . "563")
+    ("imap"  . "143")
+    ("imaps" . "993")))
+
+(defun mew-serv-to-port (serv)
+  (if (string-match "^[0-9]+$" serv)
+      serv
+    (or (cdr (assoc serv mew-port-db)) "0")))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Net folders
+;;;
+
 (defvar mew-pop-folder  "+.pop/%s@%s%%%s")
 (defvar mew-imap-folder "+.imap/%s@%s%%%s")
 (defvar mew-nntp-folder "+.nntp/%s@%s%%%s")
@@ -29,32 +56,43 @@
 ;;; Net status
 ;;;
 
-(defun mew-summary-lock (key status)
+(defun mew-summary-lock (key status &optional secure)
   (setq mew-summary-buffer-process key)
-  (setq mew-summary-buffer-process-status (format " %s" status)))
+  (setq mew-summary-buffer-process-status (format " %s" status))
+  (setq mew-summary-buffer-secure-process secure))
+  
 
 (defun mew-summary-unlock ()
   (setq mew-summary-buffer-process nil)
-  (setq mew-summary-buffer-process-status nil))
+  (setq mew-summary-buffer-process-status nil)
+  (setq mew-summary-buffer-secure-process nil))
 
-(defun mew-net-status (buf status &optional substatus)
+(defun mew-net-status-clear (buf)
+  (when mew-use-net-status
+    (save-excursion
+      (set-buffer buf)
+      (setq mew-summary-buffer-process-status nil)
+      (setq mew-summary-buffer-secure-process nil))))
+
+(defun mew-net-status (buf status &optional substatus secure)
   (when mew-use-net-status
     (save-excursion
       (set-buffer buf)
       (if substatus
 	  (setq mew-summary-buffer-process-status
 		(format " %s:%s" status substatus))
-	(setq mew-summary-buffer-process-status (format " %s" status))))))
+	(setq mew-summary-buffer-process-status (format " %s" status)))
+      (setq mew-summary-buffer-secure-process secure))))
 
-(defun mew-net-status1 (buf dttl dcnt)
+(defun mew-net-status1 (buf dttl dcnt &optional secure)
   (when mew-use-net-status
     (when (<= dcnt dttl)
       (let ((substatus (format "%d/%d" dcnt dttl)))
-	(mew-net-status buf "Deleting" substatus)))))
+	(mew-net-status buf "Deleting" substatus secure)))))
 
 (defvar mew-net-status-percent-size 51200)
 
-(defun mew-net-status2 (buf rttl rcnt siz &optional zero)
+(defun mew-net-status2 (buf rttl rcnt siz &optional zero secure)
   (when mew-use-net-status
     (when (<= rcnt rttl)
       (let ((msiz (string-to-int siz))
@@ -64,13 +102,13 @@
 		 (< msiz mew-net-status-percent-size))
 	    (progn
 	      (setq substatus (format "%d/%d" rcnt rttl))
-	      (mew-net-status buf "Retrieving" substatus))
+	      (mew-net-status buf "Retrieving" substatus secure))
 	  (if (= msiz 0) (setq msiz 1))
 	  (if (< 10000 msiz)
 	      (setq percent (/ bsiz (/ msiz 100)))
 	    (setq percent (/ (* bsiz 100) msiz)))
 	  (setq substatus (format "%d/%d:%02d%%" rcnt rttl percent))
-	  (mew-net-status buf "Retrieving" substatus))))))
+	  (mew-net-status buf "Retrieving" substatus secure))))))
 
 (defvar mew-local-status-unit 10)
 (defvar mew-local-status-threshold 100)
@@ -252,7 +290,7 @@
       (if (setq buf (get-file-buffer work)) (mew-remove-buffer buf))
       (mew-frwlet
        mew-cs-text-for-read mew-cs-dummy
-       (set-buffer (find-file-noselect work)))
+       (set-buffer (mew-find-file-noselect work)))
       (setq data (mew-lisp-load info))
       (while (< i n)
 	(aset (mew-info pnm) i (aref data i))
@@ -631,7 +669,7 @@ The messages in the server side is always retained."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2002 Mew developing team.
+;; Copyright (C) 2002-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
