@@ -738,7 +738,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 	  (setq type (mew-pgp-check-old-pgp))
 	  (unless type
 	    (with-temp-buffer
-	      (insert-file-contents file)
+	      (mew-insert-file-contents file)
 	      (setq type (mew-pgp-check-old-pgp))))
 	  (if (null type)
 	      (message "No PGP message was found")
@@ -763,7 +763,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 	    (when (file-exists-p file1)
 	      (mew-frwlet
 	       mew-cs-text-for-read mew-cs-dummy
-	       (insert-file-contents file1))
+	       (mew-insert-file-contents file1))
 	      (mew-delete-file file1))
 	    (if mew-mule-p
 		(mew-cs-decode-region (point) (point-max) mew-cs-autoconv))
