@@ -787,10 +787,12 @@ This is very convenient to refile all messages picked by '\\[mew-summary-search-
       (save-excursion
 	(save-window-excursion
 	  (goto-char (point-min))
-	  (while (re-search-forward mew-regex-msg-valid-review nil t)
-	    (setq last (mew-summary-refile-body
-			last nil 'no-msg nil (mew-summary-folder-name)))
-	    (forward-line))
+	  (catch 'loop
+	    (while (re-search-forward mew-regex-msg-valid-review nil t)
+	      (setq last (mew-summary-refile-body
+			  last nil 'no-msg nil (mew-summary-folder-name)))
+	      (unless last (throw 'loop t))
+	      (forward-line)))
 	  (message "Mark copying...done")))))))
 
 (defun mew-summary-mark-refile ()
@@ -806,9 +808,11 @@ This is very convenient to refile all messages picked by '\\[mew-summary-search-
       (save-excursion
 	(save-window-excursion
 	  (goto-char (point-min))
-	  (while (re-search-forward mew-regex-msg-valid-review nil t)
-	    (setq last (mew-summary-refile-body last nil 'no-msg))
-	    (forward-line))
+	  (catch 'loop
+	    (while (re-search-forward mew-regex-msg-valid-review nil t)
+	      (setq last (mew-summary-refile-body last nil 'no-msg))
+	      (unless last (throw 'loop t))
+	      (forward-line)))
 	  (message "Mark refiling...done")))))))
 
 (provide 'mew-refile)
