@@ -26,11 +26,13 @@
     (cond
      ((get-buffer folder)
       (switch-to-buffer folder)
+      (mew-set-buffer-cs mew-cs-m17n)
       (unless (mew-folder-virtualp folder) (mew-summary-folder-cache-load))
       (unless (string= ofolder folder) (mew-window-configure 'summary)))
      (t
       (setq new-folder t)
       (switch-to-buffer (get-buffer-create folder))
+      (mew-set-buffer-cs mew-cs-m17n)
       (mew-buffers-setup folder)
       (if (null (setq case-folder (mew-folder-case folder)))
 	  (mew-sinfo-set-folder folder)
@@ -57,7 +59,9 @@
 Summary mode."
   (interactive "P")
   (let* ((proto (mew-proto-to-go (mew-summary-folder-name 'ext)))
-	 (case (mew-sinfo-get-case)) ;; if in a remote folder
+	 (case (if (mew-folder-remotep proto)
+		   (mew-sinfo-get-case) ;; a remote folder
+		 mew-case-input)) ;; a local/virtual folder
 	 (inbox (mew-proto-inbox-folder	proto case))
 	 (folder (mew-input-folder case inbox))
 	 buf win frame goend)
@@ -120,7 +124,9 @@ Summary mode."
   "Jump to a message according to the number which you input.
 If 'mew-summary-jump-message-then-display' is non-nil,
 the message is then displayed."
-  (interactive)
+  (interactive "P")
+  (if (integerp msg) (setq msg (int-to-string msg))
+    (if (not (stringp msg)) (setq msg nil)))
   (let ((here (point)))
     (unless msg (setq msg (read-string "Message No.: " "")))
     (cond 
@@ -310,12 +316,74 @@ If called with '\\[universal-argument]', you can specify a printer name."
   (when (y-or-n-p "Print this message? ")
     (let ((printer-name (if arg
 			    (read-string "Printer name: ")
-			  printer-name)))
+			  printer-name))
+	  have-header str)
       (save-excursion
 	(set-buffer (mew-buffer-message))
 	(save-restriction
 	  (widen)
-	  (funcall mew-print-function))))))
+	  (setq have-header (mew-header-p))
+	  (setq str (mew-buffer-substring (point-min) (point-max)))))
+      (with-temp-buffer
+	(insert str)
+	(if have-header
+	    (mew-header-delete-other-lines mew-field-for-printing))
+	(funcall mew-print-function)))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; X-Face:
+;;;
+
+(defun mew-summary-x-face ()
+  "Display xface."
+  (interactive)
+  (mew-summary-msg
+   (let ((file (mew-make-temp-name)) xface)
+     (save-excursion
+       (set-buffer (mew-buffer-message))
+       (setq xface (mew-header-get-value mew-x-face:)))
+     (when xface
+       (with-temp-buffer
+	 (insert xface)
+	 (mew-x-face-compface-to-xbm)
+	 (mew-frwlet
+	  mew-cs-dummy mew-cs-text-for-write
+	  (write-region (point-min) (point-max) file nil 'no-msg)))
+       (mew-mime-start-process
+	mew-prog-image/*-ext mew-prog-image/*-ext-arg file)))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Calling a command
+;;;
+
+(defun mew-summary-cmd-msg ()
+  "Executing an external command specifying this message as an
+argument."
+  (interactive)
+  (mew-summary-msg-or-part
+   (let* ((fld (mew-summary-folder-name))
+	  (msg (mew-summary-message-number))
+	  (file (mew-expand-folder fld msg))
+	  (command (read-string "Command: ")))
+     (while (not (mew-which-exec command))
+       (setq command (read-string "Command: ")))
+     (message (format "Executing %s for %s..." command msg))
+     (call-process command nil nil nil file)
+     (message (format "Executing %s for %s...done" command msg)))))
+
+(defun mew-summary-cmd-msgs ()
+  "Executing an external command specifying messages
+marked with '@' as arguments."
+  (interactive)
+  (mew-summary-multi-msgs
+   (let ((command (read-string "Command: ")))
+     (while (not (mew-which-exec command))
+       (setq command (read-string "Command: ")))
+     (message (format "Executing %s ..." command))
+     (apply 'call-process command nil nil nil FILES)
+     (message (format "Executing %s ...done" command)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -454,40 +522,6 @@ If called with '\\[universal-argument]', options are asked."
 		    (mew-mime-start-process program options file)
 		  (mew-mime-call-process program options file)))))))))))
   
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; X-Face:
-;;;
-
-(defun mew-summary-x-face ()
-  "Display xface."
-  (interactive)
-  (mew-summary-msg
-   (let ((filters mew-x-face-filter) filter filter-arg file xface buf)
-     (save-excursion
-       (set-buffer (mew-buffer-message))
-       (when (setq xface (mew-header-get-value mew-x-face:))
-	 (setq buf (generate-new-buffer mew-buffer-prefix))
-	 (set-buffer buf)
-	 (mew-erase-buffer)
-	 (insert xface)
-	 (while filters
-	   (setq filter (car filters))
-	   (if (stringp filter)
-	       (setq filter-arg nil)
-	     (setq filter-arg (cdr filter))
-	     (setq filter (car filter)))
-	   ;; call-process-region is OK...
-	   (mew-plet
-	    (apply 'call-process-region
-		   (point-min) (point-max) filter 'delete t nil filter-arg))
-	   (setq filters (cdr filters)))
-	 (setq file (mew-make-temp-name))
-	 ;; NEVER use call-process-region for privary reasons
-	 (mew-flet
-	  (write-region (point-min) (point-max) file nil 'no-msg))
-	 (mew-mime-start-process mew-x-face-prog mew-x-face-args file))))))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Flushing the queue
@@ -597,17 +631,15 @@ the other network processes are also hung up."
     (t
      (let ((dir (mew-summary-input-directory-name))
 	   (fn nil)
-	   (buf (generate-new-buffer mew-buffer-prefix))
 	   (tmp (mew-make-temp-name))
 	   (case-fold-search nil)
 	   (files FILES) start)
-       (save-excursion
-	 (set-buffer buf)
+       (with-temp-buffer
 	 (mew-frwlet
 	  mew-cs-text-for-read mew-cs-text-for-write
 	  (while files
 	    (mew-erase-buffer)
-	    (insert-file (car files))
+	    (insert-file-contents (car files))
 	    (goto-char (point-min))
 	    (if (re-search-forward mew-eoh nil t)
 		(forward-line))
@@ -633,8 +665,7 @@ the other network processes are also hung up."
 		      (y-or-n-p (format "Execute %s for %s? " mew-prog-tar fn)))
 	     (message "Executing %s for %s..." mew-prog-tar fn)
 	     (call-process mew-prog-tar nil nil nil "-xf" fn)
-	     (message "Executing %s for %s...done" mew-prog-tar fn))))
-       (mew-remove-buffer buf))))))
+	     (message "Executing %s for %s...done" mew-prog-tar fn)))))))))
 
 (defun mew-summary-prog-exec (prog opts suffix tarfile)
   (if (string-match (format "^\\(.*\\)\\.%s$" suffix) tarfile)
@@ -662,14 +693,11 @@ message."   (interactive)
 	   (folder (mew-input-burst-folder))
 	   (tfile (mew-make-temp-name))
 	   (targets FLD-MSG-LIST)
-	   (buf (generate-new-buffer mew-buffer-prefix))
 	   fld-msg num ct med dbl vec len idx ttl TTL fid FID i ent
 	   beg regex)
        (message "Joining...")
-       (save-excursion
-	 (set-buffer buf)
+       (with-temp-buffer
 	 (mew-set-buffer-multibyte t)
-	 (mew-erase-buffer)
 	 (insert "CD: " cfld "\n")
 	 (while targets
 	   (setq fld-msg (car targets))
@@ -729,7 +757,7 @@ message."   (interactive)
 	  (goto-char (point-max))
 	  (save-restriction
 	    (narrow-to-region (point) (point))
-	    (insert-file-contents (mew-expand-folder cfld (aref vec 0)))
+	    (mew-insert-file-contents (mew-expand-folder cfld (aref vec 0)))
 	    ;; Removing unnecessary fields from the encapsulating
 	    ;; (outer) header.
 	    (goto-char (point-min))
@@ -762,7 +790,7 @@ message."   (interactive)
 	    (goto-char (point-max))
 	    (save-restriction
 	      (narrow-to-region (point) (point))
-	      (insert-file-contents (mew-expand-folder cfld (aref vec i)))
+	      (mew-insert-file-contents (mew-expand-folder cfld (aref vec i)))
 	      (goto-char (point-min))
 	      (re-search-forward mew-eoh nil t)
 	      (forward-line)
@@ -770,10 +798,9 @@ message."   (interactive)
 	    (setq i (1+ i)))
 	  (write-region (point-min) (point-max)
 			(mew-folder-new-message folder)
-			nil 'no-msg))
-	 (mew-touch-folder folder)
-	 (message "Joining...done"))
-       (mew-remove-buffer buf)
+			nil 'no-msg)))
+       (mew-touch-folder folder)
+       (message "Joining...done")
        (if (y-or-n-p (format "Go to %s? " folder))
 	   (mew-summary-visit-folder folder 'goend)))))))
 
@@ -877,17 +904,13 @@ folder."
 	 (message "Copying...done")))))))
 
 (defun mew-summary-local-copy-one (srcfile dstfile)
-  (let ((buf (generate-new-buffer mew-buffer-prefix)))
-    (save-excursion
-      (set-buffer buf)
-      (mew-erase-buffer)
-      (mew-frwlet
-       mew-cs-text-for-read mew-cs-text-for-write
-       (insert-file-contents srcfile)
-       (goto-char (point-min))
-       (mew-header-delete-lines (list mew-x-mew-uidl:))
-       (write-region (point-min) (point-max) dstfile nil 'no-msg)))
-    (mew-remove-buffer buf)))
+  (with-temp-buffer
+    (mew-frwlet
+     mew-cs-text-for-read mew-cs-text-for-write
+     (mew-insert-file-contents srcfile)
+     (goto-char (point-min))
+     (mew-header-delete-lines (list mew-x-mew-uidl:))
+     (write-region (point-min) (point-max) dstfile nil 'no-msg))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -997,7 +1020,7 @@ If 'mew-warning-field-level' is 1, set it to 2."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2002 Mew developing team.
+;; Copyright (C) 1996-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
