@@ -156,13 +156,13 @@
 (defsubst mew-syntax-cat (syntax part)
   (vconcat syntax (vector part)))
 
-(defun mew-syntax-cdp-format (ct file &optional cdp)
+(defun mew-syntax-cdp-format (ct file &optional cdp force-file)
   (let* ((cdpdb (mew-cdpdb-by-ct ct))
 	 (val (mew-cdpdb-val cdpdb))
 	 (filep (mew-cdpdb-file cdpdb)))
     (if (stringp cdp) (setq file cdp))
     (when val
-      (if (and filep file)
+      (if (and (or filep force-file) file)
 	  (list val (list "filename" (file-name-nondirectory (mew-header-sanity-check-string file))))
 	(list val)))))
 
@@ -404,7 +404,7 @@
 (defun mew-encode-syntax-single (file &optional ctl cte cd cid cdp 
 				      privacy decrypters)
   ;; cid is just for beauty
-  ;; if cdp is t, set cdp from file.
+  ;; if cdp is non-nil, set cdp from file.
   ;; if cdp is a string, set cdp from cdp.
   (let* (ct ctdb)
     (cond
@@ -440,7 +440,7 @@
   (let ((i 1) (ret))
     (while (<= i n)
       (setq ret (vconcat ret (list (mew-encode-syntax-single
-				    (int-to-string i) nil nil nil nil t))))
+				    (int-to-string i) nil nil nil nil 'cdp))))
       (setq i (1+ i)))
     (vconcat (mew-encode-syntax-multi dir mew-type-mlm)
 	     (list (mew-encode-syntax-single mew-draft-coverpage 
@@ -885,9 +885,6 @@
 (defsubst mew-encode-syntax-have-one-part ()
   (= (length mew-encode-syntax) (1+ mew-syntax-magic)))
 
-(defsubst mew-encode-syntax-have-multiple-parts ()
-  (> (length mew-encode-syntax) (1+ mew-syntax-magic)))
-
 ;; point
 
 (defsubst mew-decode-syntax-begin ()
@@ -1011,7 +1008,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2002 Mew developing team.
+;; Copyright (C) 1996-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
