@@ -26,8 +26,11 @@
 (defvar mew-mark-tmp    ?\0) ;; temporary use only.
 (defvar mew-mark-clean
   `(,mew-mark-delete ,mew-mark-unlink ,mew-mark-refile))
+(defvar mew-mark-all
+  `(,mew-mark-multi ,mew-mark-review ,mew-mark-delete ,mew-mark-unlink ,mew-mark-refile))
 (defvar mew-mark-default-walk mew-mark-review)
 (defvar mew-mark-walk         mew-mark-default-walk)
+(defvar mew-mark-duplicated   mew-mark-delete)
 (defvar mew-mark-show-list (list mew-mark-review))
 
 (defvar mew-summary-mark-undo-marks
@@ -103,9 +106,6 @@ cdr is new text. ")
 
 ;; Under mew-conf-path
 
-(defvar mew-folder-list-file ".mew-folder-list")
-(defvar mew-folder-alist-file ".mew-folder-alist")
-
 (defcustom mew-addrbook-file "Addrbook"
   "*A file which contains AddrBook information."
   :group 'mew-addrbook
@@ -118,6 +118,7 @@ cdr is new text. ")
 (defvar mew-refile-from-file ".mew-refile-from-alist")
 
 (defvar mew-smtp-log-file "Smtplog")
+(defvar mew-nntp-log-file "Nntplog")
 
 ;; xxx Home
 
@@ -129,11 +130,11 @@ cdr is new text. ")
 ;;;
 
 (defvar mew-folder-mode 448 ;; decimal for octal 0700
-  "Secure file mode for folders. 448(0700 in octal) is STRONGLY recommended
-for privacy reasons.")
+  "Secure file mode for folders. 448(0700 in octal) is STRONGLY
+recommended for privacy reasons.")
 
-(defvar mew-file-mode 384 ;; decimal for octal 0600
-  "Secure file mode. 384(0600 in octal) is STRONGLY recommended
+(defvar mew-file-mode-mask 432 ;; decimal for octal 0660
+  "Secure file mode mask. 432(0660 in octal) is STRONGLY recommended
 for privacy reasons.")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -142,6 +143,8 @@ for privacy reasons.")
 ;;;
 
 (defvar mew-path-separator "/")
+(defvar mew-regex-drive-letter (file-name-as-directory "^[a-zA-Z]:"))
+(defvar mew-regex-file-absolute (format "^[~%s]" mew-path-separator))
 (defvar mew-header-separator "----")
 (defvar mew-eoh nil) ;; set by mew-regex-setup
 (defvar mew-lwsp "^[ \t]")
@@ -186,16 +189,19 @@ for privacy reasons.")
 ;;;
 
 (defsubst mew-ct-textp (ct)
-  (string-match "^Text/" (capitalize ct)))
+  (string-match "^Text" (capitalize ct)))
 
 (defsubst mew-ct-imagep (ct)
-  (string-match "^Image/" (capitalize ct)))
+  (string-match "^Image" (capitalize ct)))
+
+(defsubst mew-ct-modelp (ct)
+  (string-match "^Model" (capitalize ct)))
 
 (defsubst mew-ct-multipartp (ct)
-  (string-match "^Multipart/" (capitalize ct)))
+  (string-match "^Multipart" (capitalize ct)))
 
 (defsubst mew-ct-messagep (ct)
-  (string-match "^Message/" (capitalize ct)))
+  (string-match "^Message" (capitalize ct)))
 
 (defsubst mew-ct-linebasep (ct)
   (or (mew-ct-textp ct)
@@ -216,17 +222,18 @@ in draft buffer.")
   '(alist key-type: string :value-type function))
 
 (defcustom mew-field-completion-switch
-  '(("To:"         . mew-complete-address)
-    ("Cc:"         . mew-complete-address)
-    ("Dcc:"        . mew-complete-address)
-    ("Bcc:"        . mew-complete-address)
-    ("Reply-To:"   . mew-complete-address)
-    ("Fcc:"        . mew-complete-folder)
-    ("Resent-To:"  . mew-complete-address)
-    ("Resent-Cc:"  . mew-complete-address)
-    ("Resent-Dcc:" . mew-complete-address)
-    ("Resent-Bcc:" . mew-complete-address)
-    ("Newsgroups:" . mew-complete-newsgroups))
+  '(("To:"          . mew-complete-address)
+    ("Cc:"          . mew-complete-address)
+    ("Dcc:"         . mew-complete-address)
+    ("Bcc:"         . mew-complete-address)
+    ("Reply-To:"    . mew-complete-address)
+    ("Fcc:"         . mew-complete-local-folder)
+    ("Resent-To:"   . mew-complete-address)
+    ("Resent-Cc:"   . mew-complete-address)
+    ("Resent-Dcc:"  . mew-complete-address)
+    ("Resent-Bcc:"  . mew-complete-address)
+    ("Newsgroups:"  . mew-complete-newsgroups)
+    ("Followup-To:" . mew-complete-newsgroups))
   "*Completion function alist concerned with the key."
   :group 'mew-complete
   :type mew-custom-type-of-field-completion)
@@ -273,7 +280,7 @@ in draft buffer.")
   "*The default Content-Type: for a file whose suffix is unknown.")
 
 (defvar mew-mime-content-type
-  '(("multipart/"  nil         nil     nil            mew-icon-multipart)
+  '(("multipart"    nil         nil     nil            mew-icon-multipart)
     ;;
     ("audio/basic"  "\\.au$"    mew-b64 mew-prog-audio mew-icon-audio)
     ("audio/x-wav"  "\\.wav$"   mew-b64 mew-prog-audio mew-icon-audio)
@@ -291,7 +298,12 @@ in draft buffer.")
     ("image/x-bmp" "\\.bmp$"   mew-b64 mew-prog-image mew-icon-image)
     ("image/x-pcx" "\\.pcx$"   mew-b64 mew-prog-image mew-icon-image)
     ("image/x-tga" "\\.tga$"   mew-b64 mew-prog-image mew-icon-image)
-    ("image/"      "^$"        mew-b64 mew-prog-image mew-icon-image)
+    ("image"       "^$"        mew-b64 mew-prog-image mew-icon-image)
+    ;;
+    ("model/iges" "\\.ige?s$" mew-b64 mew-prog-iges  mew-icon-image) ;; xxx
+    ("model/vrml" "\\.wrl$"   mew-b64 mew-prog-vrml  mew-icon-image)
+    ("model/mesh" "\\.me?sh$" mew-b64 mew-prog-mesh  mew-icon-image)
+    ("model"      "^$"        mew-b64 mew-prog-model mew-icon-image)
     ;;
     ("video/mpeg"      "\\.mpe?g$" mew-b64 mew-prog-video mew-icon-video)
     ("video/quicktime" "\\.mov$"   mew-b64 mew-prog-video mew-icon-video)
@@ -320,8 +332,14 @@ in draft buffer.")
      mew-prog-pgp-keys               mew-icon-unknown)
     ("application/x-pkcs7-signature" "\\.p7s$"  mew-b64
      nil			     mew-icon-unknown)
+    ("application/vnd.fujitsu.oasys" "\\.oas$"  mew-b64
+     mew-prog-oasys                  mew-icon-text)
+    ("application/vnd.fujitsu.oasys2" "\\.oa2$" mew-b64
+     mew-prog-oasys                  mew-icon-text)
+    ("application/vnd.fujitsu.oasys3" "\\.oa3$" mew-b64
+     mew-prog-oasys                  mew-icon-text)
     ("application/octet-stream"
-     "\\.tar$\\|\\.tar\\.\\|\\.gz$\\|\\.Z$\\|\\.taz$\\|\\.tgz$\\|\\.bz2?$\\|\\.lzh$\\|\\.zip$\\|\\.bin$\\|\\.pgp$\\|\\.gpg$\\|\\.exe$\\|\\.dll$"
+     "\\.tar$\\|\\.tar\\.\\|\\.gz$\\|\\.Z$\\|\\.taz$\\|\\.tgz$\\|\\.tbz$\\|\\.bz2?$\\|\\.lzh$\\|\\.zip$\\|\\.bin$\\|\\.pgp$\\|\\.gpg$\\|\\.exe$\\|\\.dll$"
       mew-b64 mew-prog-octet-stream mew-icon-application/octet-stream)
     ;;
     ("text/html"     "\\.html?$" nil     mew-prog-html      mew-icon-text)
@@ -331,7 +349,9 @@ in draft buffer.")
     ("text/plain"    "\\.txt$\\|\\.c$\\|\\.h$\\|\\.el$\\|\\.diff$\\|\\.patch$"
                                  nil     mew-prog-plain     mew-icon-text)
     ("text/xml"      "\\.xml$"   nil     mew-prog-xml       mew-icon-text)
-    ("text/"         "^$"        nil     mew-prog-text      mew-icon-text)
+    ("text/rfc822-headers" "\\.hdr$" nil
+     mew-prog-rfc822-headers     mew-icon-message/rfc822)
+    ("text"         "^$"        nil     mew-prog-text      mew-icon-text)
     ;; Unkown CT: matches here.
     (t               "^$"       nil   mew-prog-octet-stream mew-icon-unknown)
     ;; Unknown suffix matches here and return the entry specified 
@@ -383,11 +403,11 @@ in draft buffer.")
 ;;;
 
 (defvar mew-mime-content-disposition
-  '(("text/"      "inline"     t)
-    ("image/"     "inline"     t)
-    ("message/"   "inline"     nil)
-    ("multipart/" nil          nil)
-    (t            "attachment" t))
+  '(("text"      "inline"     t)
+    ("image"     "inline"     t)
+    ("message"   "inline"     nil)
+    ("multipart" nil          nil)
+    (t           "attachment" t))
   "(content-type inline/attachment filename)")
 
 (defsubst mew-cdpdb-by-ct (ct)
@@ -422,7 +442,7 @@ in draft buffer.")
   :type 'sexp)
 
 (defsubst mew-pcdb-services ()
-  (mapcar (function car) mew-privacy-database))
+  (mapcar 'car mew-privacy-database))
 
 (defsubst mew-pcdb-by-service (service)
   (assq service mew-privacy-database))
@@ -476,8 +496,11 @@ in draft buffer.")
     (,mew-subj:          text       text)
     (,mew-keywords:      comma-text text)
     (,mew-received:      unstruct   unstruct)
+    (,mew-message-id:    unstruct   unstruct)
     (,mew-references:    unstruct   unstruct)
-    (,mew-x-face:	 unstruct   unstruct))
+    (,mew-in-reply-to:   unstruct   unstruct)
+    (,mew-x-face:	 unstruct   unstruct)
+    (,mew-face:          unstruct   unstruct))
   "(field enc dec)")
 
 (defsubst mew-field-type-for-encoding (key)
@@ -607,7 +630,7 @@ in draft buffer.")
     ("^\\(To\\|Apparently-To\\):$" t
      mew-face-header-important
      mew-face-header-to)
-    ("^\\(Cc\\|Bcc\\):$" t
+    ("^\\(Cc\\|Dcc\\|Bcc\\):$" t
      mew-face-header-important
      mew-face-header-to)
     ("^Newsgroups:$" t
@@ -622,6 +645,8 @@ in draft buffer.")
      mew-face-header-important
      mew-face-header-xmew)
     ("^\\(Received\\|Return-Path\\|Sender\\|Errors-To\\):$" nil)
+    ("^\\(Path\\|Distribution\\|Xref\\):$" nil)
+    ("^NNTP-Posting-" nil)
     ("^\\(Message-Id\\|Posted\\|In-Reply-To\\|References\\|Precedence\\):$" nil)
     ("^Delivered-" nil)
     ("^List-" nil) ;; RFC 2369
@@ -629,6 +654,9 @@ in draft buffer.")
     ("^\\(Mime-Version\\|Lines\\):$" nil)
     ("^From$" nil)
     ("^Status:$" nil)
+    ("^Face:$" nil
+     mew-face-header-private
+     mew-face-header-marginal)
     ("^X-" nil
      mew-face-header-private
      mew-face-header-marginal))
@@ -671,7 +699,7 @@ present, mew-face-header-marginal is used."
 (defvar mew-highlight-body-regex-comment "^#+.*")
 
 (defvar mew-highlight-body-regex-cite
-  "^\\(\\([ \t]*\\w*[A-Za-z0-9'-.]*[>|]+\\)+\\).*")
+  "^\\(\\([ \t]*\\([>:|]\\|\\w+\\(['._-]+\\w+\\)*>+\\)\\)+\\).*")
 
 (defcustom mew-highlight-body-prefix-width 20
   "*Maximum string width assume prefix for fancy highlight a body."
@@ -701,7 +729,7 @@ present, mew-face-header-marginal is used."
 (defsubst mew-highlight-mark-get-face (mark)
   (cdr (assoc mark mew-highlight-mark-keywords)))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Mark spec
 ;;;
@@ -747,12 +775,12 @@ For more detail, see mew-mark-put-mark and mew-mark-afterstep.")
   (nth case (assoc mark mew-mark-afterstep-spec)))
 
 (defvar mew-mark-spec
-  `((,mew-mark-review "review" 0 nil nil nil nil nil)
+  `((,mew-mark-review "review" 0 nil nil nil nil nil )
     (,mew-mark-multi  "multi"  0 nil nil nil nil nil)
     (,mew-mark-delete "delete" 1 nil t   nil mew-mark-exec-delete nil)
     (,mew-mark-unlink "unlink" 1 nil t   nil mew-mark-exec-unlink nil)
-    (,mew-mark-refile "refile" 1 t   t   mew-mark-unrefile
-		      mew-mark-exec-refile mew-mark-sanity-refile))
+    (,mew-mark-refile "refile" 1 t   mew-mark-kill-refile mew-mark-unrefile
+		                     mew-mark-exec-refile mew-mark-sanity-refile))
   "*A list of lists which consists of
 mark, name, level, statefullp, kill-line-p, 
 undo-func, exec-func, and sanity-fucn.")
@@ -761,7 +789,7 @@ undo-func, exec-func, and sanity-fucn.")
 
 (defun mew-mark-get-all-marks ()
   "Collecting all defined marks."
-  (mapcar (function car) mew-mark-spec))
+  (mapcar 'car mew-mark-spec))
 
 (defsubst mew-markdb-by-mark (mark)
   (assoc mark mew-mark-spec))
@@ -776,7 +804,7 @@ undo-func, exec-func, and sanity-fucn.")
   (nth 3 (mew-markdb-by-mark mark)))
 
 (defsubst mew-markdb-killp (mark)
-  (nth 6 (mew-markdb-by-mark mark)))
+  (nth 4 (mew-markdb-by-mark mark)))
 
 (defsubst mew-markdb-func-undo (mark)
   (nth 5 (mew-markdb-by-mark mark)))
@@ -794,8 +822,8 @@ undo-func, exec-func, and sanity-fucn.")
 ;;;
 
 (defvar mew-ext-url-alist
-  '(("^application/" "Fetch by w3" mew-ext-url-fetch-by-w3 nil)
-    (t		     "Browse by netscape" "netscape" ("-install")))
+  '(("^application" "Fetch by w3" mew-ext-url-fetch-by-w3 nil)
+    (t		    "Browse by mozilla" "mozilla" ("-install")))
   "*Alist of (REGEXP DOC PROGRAM ARGS-LIST) to define
 an appropriate method for a content-type in external-body. Note
 this phantom body is defined RFC 2017.
@@ -807,11 +835,11 @@ If PROGRAM is a symbol, the lisp function whose name is PROGRAM is called.
 'mew-ext-url-show-by-w3' and 'mew-ext-url-fetch-by-w3' are pre-defined
 as lisp function.
 
-If you want to use \"w3.el\" instead of \"netscape\", put the 
+If you want to use \"w3.el\" instead of \"mozilla\", put the 
 following in .emacs.
 	(setq mew-ext-url-alist
               '((t (\"Browse by w3\" mew-ext-url-show-by-w3 nil))))
-If you want to use \"lynx\" instead of \"netscape\", put the 
+If you want to use \"lynx\" instead of \"mozilla\", put the 
 following in .emacs.
         (setq mew-ext-url-alist
               '((t (\"Browse by lynx\" \"kterm\" (\"-e\" \"lynx\" \"-color\")))))
@@ -829,6 +857,74 @@ following in .emacs.
 (defsubst mew-ext-url-get-args (prog-list)
   (nth 3 prog-list))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Proto
+;;;
+
+(defvar mew-proto mew-folder-local)
+
+(defvar mew-proto-spec
+  ;; key go refile flush
+  '(("+" "+" "+" "+")
+    ("$" "+" nil "+")
+    ("%" "%" "%" "+")
+    ("*" nil nil nil)
+    ("-" "-" nil "-")))
+
+(defun mew-proto-to-body (fld n)
+  (if (mew-folder-virtualp fld)
+      (setq fld (mew-thread-to-folder fld)))
+  (setq fld (mew-folder-folder fld))
+  (let ((proto (mew-folder-prefix fld)))
+    (unless (member proto mew-folder-prefixes)
+      (setq proto (mew-proto))) ;; default proto
+    (nth n (assoc proto mew-proto-spec))))
+
+(defun mew-proto-to-go (fld)
+  (mew-proto-to-body fld 1))
+
+(defun mew-proto-to-refile (fld)
+  (mew-proto-to-body fld 2))
+
+(defun mew-proto-to-flush (fld)
+  (mew-proto-to-body fld 3))
+
+(defun mew-proto-inbox-folder (proto &optional case)
+  (or proto (setq proto (mew-proto case)))
+  (cond
+   ((mew-folder-popp  proto) mew-pop-inbox-folder)
+   ((mew-folder-imapp proto) mew-imap-inbox-folder)
+   ((mew-folder-nntpp proto) (mew-nntp-newsgroup case))
+   (t ;; local 
+    (mew-inbox-folder case))))
+
+(defun mew-proto-queue-folder (proto &optional case)
+  (or proto (setq proto (mew-proto case)))
+  (cond
+   ((mew-folder-nntpp proto) (mew-postq-folder case))
+   (t (mew-queue-folder case))))
+
+(defun mew-proto-friend-folder (proto &optional case)
+  (cond
+   ((mew-folder-imapp proto) (mew-imap-friend-folder case))
+   (t ;; local 
+    mew-friend-folder)))
+
+(defun mew-proto-friend-folder-list (proto &optional case)
+  (cond
+   ((mew-folder-imapp proto) (mew-imap-friend-folder-list case))
+   (t ;; local 
+    (mew-local-friend-folder-list))))
+
+(defun mew-proto-folder-alist (proto &optional case)
+  (cond
+   ((mew-folder-popp  proto) (mew-pop-folder-alist))
+   ((mew-folder-imapp proto) (mew-imap-folder-alist case))
+   ((mew-folder-nntpp proto) (mew-nntp-folder-alist case))
+   (t ;; local 
+    (mew-local-folder-alist))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Config
@@ -843,21 +939,31 @@ following in .emacs.
 
 Currently, the following keys are supported: 
 \"name\", \"user\", \"mail-domain\", 
-\"smtp-msgid-domain\", \"smtp-msgid-user\", 
 \"cc\", \"fcc\", \"dcc\", \"reply-to\",
 \"organization\", \"header-alist\", 
 \"smtp-server\", \"smtp-port\", \"smtp-ssh-server\", 
+\"smtp-ssl\", \"smtp-ssl-port\", 
+\"smtp-msgid-user\", \"smtp-msgid-domain\", 
 \"smtp-helo-domain\", \"smtp-mail-from\", 
 \"smtp-user\", \"smtp-auth\", \"smtp-auth-list\", 
 \"pop-server\", \"pop-port\", \"pop-ssh-server\", 
+\"pop-ssl\", \"pop-ssl-port\", 
 \"pop-user\", \"pop-auth\", 
-\"pop-size\", \"pop-body-lines\", \"pop-delete\", 
+\"pop-size\", \"pop-body-lines\", \"pop-delete\", \"pop-folder\", 
+\"imap-server\", \"imap-port\", \"imap-ssh-server\", 
+\"imap-ssl\", \"imap-ssl-port\", 
+\"imap-user\", \"imap-auth\", \"imap-size\", \"imap-folder\", 
+\"nntp-server\", \"nntp-port\", \"nntp-ssh-server\", 
+\"nntp-ssl\", \"nntp-ssl-port\", 
+\"nntp-user\", \"nntp-size\", \"nntp-folder\",
+\"nntp-msgid-user\", \"nntp-msgid-domain\", 
 \"inbox-folder\", \"queue-folder\",
 \"mailbox-type\", \"mbox-command\", \"mbox-command-arg\", 
-\"nntp-server\", \"signature-file\", \"content-type\" .
+\"signature-file\", \"content-type\" .
 
 from = name <user@mail-domain>
-message-id = *random*.user@smtp-msgid-domain
+message-id = *random*.smtp-msgid-user@smtp-msgid-domain
+message-id = *random*.nntp-msgid-user@nntp-msgid-domain
 
 An example is as follows:
 
@@ -886,7 +992,7 @@ An example is as follows:
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2000-2002 Mew developing team.
+;; Copyright (C) 2000-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
