@@ -15,7 +15,7 @@
 ;;
 
 ;;; for PRINTING
-;;    mew-w32-prog-print     --- print command
+;;    mew-w32-prog-print     ---  print command
 ;;    mew-w32-prog-print-arg ---  print command argument
 ;;    mew-w32-cs-print       ---  coding-system for printing
 ;;    define-process-argument-editing  ---  for argument-editing
@@ -28,7 +28,7 @@
 ;;   (define-process-argument-editing "/notepad\\.exe$"
 ;;     (lambda (x)
 ;;       (general-process-argument-editing-function x nil t)))
-;;   (setq mew-print-function (function mew-w32-print-buffer))
+;;   (setq mew-print-function 'mew-w32-print-buffer)
 
 ;; Win32 programs.
 (defvar mew-w32-exec           "fiber.exe")
@@ -48,7 +48,7 @@
       (progn
 	(require 'mw32script)
 	(mw32script-init))
-    (if (require 'mw32script nil t) ;; NTEmacs
+    (if (condition-case nil (require 'mw32script) (file-error nil)) ;; NTEmacs
 	(progn
 	  (mw32script-make-pathext-regexp)
 	  (defalias 'call-process-original 'call-process)
@@ -60,10 +60,10 @@
 		    (setq prog (mew-which-exec PROGRAM))
 		    (setq sargs (mw32script-resolve-script prog))))
 	      (if sargs
-		  (apply (function call-process-original)
+		  (apply 'call-process-original
 			 (car sargs) INFILE BUFFER DISPLAY
 			 prog PROGRAM-ARGS)
-		(apply (function call-process-original)
+		(apply 'call-process-original
 		       PROGRAM INFILE BUFFER DISPLAY PROGRAM-ARGS))))
 	  (defalias 'start-process-original 'start-process)
 	  (defun start-process (NAME BUFFER PROGRAM &rest PROGRAM-ARGS)
@@ -74,9 +74,9 @@
 		    (setq prog (mew-which-exec PROGRAM))
 		    (setq sargs (mw32script-resolve-script prog))))
 	      (if sargs
-		  (apply (function start-process-original)
+		  (apply 'start-process-original
 			 NAME BUFFER (car sargs) prog PROGRAM-ARGS)
-		(apply (function start-process-original)
+		(apply 'start-process-original
 		       NAME BUFFER PROGRAM PROGRAM-ARGS))))))
     )))
 
@@ -88,18 +88,18 @@
 ;; printing
 (defun mew-w32-print-buffer ()
   (interactive)
-  (setq tempfile (mew-make-temp-name))
-  (mew-frwlet
-   mew-cs-dummy mew-w32-cs-print
-   (write-region (point-min) (point-max) tempfile nil nil))
-  (setq w32-start-process-show-window t)
-  (cond
-   ((eq mew-w32-prog-print-arg nil)
-    (call-process mew-w32-prog-print nil nil nil tempfile))
-   (t
-    (call-process mew-w32-prog-print nil nil nil mew-w32-prog-print-arg tempfile)))
-  (setq w32-start-process-show-window nil)
-  (delete-file tempfile))
+  (let ((tempfile (mew-make-temp-name)))
+    (mew-frwlet
+     mew-cs-dummy mew-w32-cs-print
+     (write-region (point-min) (point-max) tempfile nil nil))
+    (setq w32-start-process-show-window t)
+    (cond
+     ((eq mew-w32-prog-print-arg nil)
+      (call-process mew-w32-prog-print nil nil nil tempfile))
+     (t
+      (call-process mew-w32-prog-print nil nil nil mew-w32-prog-print-arg tempfile)))
+    (setq w32-start-process-show-window nil)
+    (mew-delete-file tempfile)))
 
 
 ;; MIME setting
@@ -109,22 +109,27 @@
 (defvar mew-prog-xml      '(mew-mime-text/xml  mew-mime-text/xml-ext))
 (defvar mew-prog-enriched 'mew-mime-text/enriched)
 (defvar mew-prog-text     'mew-mime-text/plain)
+(defvar mew-prog-audio           `(,mew-w32-exec () t))
+(defvar mew-prog-audio2          `(,mew-w32-exec () t)) ;; dummy
 (defvar mew-prog-image           '(mew-mime-image/* mew-mime-image/*-ext))
-(defvar mew-prog-audio           `(,mew-w32-exec           () t))
-(defvar mew-prog-audio2          `(,mew-w32-exec           () t)) ;; dummy
-(defvar mew-prog-video           `(,mew-w32-exec           () t))
+(defvar mew-prog-iges            `(,mew-w32-exec () t))
+(defvar mew-prog-vrml            `(,mew-w32-exec () t))
+(defvar mew-prog-mesh            `(,mew-w32-exec () t))
+(defvar mew-prog-video           `(,mew-w32-exec () t))
 (defvar mew-prog-rfc822          'mew-mime-message/rfc822)
+(defvar mew-prog-rfc822-headers  'mew-mime-text/rfc822-headers)
 (defvar mew-prog-external-body   '(mew-mime-external-body mew-mime-external-body-ext))
 (defvar mew-prog-delivery-status 'mew-mime-text/plain)
-(defvar mew-prog-postscript      `(,mew-w32-exec           () t))
+(defvar mew-prog-postscript      `(,mew-w32-exec () t))
 (defvar mew-prog-pgp-keys        '(mew-mime-pgp-keys mew-mime-pgp-keys-ext))
-(defvar mew-prog-pdf             `(,mew-w32-exec           () t))
+(defvar mew-prog-pdf             `(,mew-w32-exec () t))
 (defvar mew-prog-xml2            '(mew-mime-application/xml
 				   mew-mime-application/xml-ext))
-(defvar mew-prog-octet-stream    `(,mew-w32-exec           () t))
-(defvar mew-prog-msword          `(,mew-w32-exec           () t))
-(defvar mew-prog-msexcel         `(,mew-w32-exec           () t))
-(defvar mew-prog-mspowerpoint    `(,mew-w32-exec           () t))
+(defvar mew-prog-oasys           `(,mew-w32-exec () t))
+(defvar mew-prog-octet-stream    `(,mew-w32-exec () t))
+(defvar mew-prog-msword          `(,mew-w32-exec () t))
+(defvar mew-prog-msexcel         `(,mew-w32-exec () t))
+(defvar mew-prog-mspowerpoint    `(,mew-w32-exec () t))
 
 ;;;
 ;;; Text/Html, Application/Xml, Image
@@ -153,23 +158,20 @@
 (defvar mew-prog-application/msexcel nil)
 (defvar mew-prog-application/mspowerpoint nil)
 
-(cond
- ((eq mew-mule-ver 2)
-  (setq mew-cs-database-for-arg
-	'((*iso-2022-jp* . *sjis*unix)
-	  (*iso-2022-kr* . *euc-korea*unix))))
- ((eq mew-mule-ver 3)
-  (setq mew-cs-database-for-arg
-	'((iso-2022-jp . shift_jis-unix)
-	  (iso-2022-kr . euc-kr-unix)))))
+(if (eq mew-mule-ver 3)
+    (setq mew-cs-database-for-arg
+	  '((iso-2022-jp . shift_jis-unix)
+	    (iso-2022-kr . euc-kr-unix))))
 
 (setq mew-prog-grep-max-msgs 2000) ;; for external grep (pick & virtual)
 
+(defvar mew-dir-list-function 'mew-dir-list-without-link-count)
+
 (provide 'mew-win32)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2002 Mew developing team.
+;; Copyright (C) 1996-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
