@@ -5,7 +5,11 @@
 
 ;;; Code:
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+(defface mew-bitmap '((t (:background "white" :foreground "black")))
+  "Face for bitmap."
+  :group 'mew-highlight)
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Basic functions
 ;;;
@@ -42,7 +46,7 @@
       (if s (setq ret (cons (vector icon (aref s 1) (aref s 2) (aref s 0)) ret))))
     (nreverse ret)))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Toolbar and button setting
 ;;;
@@ -71,7 +75,7 @@
   (define-key toolbar-map 'button3   'pressed-and-activate-toolbar-button)
   (define-key toolbar-map 'button3up 'release-and-activate-toolbar-button)))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Button functions
 ;;;
@@ -136,7 +140,7 @@ If event is button 3, show a menu."
     (if (and image-extent (glyphp image-extent))
 	(mew-overlay-delete image-extent))
     (when (string-match "(\\(.*\\))" str)
-      (setq ct (mew-match 1 str))
+      (setq ct (match-string 1 str))
       (if (mew-ct-imagep ct)
 	  (let* ((subdir (mew-attach-expand-path mew-encode-syntax nums))
 		 (syntax (mew-syntax-get-entry mew-encode-syntax nums))
@@ -171,7 +175,7 @@ If event is button 3, show a menu."
 		  (message "Loading image...")
 		  (setq glyph (make-glyph (vector format :file file)))
 		  (if (eq format 'xbm)
-		      (set-glyph-property glyph 'face 'x-face))
+		      (set-glyph-property glyph 'face 'mew-bitmap))
 		  (set-extent-begin-glyph
 		   (mew-overlay-make (point-max) (point-max)) glyph)
 		  (message "Loading image...done"))
@@ -203,7 +207,7 @@ popup menu can be implemented."
 	(setq toolbar-active t)
 	(set-toolbar-button-down-flag button t)))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Toolbar functions
 ;;;
@@ -253,7 +257,7 @@ popup menu can be implemented."
       (set-specifier default-toolbar
 		     (cons (current-buffer) mew-header-toolbar))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; End of messages
 ;;;
@@ -278,7 +282,7 @@ popup menu can be implemented."
 (defsubst mew-message-set-end-of-nil ()
   (overlay-put (mew-minfo-get-overlay) 'begin-glyph nil))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Image
 ;;;
@@ -286,33 +290,25 @@ popup menu can be implemented."
 (defun mew-image-inline-p (format)
   ;; XEmacs checks window-system by itself
   (valid-image-instantiator-format-p format))
-  
+
 (defun mew-mime-image (cache begin end format)
   (message "Loading image...")
   (cond 
    ((eq format 'xbm)
-    ;; XEmacs depends on libX when it converts XBM to bitmap.
-    ;; The convert function can handle a file only, not a buffer.
-    ;; So, we have to use a temporary file.
-    (let ((tfile (mew-make-temp-name))
-	  glyph)
+    (let (glyph)
       (save-excursion
 	(set-buffer cache)
-	(write-region begin end tfile nil 'no-msg)
-	(set-buffer (mew-buffer-message))
-	(mew-elet
-	 (setq glyph (make-glyph (vector 
-				  'xbm
-				  :file
-				  tfile)))
-	 (set-glyph-property glyph 'face 'x-face)
-	 (set-extent-end-glyph
-	  (mew-overlay-make (point-min) (point-min)) glyph)
-	 (if (file-exists-p tfile)
-	     (delete-file tfile))))))
+	(setq glyph (mew-xemacs-make-xbm-glyph begin end)))
+      (set-buffer (mew-buffer-message))
+      (mew-elet
+       (set-extent-end-glyph
+	(mew-overlay-make (point-min) (point-min)) glyph))))
    ((eq format 'gif)
-    (set-buffer (mew-buffer-message))
-    (let ((glyph (mew-xemacs-make-gif-glyph cache begin end)))
+    (let (glyph)
+      (save-excursion
+	(set-buffer cache)
+	(setq glyph (mew-xemacs-make-gif-glyph begin end)))
+      (set-buffer (mew-buffer-message))
       (mew-elet
        (if glyph
 	   (set-extent-end-glyph
@@ -333,21 +329,29 @@ popup menu can be implemented."
 					   begin end))))))))
   (message "Loading image...done"))
 
+;; XEmacs depends on libX when it converts XBM to glyph.
+;; The convert function can handle a file only, not a buffer.
+;; So, we have to use a temporary file.
+
+(defun mew-xemacs-make-xbm-glyph (beg end)  
+  (let ((tfile (mew-make-temp-name)) glyph)
+    (mew-flet
+     (write-region beg end tfile nil 'no-msg)
+     (setq glyph (make-glyph (vector 'xbm :file tfile)))
+     (set-glyph-property glyph 'face 'mew-bitmap)
+     (mew-delete-file tfile)
+     glyph)))
+
 ;; Fix Animation GIF and Interlaced GIF
-;; This code is stolen form w3m-xmas.el.
-(defun mew-xemacs-make-gif-glyph (cache begin end)
-  (let ((prog (mew-which-exec "gifsicle"))
-	(image (save-excursion
-		 (set-buffer cache)
-		 (mew-buffer-substring begin end)))
-	should-be-non-interlaced should-unoptimize size1 size2 tmpbuf)
-    (if (null prog)
-	nil
-      (setq tmpbuf (generate-new-buffer mew-buffer-prefix))
-      (save-excursion
-	(set-buffer tmpbuf)
-	(mew-erase-buffer)
-	(mew-set-buffer-multibyte nil)
+
+(defvar mew-prog-gifsicle "gifsicle")
+
+(defun mew-xemacs-make-gif-glyph (beg end)
+  (let ((prog (mew-which-exec mew-prog-gifsicle))
+	(image (mew-buffer-substring beg end))
+	should-be-non-interlaced should-unoptimize size1 size2)
+    (when prog
+      (with-temp-buffer
 	(mew-plet
 	 (insert image)
 	 (goto-char (point-min))
@@ -365,15 +369,15 @@ popup menu can be implemented."
 	   ;; Check whether a `data' is optimized or larger than
 	   ;; the value of `w3m-animated-gif-maximum-size'.
 	   (when (looking-at ".+ \\([0-9]+\\) images\r?$")
-	     (setq size1 (string-to-number (match-string 1)))
+	     (setq size1 (string-to-number (mew-match-string 1)))
 	     (forward-line 1)
 	     (unless (and (looking-at ".+ \\([0-9]+\\)x\\([0-9]+\\)\r?$")
 			  (natnump (setq size1 (* size1
 						  (string-to-number
-						   (match-string 1)))))
+						   (mew-match-string 1)))))
 			  (natnump (setq size1 (* size1
 						  (string-to-number
-						   (match-string 2))))))
+						   (mew-match-string 2))))))
 	       ;; It should be truncated to be only one frame.
 	       (setq should-unoptimize "#0"))
 	     (setq size1 nil)
@@ -382,11 +386,11 @@ popup menu can be implemented."
 			  "  \\+ image #[0-9]+ \\([0-9]+x[0-9]+\\)"
 			  nil t))
 	       (if size1
-		   (if (string-equal size1 (setq size2 (match-string 1)))
+		   (if (string-equal size1 (setq size2 (mew-match-string 1)))
 		       (setq size1 size2)
 		     (setq should-unoptimize "--unoptimize"))
-		 (setq size1 (match-string 1)))))
-	   (erase-buffer)
+		 (setq size1 (mew-match-string 1)))))
+	   (mew-erase-buffer)
 	   (insert image)
 	   (when (or should-unoptimize should-be-non-interlaced))
 	   ;; Unoptimize anyway.
@@ -402,17 +406,33 @@ popup menu can be implemented."
 	   (unless (looking-at "GIF8[79]")
 	     ;; Unoptimization is failed. :-<
 	     ;; Attempt to extract the first frame.
-	     (erase-buffer)
-	     (insert data)
+	     (mew-erase-buffer)
+	     (insert image)
 	     (call-process-region (point-min) (point-max)
 				  prog
 				  t t nil "#0" "--no-interlace")))
 	 ;; buffer is fixed gif
-	 (setq image (mew-buffer-substring (point-min) (point-max))))
-	(kill-buffer tmpbuf))
+	 (setq image (mew-buffer-substring (point-min) (point-max)))))
       (make-glyph (vector 'gif :data image)))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; X Face
+;;;
+
+(defun mew-x-face-create ()
+  (mew-xemacs-make-xbm-glyph (point-min) (point-max)))
+
+(defun mew-x-face-display (xface)
+  (save-excursion
+    (goto-char (point-min))
+    (let ((regex2 (concat "^\\(" mew-from: "\\).*"))
+	  overlay)
+      (when (re-search-forward regex2 nil t)
+	(setq overlay (mew-overlay-make (match-end 1) (match-end 1)))
+	(set-extent-begin-glyph overlay xface)))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Profile
 ;;;
@@ -428,11 +448,24 @@ popup menu can be implemented."
       (let ((elp-results-buffer mew-buffer-debug))
 	(elp-results))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; SSL/SSH notification
+;;;
+
+(defvar mew-secure-format
+  (if window-system
+      (let ((data-directory mew-icon-directory))
+	(make-glyph [xpm :file "mew-lock.xpm"]))
+    " [Sec]"))
+
+(defvar mew-secure-format2 nil)
+
 (provide 'mew-xemacs)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2002 Mew developing team.
+;; Copyright (C) 1997-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
