@@ -2,7 +2,7 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Mar 23, 1994
-;; Revised: Feb 25, 2003
+;; Revised: Jul 24 2003
 
 ;;; Commentary:
 
@@ -19,7 +19,7 @@
 ;;; Mew version
 ;;;
 
-(defconst mew-version-number "3.2"
+(defconst mew-version-number "3.3"
   "Version number for this version of Mew.")
 (defconst mew-version (format "Mew version %s" mew-version-number)
   "Version string for this version of Mew.")
@@ -424,7 +424,8 @@ the lower window is not zero, switch to the buffer."
     (unless (zerop msg-height)
       (split-window nil sum-height)
       (other-window 1)
-      (switch-to-buffer msgbuf 'norecord))))
+      (switch-to-buffer msgbuf 'norecord)
+      (mew-set-buffer-cs mew-cs-m17n))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -607,7 +608,7 @@ It is typically called by kill-emacs-hook."
 	(save-excursion
 	  (set-buffer buf)
 	  (when (mew-summary-or-virtual-p)
-	    (mew-summary-kill-subprocess 'no-msg)
+	    (mew-summary-kill-subprocess)
 	    (setq obuf (mew-local-buffer-name folder))
 	    ;; xxx should kill pop's buffer...
 	    (mew-remove-buffer obuf))
